/**
 * ConfigurationXMLFile : Definition du fichier de configuration en XML
 * 
 * @author David Sagnol
 * @version 0.1, 16/06/03
 */

package graphics2;

import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
 
import org.xml.sax.SAXException;  
import org.xml.sax.SAXParseException;  
import org.w3c.dom.Document;
import java.io.*;

// Pour la sortie.
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult; 


public class ConfigurationXMLFile {
	
    public Document document;
    // nom du fichier xml qui va etre cree
    public String xmlFileName;
    
    public ConfigurationXMLFile(String fileName) throws java.io.IOException {
    	
    	this.xmlFileName = fileName; // initialisation peut etre modifie par la suite
    	
        try {
                // Parse the xml file only if filname !=NULL and it exists.
            if (fileName.length()!=0 && utils.FileTest.isExistent(fileName)) {
                DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
                DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
                document = docBuilder.parse (new File(fileName));
                
                    // normalize text representation
                document.getDocumentElement ().normalize ();
            }
            else // Create a new DOM document
            {
                DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
                DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
                document = docBuilder.newDocument();
                    // Create root node (Scenario)
                document.createComment("This file is used to store predefined Scenarios");
                org.w3c.dom.Element root = document.createElement("Scenario"); 
                document.appendChild(root); // Add it as a child of the document.
                org.w3c.dom.Element flows = document.createElement("Flows"); 
                root.appendChild(flows);
//                root.appendChild( document.createTextNode("Some") );
//                root.appendChild( document.createTextNode(" ")    );
//                root.appendChild( document.createTextNode("text") );
                
            }
        }
        catch (SAXParseException err) {
            System.out.println ("** Parsing error" + ", line " + err.getLineNumber () + ", uri " + err.getSystemId ());
            System.out.println(" " + err.getMessage ());
        }
        catch (SAXException e) {
            Exception x = e.getException ();
            ((x == null) ? e : x).printStackTrace ();
        }
        catch (Throwable t) {
            t.printStackTrace ();
        }
    }


   	/**
	 * Write the DOM Tree in an xml file.
	 * @param output file name of the XML file written.
	 */
    public void writeFile(String outputfile) {
    	
    	// Met a jour le nom du fichier Xml
    	if (! xmlFileName.equals(outputfile) ) {
    		setXmlFileName(outputfile); 
    	}
    	
        try {
                // Use a Transformer for output
            TransformerFactory tFactory = TransformerFactory.newInstance();
            Transformer transformer = tFactory.newTransformer();
            
            DOMSource source = new DOMSource(document);
            File xmloutputfile = new File(outputfile);
            StreamResult result = new StreamResult(xmloutputfile);
            transformer.transform(source, result);

        } 
        catch (TransformerConfigurationException tce) {
                // Error generated by the parser
            System.out.println ("\n** Transformer Factory error");
            System.out.println("   " + tce.getMessage() );
            
                // Use the contained exception, if any
            Throwable x = tce;
            if (tce.getException() != null)
                x = tce.getException();
            x.printStackTrace();
            
        } 
        catch (TransformerException te) {
                // Error generated by the parser
            System.out.println ("\n** Transformation error");
            System.out.println("   " + te.getMessage() );
            
                // Use the contained exception, if any
            Throwable x = te;
            if (te.getException() != null)
                x = te.getException();
            x.printStackTrace();
        }
    }


    public int numberOfScenario() { 
        return document.getElementsByTagName("Flow").getLength();
    }
    
    	/**
	 * Method addScenario. Add a Scenario to the XML Tree. We takeGet parameters from the GUI and put it in ObjectOutputStream.
	 * @param ClientInfo Frame of the GUI with all the parameters describing the scenario.
	 */
    public void addScenario(graphics2.info.ClientInfo client) {
            // On recupere le nom du modele..
        String modelname = client.getModelTestName();
        System.out.println("==> "+modelname);
            // ainsi que les parametres des modeles...
        String [] modparam = client.getModelParam();

            // Un de ces models... MMPPN, OnOff, MGInfComplex, MMPP2, MGInfComplexAggregatedFlow, OnOffHeterogeneousAggregatedFlow
        if (modelname.equals("MMPPN"))
            addMMPPNScenario(client, modparam);
        else if (modelname.equals("MMPP2"))
            addMMPP2Scenario(client,modparam);
        else if (modelname.equals("OnOff"))
            addOnOffScenario(client, modparam);
        else if (modelname.equals("MGInfComplex"))
            addMGInfComplexScenario(client, modparam);
        else if (modelname.equals("MGInf"))
            addMGInfScenario(client, modparam);
        else if (modelname.equals("MGInfComplexAggregatedFlow"))
            addMGInfComplexAggregatedFlowScenario(client, modparam);
        else if (modelname.equals("OnOffHeterogeneousAggregatedFlow"))
            addOnOffHeterogeneousAggregatedFlowScenario(client, modparam);
        else
            System.out.println("Unknown Model...");

            // On fait la meme chose pour le codec...
    }

    public org.w3c.dom.Element createElement (String name) {
        return document.createElement(name);
    }

    public void addElement(org.w3c.dom.Element newElt) {
        org.w3c.dom.Node flows = getFlows();
        flows.appendChild(newElt);
    }

    public org.w3c.dom.Element createNewXMLElement (String name) {
        org.w3c.dom.Element elem = document.createElement("Flow");
        int number= numberOfScenario()+1;
        elem.setAttribute ("number", String.valueOf(number));
        elem.setAttribute ("name",name+"_"+number);
        return elem;
    }

    private org.w3c.dom.Node getFlows() {
            // Find the Flows node...
        org.w3c.dom.NodeList list = document.getElementsByTagName("Flows");
        if (list.getLength() != 1) {
            System.out.println("Error, wrong number of Flows tag in your XML file !");
        }
        return list.item(0);
    }

	public Document getDocument() { return document; }

    protected void addMMPPNScenario(graphics2.info.ClientInfo client, String[] params) {
        System.out.println("addMMPPNScenario");
        for (int i=0; i <params.length; i++)
            System.out.println(params[i]);
    }

        /**
 		 *	Create a new Flow Node in XML Tree.
         * XML sample:<BR>
         *
	     *  <Flow number="4" name="scenario_onoff_1" srcnumber="1">
         *	        <model name="OnOff">
		 *		<sstate value="0" />
		 *		<distribution1 name="exponential" param1="0.5" />
		 *		<distribution2 name="pareto" param1="0.5" param2="0.1" />
		 *	</model>
		 *	</Flow>
         */
    protected void addOnOffScenario(graphics2.info.ClientInfo client, String[] params) {
        System.out.println("addOnOffScenario "+params.length);

            // Add a new flow
        String scnname = "scenario_onoff"+ numberOfScenario(); // TODO: Definition of a name is missing 
        org.w3c.dom.Element newflow = createNewXMLElement(scnname);

            // Add the flow in the tree.
        org.w3c.dom.Node flows = getFlows();
        flows.appendChild(newflow);

            // Now, add two childs to this node, the model and the codec...
        org.w3c.dom.Element modelchild = document.createElement("model");
        modelchild.setAttribute("name", "OnOff");
            // Add state value node
        org.w3c.dom.Element statechild = document.createElement("sstate");
        statechild.setAttribute("value", params[0]);
        modelchild.appendChild(statechild);
            // add talkspurt length
        org.w3c.dom.Element dis1child = document.createElement("distribution1");
        int parpos=0;
        if (params[1].equals("Poisson") || params[1].equals("Exponential") || params[1].equals("Constant") || params[1].equals("Geometric")) {
            dis1child.setAttribute("name", params[1]);
            dis1child.setAttribute("param1", params[3+parpos]);
            parpos++;
        }
        else {
            dis1child.setAttribute("name", params[1]);
            dis1child.setAttribute("param1", params[3+parpos]);
            dis1child.setAttribute("param2", params[4+parpos]);
            parpos++;
            parpos++;
        }
        modelchild.appendChild(dis1child);

        org.w3c.dom.Element dis2child = document.createElement("distribution2");
        if (params[2].equals("Poisson") || params[2].equals("Exponential") || params[2].equals("Constant") || params[2].equals("Geometric")) {
            dis2child.setAttribute("name", params[2]);
            dis2child.setAttribute("param1", params[3+parpos]);
        }
        else {
            dis2child.setAttribute("name", params[2]);
            dis2child.setAttribute("param1", params[3+parpos]);
            dis2child.setAttribute("param2", params[4+parpos]);
        }
        modelchild.appendChild(dis2child);
        newflow.appendChild(modelchild);

            // Now, read the codec infos...
        for (int i=0; i <params.length; i++) {
            System.out.println(params[i]);
        }
    }

    protected void addMGInfComplexScenario(graphics2.info.ClientInfo client, String[] params) {
        System.out.println("addMGInfComplexScenario");
        for (int i=0; i <params.length; i++) {
            System.out.println(params[i]);
        }

    }
    
    protected void addMMPP2Scenario(graphics2.info.ClientInfo client, String[] params) {
            // Add a new flow
        String scnname = "scenario_mmpp2_"+ numberOfScenario(); // TODO: Defination of a name is missing 
        org.w3c.dom.Element newflow = createNewXMLElement(scnname);

            // Add the flow in the tree.
        org.w3c.dom.Node flows = getFlows();
        flows.appendChild(newflow);

            // Now, add two childs to this node, the model and the codec...
        org.w3c.dom.Element modelchild = document.createElement("model");
        modelchild.setAttribute("name", "MMPP2");
            // Add state value node
        org.w3c.dom.Element statechild = document.createElement("sstate");
        statechild.setAttribute("value", params[0]);
        modelchild.appendChild(statechild);
            // Add alpha value node
        org.w3c.dom.Element alphachild = document.createElement("alpha");
        statechild.setAttribute("value", params[1]);
        modelchild.appendChild(alphachild);
            // Add beta value node
        org.w3c.dom.Element betachild = document.createElement("beta");
        statechild.setAttribute("value", params[2]);
        modelchild.appendChild(betachild);
            // Add lambda1 value node
        org.w3c.dom.Element lambda1child = document.createElement("lambda1");
        statechild.setAttribute("value", params[3]);
        modelchild.appendChild(lambda1child);
            // Add lambda2 value node
        org.w3c.dom.Element lambda2child = document.createElement("lambda2");
        statechild.setAttribute("value", params[4]);
        modelchild.appendChild(lambda2child);

        newflow.appendChild(modelchild);

        for (int i=0; i <params.length; i++)
            System.out.println(params[i]);
   }
    
    protected void addMGInfScenario(graphics2.info.ClientInfo client, String[] params) {
        for (int i=0; i <params.length; i++)
            System.out.println(params[i]);
    }
    
    protected void addMGInfComplexAggregatedFlowScenario(graphics2.info.ClientInfo client, String[] params) {
        
        for (int i=0; i <params.length; i++)
            System.out.println(params[i]);
    }
    
    protected void addOnOffHeterogeneousAggregatedFlowScenario(graphics2.info.ClientInfo client, String[] params) {

        for (int i=0; i <params.length; i++)
            System.out.println(params[i]);
    }

    public void addAgregatedScenario() {

    }

    public org.w3c.dom.Element findScenario(String scname) {
		org.w3c.dom.Element elem;
		org.w3c.dom.NodeList lnodes = document.getElementsByTagName("Flow");
		
		for(int i=0; i <lnodes.getLength(); i++) {
			if ( ((org.w3c.dom.Element)lnodes.item(i)).getAttribute("name").equals(scname))
				return (org.w3c.dom.Element) lnodes.item(i);
		}
		return createElement("UnknownElement");
    }

	public void printScenario (org.w3c.dom.Element elem) {
		System.out.println("Name: "+ elem.getAttribute("name"));		
		System.out.println("Name Model: "+((org.w3c.dom.Element )elem.getChildNodes().item(0)).getAttribute("name"));
		//System.out.println("Name Codec: "+((org.w3c.dom.Element )elem.getChildNodes().item(1)).getAttribute("name"));
	}
	
    public void deleteScenario() {

    }





    /**
	 * To test this class
	 */
	public static void main (String[] args) {
        if (args.length < 1) {
            System.out.println("Usage: ConfigurationXMLFile <xmlfile>");
            System.exit(-1);
        }
        try {
            ConfigurationXMLFile cf= new ConfigurationXMLFile(args[0]);
            cf.writeFile("test.xml");
            System.out.println(" Number of scenario : "+cf.numberOfScenario());
        }
        catch(java.io.IOException ioe) {
            System.out.println(ioe);
        }
    }
	/**
	 * @return String
	 */
	public String getXmlFileName() { return xmlFileName; }

	/**
	 * @param string
	 * void
	 */
	public void setXmlFileName(String string) { xmlFileName = string; }

}
