package graphics2.listener.generator;

/**
 * MGInfGeneratorConstructor : Defined to create an MGInf Model and a MGInf Generator
 * 
 * @author mladoue
 * @version 0.1, 17/01/03
 */

import java.io.IOException;

import graphics2.*;
import multimedia.generation.*;
import multimedia.model.*;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;
import statistic.distribution.*;
import statistic.function.*;

/**
 * MGInfGeneratorConstructor : Defined to create an MGInf Model and a MGInf Generator
 *
 * @author mladoue
 * @version 0.1, 17/01/03
 */

public class MGInfGeneratorConstructor extends SimpleModelGeneratorConstructor {
    // En ce qui concerne le modele
    protected GenericFunction autoCorr;
    protected double lambda;
    protected GenericContinuousDistribution distributionTest;
    protected GenericContinuousDistribution distributionSlot;
    protected GenericContinuousDistribution distributionPacketSize;

    //En ce qui concerne le generateur
    protected String fileName;
    protected short unit;

    /**
     * @see graphics.listener.ModelGeneratorConstructor#ModelGeneratorConstructor(GestionTest)
     */
    public MGInfGeneratorConstructor(GestionTest gt, Element nlFlow) {
        super(gt, nlFlow);

// TODO: Est-ce que les 3 lignes suivantes ont raison d'etre ????
        fileName = sfp.createScenarioFileName(cp);
        unit = 0; // fixer pour l'instant en bytes uniquement
    }

	protected GenericContinuousDistribution getDistribution(Element distrib) {
		
		GenericContinuousDistribution distribution=null;
		String name = distrib.getAttribute("name");
		try {
		  distribution = (GenericContinuousDistribution)Class.forName("statistic.distribution." + name+"Distribution").newInstance();
		}
		catch (IllegalAccessException iae) {
			   System.out.println(iae);
		}
	   catch(InstantiationException ie) { 
		   System.err.println("MGInfGeneratorConstructor.getDistribution : the initialization provoked by this methode fails => Distribution Name : " + name + " "  + ie);
		}
		catch (ClassNotFoundException cnfe) {
		   System.err.println("MGInfGeneratorConstructor.getDistribution : the class cannot be located => Distribution Name : " + name + " " + cnfe);
		}
			// Set the distribution parameters.
		double[] params;
		if (distrib.getAttribute("param2").equals("")) {
			params = new double[1];
		}
		else {
			params = new double[2];
			params[1]=Double.valueOf(distrib.getAttribute("param2")).doubleValue();
		}
		params[0]=Double.valueOf(distrib.getAttribute("param1")).doubleValue();
		distribution.setParams( params);
		return distribution;
	}

	protected GenericFunction getFunction(Element elem) {
		GenericFunction function=null;
		String namefunc = elem.getAttribute("name");
		double param1 =  Double.valueOf(elem.getAttribute("param1")).doubleValue();
		try {
		   function = (GenericFunction)Class.forName("statistic.function." + namefunc+"Function").newInstance();
		   function.setParam( param1);
		}
		catch (IllegalAccessException iae) {
			System.out.println(iae);
		}
		catch(InstantiationException ie) { 
			System.err.println("MGInfGeneratorConstructor.getFunction : the initialization provoked by this methode fails => Function Name : " + namefunc + " "  + ie);
		 }
		 catch (ClassNotFoundException cnfe) {
			System.err.println("MGInfGeneratorConstructor.getFunction : The class cannot be located => Function Name : " + namefunc + " " + cnfe);
		 }   
		return function;
	}
	
	
	protected void setModelParameters(Element model) {
		//System.out.println("Model node: " + model.toString()); //DEBUG
		NodeList nl=model.getChildNodes();	
		for (int i=0; i < nl.getLength(); i++) {
			String name =  nl.item(i).getNodeName();

			if (name.equals("function")) {
				autoCorr = getFunction((org.w3c.dom.Element) nl.item(i));
			}
			else if(name.equals("lambda1")) {
				lambda = Double.valueOf(((org.w3c.dom.Element) nl.item(i)).getAttribute("value")).doubleValue();
			}
			else if(name.equals("distribution1")) {
				distributionTest = getDistribution((org.w3c.dom.Element) nl.item(i));
			}
			else if(name.equals("distribution2")) {
				distributionSlot = getDistribution((org.w3c.dom.Element) nl.item(i));
			}
			else if(name.equals("distribution3")) {
				distributionPacketSize = getDistribution((org.w3c.dom.Element) nl.item(i));
			}
		}
	}

    /**
     * Method constructModelGenerator :
     *                 - cree le model
     *                 - cree le model generator
     * @throws IOException
     */
    public void constructModelGenerator() throws IOException {
//    	if (srcNumber==1) {
			MGInfModel mginfModel = new MGInfModel(autoCorr, lambda, distributionTest, distributionSlot, distributionPacketSize);
			MGInfGeneration mginfGenerator = new MGInfGeneration(mginfModel, duration, fileName, codec, srcNumber, unit, true);
        	genericAggModel = mginfModel;
        	genericGen = mginfGenerator;
//    	}
//   	else {// Aggregated model.
//			MGInfModel[] mginfModel = new  MGInfModel[srcNumber];
//			int[] weights = new int[srcNumber];
//			for(int i=0; i < srcNumber; i++) {
//				mginfModel[i]= new MGInfModel(autoCorr, lambda, distributionTest, distributionSlot, distributionPacketSize);
//			}
//    	}
    }
}
