package graphics2.panel.client;

/**
 * CodecPanel : this defines the panel where you specify codecs and their parameters
 * 
 * @author mladoue
 * @version 0.1, 09/01/03 
 */

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import utils.*;
import utils.definitions.MyExceptionDefinitions;
import multimedia.codec.*;
import graphics2.*;

public class CodecPanel extends JPanel implements ActionListener,AudioCodecExample,VideoCodecExample {

 protected Hashtable listCodec;
 protected String modelName;
 protected CodecAV codec = null;
  protected GestionTest gt;
 
 // to define if it's a video codec or a n audio codec
 protected int codecType;
 protected final static int AUDIO = 0;
 protected final static int VIDEO =1;
 
  protected JLabel[] paramName= new JLabel[2];
  protected JTextField[] paramValue = new JTextField[2];
  protected JLabel[] paramType = new JLabel[2];
  protected JComboBox codecName = new JComboBox();
 protected InterneModelPanel imp;
   

public CodecPanel(  InterneModelPanel imp ) {

	super();
        this.imp=imp;
	gt = imp.getModelPanel().getClientPanel().getGestionTest();
	modelName = SpecificObjectCreation.createModelName(imp.getModelPanel().getModelName());
	
	if (modelName == "MGInf" || modelName == "MGInfComplex" || modelName == "MGInfComplexAggregatedFlow" ) {
		codecType =	1;
		this.listCodec = imp.getListVideoCodec();
	}
	else {
		codecType =	0;
		this.listCodec = imp.getListAudioCodec();
	}
	
	createComboBox();
	
	this.setLayout( new BoxLayout(this,BoxLayout.Y_AXIS) );
	
	JPanel cPanel = new JPanel();
	cPanel.setLayout( new BoxLayout(cPanel,BoxLayout.Y_AXIS) );
		
	JPanel comboPanel = new JPanel();
	comboPanel.setLayout( new BoxLayout(comboPanel,BoxLayout.X_AXIS) );
	
	comboPanel.add(Box.createRigidArea(new Dimension(10,0)));
	comboPanel.add(codecName);
	codecName.setMaximumSize(new Dimension(codecName.getPreferredSize().width, 
																		  codecName.getPreferredSize().height));
	comboPanel.add( Box.createHorizontalGlue());
	
	cPanel.add(comboPanel);
	cPanel.add(Box.createRigidArea(new Dimension(0,10)));

    // Creation des Labels contenant le nom des parametres
	JPanel[] panel = new JPanel[2];
	
    for (int i=0;i<2;i++) {
    	panel[i] = new JPanel();
    	panel[i].setLayout(new BoxLayout(panel[i],BoxLayout.X_AXIS));

    	panel[i].add(Box.createRigidArea(new Dimension(40,0)));	
		panel[i].add(paramName[i]= new JLabel());
		panel[i].add(Box.createRigidArea(new Dimension(Math.abs((i-1)*43) + 5,0)));
	    panel[i].add(paramValue[i] = new JTextField(7));
		paramValue[i].setMaximumSize(new Dimension(paramValue[i].getPreferredSize().width, 
																				paramValue[i].getPreferredSize().height));
    	panel[i].add(Box.createRigidArea(new Dimension(10,0)));
	    panel[i].add(paramType[i]= new JLabel());
	   
	    if(i == 0) {
	    panel[i].add(Box.createRigidArea(new Dimension(50,0)));
		panel[i].add(MessageManager.createHelpMessageDialogButton("Help window...to do"));
		panel[i].add(Box.createHorizontalGlue());//RigidArea(new Dimension(80,0))); //280
	    }
	    else {
	    	panel[i].add(Box.createHorizontalGlue());//createRigidArea(new Dimension(200,0))); //400
	    }
    }
  
    	cPanel.add(panel[0]);
		cPanel.add(Box.createRigidArea(new Dimension(0,10)));
		cPanel.add(panel[1]);
		cPanel.add(Box.createRigidArea(new Dimension(0,10)));
		    
    add(cPanel);
    
  	updateCodec(getCodecName());
   	updateParamName();
   	
    // Set the Border
    setBorder( new TitledBorder("Codec Definition") );
} // end constructor


public void createComboBox() {
	// Creation de la liste des lois disponibles
	   // pour acceder aux elements de la HashTable 
	   Enumeration valCodecName = listCodec.elements();

	   while (valCodecName.hasMoreElements()) {
		 codecName.addItem( valCodecName.nextElement() );
	   }

	   try {
		   codecName.setEditable(false);  
		   codecName.setSelectedIndex(0); // 0 is first item selected and -1 it's for no selection
		   codecName.setMaximumRowCount( 4 );
		   codecName.addActionListener(this);
	   }
	   catch (IllegalArgumentException iae) { iae.printStackTrace(); }
}

  /**
   * Restores les parametres a partir de l'object CodecAV
   * @param GenericDistribution
   */
  public void setCodec(CodecAV codec) {

    this.codec = codec;
    codecName.setSelectedItem( listCodec.get(codec.getClass().getName()));
    updateParamName();
  }

  /**
   * Returns selected codec
   * @return CodecAV
   */
  public CodecAV getCodec() {  return  codec;  }

/**
 * Method freeze. to do a configuration file
 * @param b boolean
 */
  public void freezeInfo ( boolean b ) {
  	codecName.setEnabled(!b);
    for(int i=0;i<2;i++) {
      paramValue[i].setEnabled(!b);
    }
  }
  
  /**
   *	Defines to be used when you define a MGInfComplexAggregatedFlowModel or a OnOffHeterogeneousAggregatedFlow
   *	Actually these models don't need current codecPanel defines in the InterneModelPanel.
   *	They are their own codecPanel 
   *   @see java.awt.Component#setEnabled(boolean)
 	*/
public void setEnabled( boolean b ) {
	codecName.setEnabled(b);
	for(int i=0;i<2;i++) {
	  paramValue[i].setEditable(b);
	}
  }
  
  public void reset() {
  	codecName.setEnabled(true);
  	codecName.setSelectedIndex(0);
	for(int i=0;i<2;i++) {
		paramValue[i].setText("");
		paramValue[i].setEnabled(true);
	}
  }

  // change le nom du codec choisit
  public void actionPerformed(ActionEvent e) {
   	updateCodec(getCodecName());
    updateParamName();  
  }

  // met a jour le CodecPanel ( essentiellement modifie le nom des parametres )
   protected void updateCodec(String classCodecName) {
     try {
     	if(codecType == 0 ) {
     		if( classCodecName == "Audio") {
     			codec = (AudioCodec)Class.forName("multimedia.codec." + classCodecName + "Codec").newInstance();
     		}
     		else {
     			codec = SpecificObjectCreation.createCodecExample(classCodecName);
     		//codec = (AudioCodec)(Class.forName("multimedia.codec.AudioCodecExample." +classCodecName).newInstance());
     		}
     	}
     	else {
     		if( classCodecName == "Video" ) {
     			codec = (VideoCodec)Class.forName("multimedia.codec." + classCodecName + "Codec").newInstance();
     		}
     		else {
     			codec =  SpecificObjectCreation.createCodecExample(classCodecName);
     		//codec = multimedia.codec.VideoCodecExample.MPEG4;
     		}
     	}
     }
	catch (IllegalAccessException iae) {  System.out.println(iae);	}
	
    catch(InstantiationException ie) { 
    	System.out.println("CodecPanel.updateCodec : the initialization provoked by this methode fails => Codec Name : " + classCodecName + " "  + ie);
     }
    catch (ClassNotFoundException cnfe) { 
     	System.out.println("CodecPanel.updateCodec : The class cannot be located => Codec Name : " + classCodecName + " "  + cnfe);
     }
  }



/**
* Methode updateParam : update codec parameters according value in the Panel.
*/
	public void updateParam() {
		// Only in these two cases, otherwise they are builtin.
		if ( codec.getName().equals("Audio") ) {
			((AudioCodec)codec).setSize(Integer.parseInt(paramValue[0].getText()));
			((AudioCodec)codec).setTpsIP(Double.parseDouble(paramValue[1].getText()));
		}
		else if(codec.getName().equals("Video") ) {
			((VideoCodec)codec).setFramesNumber(Integer.parseInt(paramValue[0].getText()));
				// Check compression type.
				// TODO Attention, ici, on suppose que l'utilisateur rentre correctement la chaine
				// name,int,int
			int M=-1; int N=-1;
			String compression = paramValue[1].getText();
			int pos = compression.lastIndexOf(",");
			String Ns = compression.substring( pos+1);
			compression = compression.substring(0, pos);
			pos = compression.lastIndexOf(",");
			String Ms = compression.substring( pos+1);
			// The reste should be the name...
			String name = compression.substring(0, pos);
			
			M = Integer.parseInt(Ms);
			N = Integer.parseInt(Ns);
			if (M!=-1 && N!=-1 && name.length() > 0) {
				((VideoCodec)codec).setCompression(new CompressionType(name,  new GOP(M,N)));
			}
		}
	}

/**
 * Method updateParamName.
 */
  public void updateParamName() {
	
  	 for (int i = 0;i<2;i++) {
	  	  paramName[i].setText( codec.getParamName(i) + " : ");  
	  	  
	   	if ( codec.getName().equals("Audio") ||  codec.getName().equals("Video") ) {	
	   		paramValue[i].setText("");
  			paramValue[i].setEditable(true);	
		}
		else {
			paramValue[i].setEditable(false);	
		}  	  
 	}
 
 if (codecType ==  0 ) {

 	 if( !codec.getName().equals("Audio") ) {  
  	 	paramValue[0].setText("" + Integer.toString( ((AudioCodec)codec).getSize() ) );
  	  	paramValue[1].setText("" + Double.toString( ((AudioCodec)codec).getTpsIP() ) );
 	 }
      paramType[0].setText("bytes");
      paramType[1].setText("s");   
  }
      
	else {
	 	if( !codec.getName().equals("Video") ) {
	 	paramValue[0].setText("" + Integer.toString( ((VideoCodec)codec).getFramesNumber() ) );
		paramValue[1].setText("" +  ((VideoCodec)codec).getCompression().toString() );
	 	}
		paramType[0].setText("frames/s");
		paramType[1].setText("");
 	}
 
  }
 
 /**
 * Method getCodecName.
 * @return String on the form : audio
 */
 public String getCodecName() { return codecName.getSelectedItem().toString(); }
 
 public void setCodecName(String unCodec) { codecName.setSelectedItem(unCodec); }

public String[] getCodecParam() {
	String[] ans = new String[2];
	for(int i=0;i<2;i++) {
		if (!paramValue[i].getText().equals(new String(""))) {
			ans[i] = paramValue[i].getText();
		}
		else {
			JOptionPane.showMessageDialog(null, "Alert: there is no value specify for the parameter: " + ((paramName[i].getText()).split(" ",2)[0]) , "Error parameter", JOptionPane.ERROR_MESSAGE); 
		}
	}
	return ans;
}

        /**
         * To create an XML Node representing the codec.
         */
    org.w3c.dom.Element buildNewXmlNode() {
		    // data values in the Panel should be ok
            // Access Configuration Tree.
        graphics2.ConfigurationXMLFile conftree = imp.getModelPanel().getClientPanel().getListScenario();
        org.w3c.dom.Element xmlcodec = conftree.createElement("codec");
        if (codecType == 0)
            xmlcodec.setAttribute("type", "Audio");
        else
            xmlcodec.setAttribute("type", "Video");
        xmlcodec.setAttribute("name", getCodecName());
        if (codecType==0) {
        	xmlcodec.setAttribute("iptime", paramValue[1].getText());
        	xmlcodec.setAttribute("psize", paramValue[0].getText());
        }
        else {
			xmlcodec.setAttribute("compression", paramValue[1].getText());
			xmlcodec.setAttribute("frames", paramValue[0].getText());
        }

        return xmlcodec;
    }

	public void updateWithXmlNode(org.w3c.dom.Element elt) {
			if (elt.getAttribute( "type").equals("Audio"))
				codecType=0;
			else
				codecType=1;
			codecName.setSelectedItem( elt.getAttribute("name"));
			if (codecType==0) {
				paramValue[0].setText( ""+elt.getAttribute( "psize"));
				paramValue[1].setText( ""+elt.getAttribute( "iptime"));
			}
			else{
				paramValue[0].setText( ""+elt.getAttribute( "frames"));
				paramValue[1].setText( ""+elt.getAttribute( "compression"));			
			}
	}
	
/**
 * To set codec parameter
 * @param params
 * @throws IllegalArgumentException
 */
public void setCodecParams(String[] params) throws IllegalArgumentException {
	if (params.length == 2) { 
		for(int i=0;i<2;i++) {
			paramValue[i].setText(params[i]);
		}
	}
	else { throw MyExceptionDefinitions.PARAMETERS_TABLE_SIZE;	}
}


 
}
