package graphics2.panel.client;

import statistic.function.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * FunctionPanel
 *
 * @author mladoue
 * @version 0.1, 22/01/03
 */
public class FunctionPanel extends JPanel implements ActionListener {
    protected GenericFunction function;
    protected JComboBox functionName = new JComboBox();
    protected JLabel paramName;
    protected JTextField paramValue;
    protected Hashtable listFunction;
    protected ParamListener paramListener;

    public FunctionPanel(String name, Hashtable listFunction) {
        super();
        this.listFunction = listFunction;

        createComboBox();

        this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        updateFunction(getFunctionName());

		JPanel panel0 = new JPanel();
		panel0.setLayout(new BoxLayout(panel0, BoxLayout.X_AXIS));
		panel0.add(functionName);
		panel0.add(Box.createHorizontalGlue());
		
        JPanel panel1 = new JPanel();
        panel1.setLayout(new BoxLayout(panel1, BoxLayout.X_AXIS));
        panel1.add(Box.createRigidArea(new Dimension(20, 0)));
        panel1.add(paramName = new JLabel("b :"));
        panel1.add(Box.createRigidArea(new Dimension(12, 0)));
        panel1.add(paramValue = new JTextField(7));
		paramValue.setMaximumSize(new Dimension(paramValue.getPreferredSize().width, 
																			  paramValue.getPreferredSize().height));

        paramValue.addActionListener(paramListener);
        panel1.add(Box.createHorizontalGlue());

        String startB = paramValue.getText();

        if ((!startB.equals("")) && (Double.parseDouble(startB) > 0)) {
            function.setParam(Double.parseDouble(startB));
        }

		add(panel0);
		add(Box.createRigidArea(new Dimension(0, 15)));
		add(panel1);
		        // Set the Border
        setBorder(new TitledBorder(name));
    }

    public void createComboBox() {
        // Creation de la liste des fonctions disponibles
        // pour acceder aux elements de la HashTable 
        Enumeration valDistributionName = listFunction.elements();

        while (valDistributionName.hasMoreElements()) {
            functionName.addItem(valDistributionName.nextElement());
        }

        try {
            functionName.setEditable(false);
            functionName.setSelectedIndex(0); // 0 is first item selected and -1 it's for no selection
            functionName.setMaximumRowCount(3);
			functionName.setMaximumSize(new Dimension(functionName.getPreferredSize().width, 
																					functionName.getPreferredSize().height));

            functionName.addActionListener(this); // pour mettre a jour l'equation quand on change de fonction
        } catch (IllegalArgumentException iae) {
            iae.printStackTrace();
        }
    }

    /**
     * Method getFunction : to return in MGInfGeneratorConstructor the Function definition
     * @return a function definition in GenericFunction form
     */
    public GenericFunction getFunction() {
        String startB = paramValue.getText();

        if ((!startB.equals("")) && (Double.parseDouble(startB) > 0)) {
            function.setParam(Double.parseDouble(startB));

            return function;
        } else {
            throw new Error("Error on getFunction");
        }
    }

    public void setFunction(GenericFunction f) {
        function = f;
    }

    public String getFunctionName() {
        return functionName.getSelectedItem().toString();
    }

    public void setFunctionName(String funcName) {
        functionName.setSelectedItem(funcName);
    }

    public void setParam(double value) throws IllegalArgumentException {
        paramValue.setText("" + value);
    }

	public boolean isAValidPanel() {
		if (!paramValue.getText().equals(new String(""))) {
			 try {
				 double ans = Double.parseDouble(paramValue.getText());
			 } catch (NumberFormatException nfe) {
				 //nfe.printStackTrace();
				JOptionPane.showMessageDialog(null,
								 "Wrong value for : " +
								 ((paramName.getText()).split(" ", 2)[0])+" in function" , "Error parameter",
								 JOptionPane.ERROR_MESSAGE);
				return false;
			 }
		 } else {
			 JOptionPane.showMessageDialog(null,
				 "Alert: there is no value specify for : " +
				 ((paramName.getText()).split(" ", 2)[0]), "Error parameter",
				 JOptionPane.ERROR_MESSAGE);
				 return false;
		 }
		return true;
	}
	
    public double getParam() {
        double ans = 0d;

        if (!paramValue.getText().equals(new String(""))) {
            try {
                ans = Double.parseDouble(paramValue.getText());
            } catch (NumberFormatException nfe) {
                nfe.printStackTrace();
                System.out.println("Parameter value: " + paramValue.getText() +
                    ".");
            }
        } else {
            JOptionPane.showMessageDialog(null,
                "Alert: there is no value specify for : " +
                ((paramName.getText()).split(" ", 2)[0]), "Error parameter",
                JOptionPane.ERROR_MESSAGE);
        }

        return ans;
    }

    // met a jour le DistributionPanel ( essentiellement modifie le nom des parametres + equation en fonction du nom de la loi
    protected void updateFunction(String classFunctionName) {
        try {
            function = (GenericFunction) Class.forName("statistic.function." +
                    classFunctionName + "Function").newInstance();
            updateEquation();
        }
         catch (IllegalAccessException iae) {
            System.out.println(iae);
        } catch (InstantiationException ie) {
            System.out.println(
                "FunctionPanel.updateFunction : the initialization provoked by this methode fails => Function Name : " +
                classFunctionName + " " + ie);
        } catch (ClassNotFoundException cnfe) {
            System.out.println(
                "FunctionPanel.updateFunction : The class cannot be located => Function Name : " +
                classFunctionName + " " + cnfe);
        }
    }

    protected void updateEquation() {
        functionName.setToolTipText("Equation: " + function.getEquation());
    }

    public void freezeInfo(boolean b) {
        functionName.setEnabled(!b);
        paramValue.setEditable(!b);
    }

    public void reset() {
        functionName.setSelectedIndex(0);
        updateEquation();
        paramValue.setText("");
    }

    //change le nom du parametre de la loi choisie
    public void actionPerformed(ActionEvent e) {
        updateFunction(getFunctionName());
        updateEquation();
    }

    // Classes Internes
    class ParamListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            function.setParam(Double.parseDouble(paramValue.getText()));
        }
    }
}
