package graphics2.panel.client;

/**
 * InterneModelPanel : this is a JPanel in the ModelPanel which is composed of a:
 * 					         GenericModelPanel  and a CodecPanel
 * 
 * @author mladoue
 * @version 0.1, 10/01/03 
 */

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

//import graphics2.GestionTest;
import graphics2.panel.client.model.*;
import utils.MessageManager;
import utils.SpecificObjectCreation;
import multimedia.*;
import statistic.function.*;


public class InterneModelPanel extends JPanel implements VideoExample { 

protected GenericModelPanel mgp;
protected CodecPanel codecPanel;
public String modelName;

protected ModelActionsPanel modelActionsPanel = new ModelActionsPanel();
   
protected ModelPanel modelPanel;
protected Hashtable listDistribution;
protected Hashtable listContinuousDistribution;
protected Hashtable listAudioCodec;
protected Hashtable listVideoCodec;
protected Hashtable listFunction;

protected VideoDefinition vd;


/**
 * Utilise dans tous les cas excepte si on veut du MGInfComplex
 * @param modelPanel
 */
public InterneModelPanel( ModelPanel modelPanel ) {
	
	super();
	this.modelPanel = modelPanel;
	this.setLayout(new BoxLayout( this, BoxLayout.Y_AXIS) );
	
	makeLists();
	
	JPanel interneModelPanel = new JPanel();
	interneModelPanel.setLayout( new BoxLayout( interneModelPanel,BoxLayout.Y_AXIS) );
	

	modelName = SpecificObjectCreation.createModelName( modelPanel.getModelName() ); 

   	mgp = SpecificObjectCreation.createModelGenericPanel( modelName, this);
   	
	interneModelPanel.add(mgp);

   	
   	if( !modelName.equals("MGInfComplexAggregatedFlow") && !modelName.equals("OnOffHeterogeneousAggregatedFlow")) {
   		codecPanel = new CodecPanel(this);
		interneModelPanel.add(codecPanel);
   	}
   	else {
		codecPanel = new CodecPanel(this); 	// existe mais n'est pas ajoute a l'interface graphique
   	}
  	

	interneModelPanel.add(Box.createRigidArea(new Dimension(0,5)));
	interneModelPanel.add(modelActionsPanel);
    
    add(interneModelPanel);
	
}

/**
 * Utilise dans ce cas car on a specifie un VideoExample
 * @param modelPanel
 * @param mginfModelName  nom recupere via la 2eme JComboBox du ModelPanel sur les films
 */
public InterneModelPanel( ModelPanel modelPanel, String mginfModelName ) {

	this.modelPanel = modelPanel;
	this.modelName = mginfModelName;
	vd = SpecificObjectCreation.createVideoExample(modelName);
	
	this.setLayout( new BoxLayout( this,BoxLayout.Y_AXIS) );
	
	makeLists();
	
	JPanel interneModelPanel = new JPanel();
	 interneModelPanel.setLayout( new BoxLayout( interneModelPanel,BoxLayout.Y_AXIS) );
	
	
	// on met a jour les parametres de l'interface en fonction de la valeur de mginfModelName: ici pour la duree
	modelPanel.clientPanel.setDuration( (short)(vd.getTpsSimul()));
	modelPanel.clientPanel.updateResultMinutes();
	
	if ( mginfModelName.equals("None") ) {
		int[] panelNb = {1,1,1,1};
		mgp = new MGInfComplexModelPanel( this, panelNb);
	}
	else { 
		mgp = new MGInfComplexModelPanel(this,getPanelsNb());
	}
	((MGInfComplexModelPanel)mgp).updateParam(vd);	// mise a jour des params du modele
	
   	codecPanel = new CodecPanel(this);
   	codecPanel.setCodecName( vd.getCodec().getName());  // mise a jour des params du codec
   
 	interneModelPanel.add(mgp);
  	interneModelPanel.add(codecPanel);
	interneModelPanel.add(Box.createRigidArea(new Dimension(0,5)));
	interneModelPanel.add(modelActionsPanel);
	
    add(interneModelPanel);
	//this.setSize(GestionTest.DEFAULT_DIMENSION);
}


/**
 * To create ComboBox content of:
 * 	distributions, continuous distributions, audio codecs, video codecs, functions
 * 
 */
public void makeLists() {
//	Liste des distributions possibles
	 //liste des distributions continues + discretes 
	 listDistribution = new Hashtable();
	 try {
		 listDistribution.put("statistic.distribution.ConstantDistribution","Constant");
		 listDistribution.put("statistic.distribution.ExponentialDistribution","Exponential");
		 listDistribution.put("statistic.distribution.GammaDistribution","Gamma");
		 listDistribution.put("statistic.distribution.GeometricDistribution","Geometric");
		 listDistribution.put("statistic.distribution.InverseGaussianDistribution","InverseGaussian");
		 listDistribution.put("statistic.distribution.LogNormalDistribution","LogNormal");
		 listDistribution.put("statistic.distribution.NormalDistribution","Normal");
		 listDistribution.put("statistic.distribution.ParetoDistribution","Pareto");
		 listDistribution.put("statistic.distribution.PoissonDistribution","Poisson");
		 listDistribution.put("statistic.distribution.UniformDistribution","Uniform");
		 listDistribution.put("statistic.distribution.WeibullDistribution","Weibull");
	 }
	 catch (NullPointerException e) { 
		 System.out.println("InterneModelPanel.constructor :" + e.toString());
	 }
	
	 // liste des distributions contimues uniquement 
	 listContinuousDistribution = new Hashtable();
	 try {
		 listContinuousDistribution.put("statistic.distribution.ConstantDistribution","Constant");
		 listContinuousDistribution.put("statistic.distribution.ExponentialDistribution","Exponential");
		 listContinuousDistribution.put("statistic.distribution.GammaDistribution","Gamma");
		 listContinuousDistribution.put("statistic.distribution.InverseGaussianDistribution","InverseGaussian");
		 listContinuousDistribution.put("statistic.distribution.LogNormalDistribution","LogNormal");
		 listContinuousDistribution.put("statistic.distribution.NormalDistribution","Normal");
		 listContinuousDistribution.put("statistic.distribution.ParetoDistribution","Pareto");
		 listContinuousDistribution.put("statistic.distribution.UniformDistribution","Uniform");
		 listContinuousDistribution.put("statistic.distribution.WeibullDistribution","Weibull");
	 }
	 catch (NullPointerException e) { 
		 System.out.println("InterneModelPanel.constructor :" + e.toString());
	 }
	
	
	
	 // liste des codecs possibles  audio/video
	 listAudioCodec = new Hashtable();
	 try {
		 listAudioCodec.put("multimedia.codec.AudioCodec","Audio");
		 listAudioCodec.put("multimedia.codec.AudioCodecExample.G711","G711");
		 listAudioCodec.put("multimedia.codec.AudioCodecExample.AudioCodecUDPIP.G723_1","G723.1");
		 listAudioCodec.put("multimedia.codec.AudioCodecExample.AudioCodecUDPIP.G726","G726");
		 listAudioCodec.put("multimedia.codec.AudioCodecExample.AudioCodecUDPIP.G728","G728");
		 listAudioCodec.put("multimedia.codec.AudioCodecExample.AudioCodecUDPIP.G729","G729");
		
	 }
	 catch (NullPointerException e) { 
		 System.out.println("InterneModelPanel.constructor :" + e.toString());
	 }

	 listVideoCodec = new Hashtable();
	 try {	
		 listVideoCodec.put("multimedia.codec.VideoCodec","Video");
		 listVideoCodec.put("multimedia.codec.VideoCodecExample.MJPEG","MJPEG");
		 listVideoCodec.put("multimedia.codec.VideoCodecExample.MPEG","MPEG");
		 listVideoCodec.put("multimedia.codec.VideoCodecExample.MPEG4","MPEG4");
		 listVideoCodec.put("multimedia.codec.VideoCodecExample.H261","H261");
	 }
	 catch (NullPointerException e) { 
		 System.out.println("InterneModelPanel.constructor :" + e.toString());
	 }
    
    
	 // Liste des fonctions possibles
	 listFunction = new Hashtable();
	 try {
		 listFunction.put("statistic.function.MarkovianFunction","Markovian");
		 listFunction.put("statistic.function.LRDFunction","LRD");
		 listFunction.put("statistic.function.SRDFunction","SRD");
	 }
	 catch (NullPointerException e) { 
		 System.out.println("InterneModelPanel.constructor :" + e.toString());
	 }
}


public String getModelName() { return modelName; }

public void updateModelChoice( String classModelChoiceName ) {
	
	try {
      	mgp = (GenericModelPanel)Class.forName("graphics2.panel.client.model." + classModelChoiceName + "ModelPanel").newInstance();
     }
    
    catch (IllegalAccessException iae) {
    	System.out.println(iae);
		System.out.println("InterneModelPanel.updateModelChoice : pb!");
   	}
     catch(InstantiationException ie) { 
     	System.out.println("InterneModelPanel.updateModelChoice : the initialization provoked by this methode fails => Model Panel Name : " + classModelChoiceName + " "   + ie);
     	 }
     catch (ClassNotFoundException e) { 
     	System.out.println("InterneModelPanel.updateModelChoice : The class cannot be located => Model Panel Name : " + classModelChoiceName + " " + e); 
     }
}


/**
 * It could only be an Audio or Video Codec
 * @param classCodecChoiceName
 */
public void updateCodecChoice() {  codecPanel = new CodecPanel(this); }

public ModelPanel getModelPanel() { return modelPanel; }

public void setGenericModelPanel(GenericModelPanel mgp) {	this.mgp = mgp;}
public  GenericModelPanel  getGenericModelPanel() { return mgp;}

public void setCodecPanel(CodecPanel codecPanel) { this.codecPanel = codecPanel;	}
public CodecPanel  getCodecPanel() { return codecPanel;	}

// retourne la liste des distributions continues + discretes
public Hashtable getListDistribution() { return listDistribution; }
// retourne la liste des distributions continues 
public Hashtable getListContinuousDistribution() { return listContinuousDistribution; }
// retourne la liste des codecs audio
public Hashtable getListAudioCodec() { return listAudioCodec; }
// retourne la liste des codecs video
public Hashtable getListVideoCodec() {	return listVideoCodec; }
// retourne la liste des fonction d'autocorrelations possibles
 public Hashtable getListFunction() {	return listFunction; }
    
    
    
public void freezeInfo(boolean b) {
   	mgp.freezeInfo(b);
   	codecPanel.freezeInfo(b);
   	modelActionsPanel.freezeInfo(b);
}
  
public void reset() {
	mgp.reset();
	codecPanel.reset();
}
    
 protected int[] getPanelsNb() {
    		int[] panelNb = new int[4];
			if ( vd.getModel().getDistributionTest().getName().equals("Mixed") ) {
				panelNb[0] = 2;	
			}
			else { panelNb[0] = 1; }
			if ( vd.getModel().getDistributionSlot().getName().equals("Mixed") ) {
				panelNb[1] = 2;	
			}
			else { panelNb[1] = 1; }
			if ( vd.getModel().getDistributionPacketSize().getName().equals("Mixed") ) {
			panelNb[2] = 2;	
			}
			else { panelNb[2] = 1; }
			if ( ((MixedFunction)vd.getModel().getAutoCorr()).getSpecificFunctions().length == 2 ) {
				panelNb[3] = 2;	
			}
			else if ( ((MixedFunction)vd.getModel().getAutoCorr()).getSpecificFunctions().length != 2 && ( (((MixedFunction)vd.getModel().getAutoCorr()).getSpecificFunctions()[0]).getName().equals("Mixed")) ){
				panelNb[3] = 2;	
			}
			else { panelNb[3] = 1; }
			
    	return panelNb;
    }
    
    public void setSourceNumber(int number) {
    	modelActionsPanel.nb.setText(""+number); 
    }
    
    /**
     * Defines panel where are "add model features" button, "reset" button... 
     * @author mladoue
     * @version 0.1
     */
    class ModelActionsPanel extends JPanel {
    	
		private JButton addFeatures = new JButton("Add Scenario");
    	private JButton reset = new JButton("Reset");
		private	JLabel srcNb = new JLabel("Nb sources for this flow: ");
		private JTextField nb = new JTextField("1",3);
    	
    	public ModelActionsPanel() {
    		
    		this.setLayout(new BoxLayout( this,BoxLayout.X_AXIS));
			JPanel maPanel = new JPanel();
			maPanel.setLayout(new BoxLayout( maPanel,BoxLayout.X_AXIS));
			
			setLayout(new BoxLayout( this,BoxLayout.X_AXIS));
		
			// set textFiel max size.
			nb.setMaximumSize(new Dimension(nb.getPreferredSize().width, nb.getPreferredSize().height));
			
			maPanel.add(Box.createRigidArea(new Dimension(20,0)));
			maPanel.add(srcNb);
			maPanel.add(Box.createRigidArea(new Dimension(10,0)));
			maPanel.add(nb);
			maPanel.add(Box.createRigidArea(new Dimension(170,0)));
			maPanel.add(addFeatures);
			maPanel.add(Box.createRigidArea(new Dimension(20,0)));
			maPanel.add(reset);
			maPanel.add(Box.createHorizontalGlue());//RigidArea(new Dimension(60,0)));
			maPanel.add(MessageManager.createHelpMessageDialogButton("Help window...to do"));
			maPanel.add(Box.createRigidArea(new Dimension(20,0)));
			
			makeButtons();
			
			add(maPanel);
			// For the moment, no aggregation.
			srcNb.setEnabled(false);
			nb.setEnabled(false);
    	}
    	
		public void makeButtons() {
                    addFeatures.addActionListener( new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                                //TODO: DS ajouter iptimemax et errormax dans la racine de l'arbre. Voir a quel moment on le fait...
                            System.out.println("Add a model in the XML Tree");
                            
                            // Before adding this scenario in the scenario list, check that all
                            // parameters are ok.
                            if (mgp.isAValidModel()) {
 	                            org.w3c.dom.Element elt = mgp.buildNewXmlNode();
    	                            // Add attribute srcnumber and time ??
        	                    elt.setAttribute("srcnumber", nb.getText());
            	                ClientPanel clientPanel = modelPanel.getClientPanel();
                	            elt.setAttribute("duration", String.valueOf(clientPanel.getDuration()));
                    	        
                        	        // Update the scenario name using the other parameters.
                            	String strname = elt.getAttribute("name");
                            	elt.setAttribute("name", "scenario: "+ strname + " srcNb: " + elt.getAttribute("srcnumber")+ " duration: " + elt.getAttribute("duration"));
                                	
	                                // Add codec info !!!!
    	                            // Only done for non aggregated flows...
        	                    if (!mgp.getClass().getName().equals("graphics2.panel.client.model.MGInfComplexAggregatedFlowModelPanel") &&
            	                !mgp.getClass().getName().equals("graphics2.panel.client.model.OnOffHeterogeneousAggregatedFlowModelPanel"))
                	            {                                
                    	        	org.w3c.dom.Element codec = codecPanel.buildNewXmlNode();
                        	        	// Add it as a child of current flow
                            		elt.appendChild(codec);
	                            }	
    	                        
        	                        // Add this new scenario in the Tree
            	                clientPanel.getListScenario().addElement(elt);
                	                // Write the Tree... it's for debug only for the moment
                    	        clientPanel.saveScenarioFile("toto.xml");

                        	        // Add the scenario name dans la liste
                            	clientPanel.addFlowToListPanel(elt);
                            }
                        }
                    });	
                    
                    reset.addActionListener( new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            mgp.reset();
                            codecPanel.reset();	
                        }
                    });	
		}
		
		public void freezeInfo( boolean b ) {
			addFeatures.setEnabled(!b);
			reset.setEnabled(!b);
			//nb.setEnabled(!b);
		}
		
		public void reset() {
			addFeatures.setEnabled(true);
			reset.setEnabled(true);
			//nb.setEnabled(true);
			nb.setText("1");
		}
    }
}
