package graphics2.panel.client;

import statistic.function.*;
import utils.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;


/**
 * // Particular case where two functions are part of the MixedFunction
 * @author mladoue
 * @version 0.2, 17/06/03
 */
public class MixedFunctionPanel extends JPanel implements ActionListener {
    protected double[] coeffFunction = new double[2];
    protected FunctionPanel[] functionPanels = new FunctionPanel[2];
    protected JCheckBox mixedFunction;
    protected JTextField[] coeff = new JTextField[2];
    protected JLabel[] coeffName = new JLabel[2];
    protected int nbPanel;
    public boolean isMixedFunc = false;
    protected FunctionNameListener fnl;

    /**
     * Method MixedFunctionPanel.
     * @param listFunction
     * @param nbPanel
     */
    public MixedFunctionPanel(Hashtable listFunction, int nbPanel) {
        super();
        this.nbPanel = nbPanel;

        if (nbPanel == 1) {
            isMixedFunc = false;
        } else {
            isMixedFunc = true;
        }

        functionPanels[0] = new FunctionPanel("", listFunction);
        functionPanels[1] = new FunctionPanel("", listFunction);

        this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        JPanel panel1 = new JPanel();
        panel1.setLayout(new BoxLayout(panel1, BoxLayout.X_AXIS));

        panel1.add(Box.createRigidArea(new Dimension(10, 0)));
        panel1.add(mixedFunction = new JCheckBox("Mixed Function", isMixedFunc));
        mixedFunction.addItemListener(fnl = new FunctionNameListener());
        mixedFunction.addActionListener(this);
        panel1.add(Box.createRigidArea(new Dimension(20, 0)));
        panel1.add(MessageManager.createHelpMessageDialogButton(
                "Help window...to do"));
        panel1.add(Box.createRigidArea(new Dimension(10, 0)));
        panel1.add(Box.createRigidArea(new Dimension(20, 0)));

        JPanel panel2 = new JPanel();
        panel2.setLayout(new BoxLayout(panel2, BoxLayout.X_AXIS));

        panel2.add(Box.createRigidArea(new Dimension(10, 0)));


		WeightListener wlistener = new WeightListener();
        for (int i = 0; i < 2; i++) {
            panel2.add(Box.createRigidArea(new Dimension(10, 0)));
            panel2.add(coeffName[i] = new JLabel("Function " + (i + 1) +
                        " weight:"));
            panel2.add(Box.createRigidArea(new Dimension(10, 0)));
            panel2.add(coeff[i] = new JTextField("1.0",6));
			coeff[i].setMaximumSize(new Dimension(coeff[i].getMaximumSize().width, 
																		  coeff[i].getPreferredSize().height));
			WeightFocusListener wflistener = new WeightFocusListener(i);
			coeff[i].addActionListener(wlistener);
			coeff[i].addFocusListener(wflistener);
            if (isMixedFunc) {
                coeff[i].setEditable(true);
            } else {
                coeff[i].setEditable(false);
            }

            panel2.add(Box.createRigidArea(new Dimension(5, 0)));
        }

        panel2.add(Box.createRigidArea(new Dimension(10, 0)));

        JPanel panel3 = new JPanel();
        panel3.setLayout(new BoxLayout(panel3, BoxLayout.X_AXIS));

        panel3.add(Box.createRigidArea(new Dimension(5, 0)));
        panel3.add(functionPanels[0]);
        panel3.add(Box.createRigidArea(new Dimension(5, 0)));
        panel3.add(functionPanels[1]);
        panel3.add(Box.createRigidArea(new Dimension(5, 0)));

        updateFunction(getFunctionsName(), getWeights());

        add(Box.createRigidArea(new Dimension(0, 5)));
        add(panel1);
        add(Box.createRigidArea(new Dimension(0, 10)));
        add(panel2);
        add(Box.createRigidArea(new Dimension(0, 10)));
        add(panel3);
        add(Box.createRigidArea(new Dimension(0, 10)));

        // Set the Border
        setBorder(new TitledBorder("AutoCorrelation Function"));
    }

    // Definitions de differents methodes
    public int getPanelNumber() {
        return nbPanel;
    }

    public void setPanelNumber(int nbPanel) {
        this.nbPanel = nbPanel;
    }

    public GenericFunction getSpecificFunction(int idx) {
        return functionPanels[idx].getFunction() ;
    }

    public void setSpecificFunction(int idx, GenericFunction f) {
		functionPanels[idx].setFunction(f);
    }

    public GenericFunction[] getFunctions() {
    	GenericFunction[] f = new GenericFunction[functionPanels.length];
    	for (int i=0; i <functionPanels.length ;i++)
    		f[i]=getSpecificFunction(i);
        return f;
    }

    public MixedFunction getMixedFunction() {
        return new MixedFunction(getFunctions(), getWeights());
    }

    public void setFunctions(GenericFunction[] functions) {
    	for (int i=0; i<functions.length;i++)
    		setSpecificFunction(i, functions[i]);
    }

//    public void setMixedFunction(MixedFunction mf) {
//        this.function = mf;
//    }

    public FunctionPanel[] getFunctionPanels() {
        return functionPanels;
    }

    public FunctionPanel getFunctionPanel(int idx)
        throws IllegalArgumentException {
        return functionPanels[idx];
    }

    public void setFunctionPanels(FunctionPanel[] fps) {
        this.functionPanels = fps;
    }

    // Function weight
    // Methode utilise pour positionner/recuperer des valeurs via load and save
    public void setWeights(double[] values) {
        for (int i = 0; i < values.length; i++) {
            coeff[i].setText("" + values[i]);
        }
    }

    public double[] getWeights() {
        double[] ans = new double[2];

        for (int i = 0; i < nbPanel; i++) {
            ans[i] = Double.parseDouble(coeff[i].getText());
        }

        return ans;
    }

    public void setWeight(int panelNb, double value) {
        coeff[panelNb].setText("" + value);
    }

    public String getWeight(int idx) throws IllegalArgumentException {
        return coeff[idx].getText();
    }

    // Function name
    public void setFunctionsName(String[] names) {
        for (int i = 0; i < names.length; i++) {
            functionPanels[i].setFunctionName(names[i]);
        }
    }

    public void setFunctionName(int idx, String name) {
        functionPanels[idx].setFunctionName(name);
    }

    public String[] getFunctionsName() {
        String[] ans = new String[2];

        for (int i = 0; i < nbPanel; i++) {
            ans[i] = functionPanels[i].getFunctionName();
        }

        return ans;
    }

    public String getFunctionName(int idx) {
        return functionPanels[idx].getFunctionName();
    }

    // Function parameter
    public void setFunctionsParameter(double[] params) {
        for (int i = 0; i < params.length; i++) {
            functionPanels[i].setParam(params[i]);
        }
    }

    public double[] getFunctionsParameter() {
        double[] ans = new double[nbPanel];

        for (int i = 0; i < nbPanel; i++) {
            ans[i] = functionPanels[i].getParam();
        }

        return ans;
    }

    public void setMixedFunction(boolean mixed) {
        mixedFunction.setSelected(mixed);
        isMixedFunc = mixed;

        if (isMixedFunc) {
            nbPanel = 2;
            coeffName[0].setEnabled(true);
            coeff[0].setEditable(true);
            coeffName[1].setEnabled(true);
            coeff[1].setEditable(true);
            functionPanels[1].freezeInfo(false);
        } else {
            nbPanel = 1;
            coeffName[0].setEnabled(true);
            coeff[0].setEditable(true);
            coeffName[1].setEnabled(false);
            coeff[1].setEditable(false);
            functionPanels[1].freezeInfo(true);
        }
    }

    public boolean isMixedFunction() {
        return isMixedFunc;
    }

    public void setFunctionParameter(int idx, double params) {
        functionPanels[idx].setParam(params);
    }

    public double getFunctionParameter(int idx) {
        return functionPanels[idx].getParam();
    }

    public void freezeInfo(boolean b) {
        mixedFunction.setEnabled(!b);

        for (int i = 0; i < nbPanel; i++) {
            coeffName[i].setEnabled(!b);
            coeff[i].setEditable(!b);
            functionPanels[i].freezeInfo(b);
        }
    }

	private boolean isAFloat(String str, String str2) {
		try {
			if (str==null || str.equalsIgnoreCase(""))
				throw new NumberFormatException();
			double value = Double.parseDouble(str);
		}
		catch (NumberFormatException nfe) {
			JOptionPane.showMessageDialog(this, "Wrong format for "+str2+". This is not a float!","Float format error",JOptionPane.ERROR_MESSAGE);
			// Please Insert a number !
			return false;
		}
		return true;
	}

	public boolean isAValidPanel() {
		if (functionPanels[0].isAValidPanel())
		{
			if (isMixedFunc &&
				(!isAFloat(coeff[0].getText(), "Function weigth1") ||
				!isAFloat(coeff[1].getText(), "Function weigth2") ||
				!functionPanels[1].isAValidPanel()))
				return false;
			return true;
		}
		else
			return false;
	}
	
    public void reset() {
        mixedFunction.setEnabled(true);

        for (int i = 0; i < nbPanel; i++) {
            coeff[i].setText("");
            functionPanels[i].reset();
        }
    }

    //change le nom du parametre de la loi choisie
    public void actionPerformed(ActionEvent e) {
        updateFunction(getFunctionsName(), getWeights());
        System.out.println(functionPanels[0].getFunction().getName() + " " +
		functionPanels[1].getFunction().getName());
    }

    // met a jour le MixedFunctionPanel ( essentiellement modifie le nom des parametres + equation en fonction du nom de la loi
    public void updateFunction(String[] classFunctionName,
        double[] coeffFunction) {
        int i;

        for (i = 0; i < nbPanel; i++) {
            try {
                GenericFunction function = (GenericFunction) Class.forName(
                        "statistic.function." + classFunctionName[i] +
                        "Function").newInstance();
                functionPanels[i].setFunction(function);
            }
             catch (IllegalAccessException iae) {
                System.out.println(iae);
            } catch (InstantiationException ie) {
                System.out.println(
                    "the initialization provoked by this methode fails => Function Name : " +
                    classFunctionName[i] + " " + ie);
            } catch (ClassNotFoundException cnfe) {
                System.out.println(
                    "The class2 cannot be located => Function Name : " +
                    classFunctionName[i] + " " + cnfe);
            }
        }

        this.coeffFunction = coeffFunction;

        for (i = 0; i < nbPanel; i++) {
            functionPanels[i].updateFunction(classFunctionName[i]);
            functionPanels[i].updateEquation();
        }
    }

    protected class FunctionNameListener implements ItemListener {
        public FunctionNameListener() {
        }

        public void itemStateChanged(ItemEvent e) {
            //Object source = e.getItemSelectable();	//26/05/03
            //Now that we know which button was pushed, find out
            //whether it was selected or deselected.
            if (e.getStateChange() == ItemEvent.DESELECTED) {
                isMixedFunc = false;
                nbPanel = 1;
                coeffName[0].setEnabled(true);
                coeff[0].setEditable(true);
                coeffName[1].setEnabled(false);
                coeff[1].setEditable(false);
                functionPanels[1].freezeInfo(true);
            } else {
                isMixedFunc = true;
                nbPanel = 2;
                coeffName[0].setEnabled(true);
                coeff[0].setEditable(true);
                coeffName[1].setEnabled(true);
                coeff[1].setEditable(true);
                functionPanels[1].freezeInfo(false);
            }
        }
    }
    
    
    // Respond to weigth modification
	/**
	 * Return a String that is the result of 1.0 - number
	 * number MUST be a number starting with "0."
	 * The float is converted into integer to prevent rounded error.
	 */
	private String getOtherValue(String number) {
		int inc=0;
		
		// First, check that the value is valid !
		if (number.equalsIgnoreCase("") || Double.parseDouble(number) > 1.0) {
			coeff[0].setText("0.5"); // Default the first TextField to 0.5 !
			coeff[1].setText("0.5");
			return ("0.5");
		}

			// Secon, remove the "0." in the string....
		number=number.substring(2);
		inc = number.length();
		long val = Long.parseLong(number);
		return new String("0."+((long)Math.pow(10,inc) - val));
	}

	// Classe interne
	// Listen weith modification (Must press ENTER to execute this action)
	 class WeightListener implements ActionListener {
			public void actionPerformed(ActionEvent e) {
				   // La modification de l'un des champs weigth entraine la modif de l'autre.
				   // getActionCommand du ActionEvent contient la valeur du champs que l'on vient de modifier.
				   if (e.getActionCommand().equals(coeff[0].getText())) {
						coeff[1].setText(getOtherValue(coeff[0].getText()));
				   }
				   else {
						coeff[0].setText(getOtherValue(coeff[1].getText()));
				   }
			}
	  }
	
	class WeightFocusListener implements FocusListener {
		private int pos;
				
		public WeightFocusListener(int pos) {
			this.pos=pos;
		}
		public void focusGained(FocusEvent e) {
			// Do nothing			
		}

		public void focusLost(FocusEvent e) {
			if (pos==0) {
				coeff[1].setText(getOtherValue(coeff[0].getText()));
		   }
		   else {
				coeff[0].setText(getOtherValue(coeff[1].getText()));
		   }
		}
	}
}
