/**
 * ModelPanel : Definition du Panel correspondant au choix du modele
 *
 * @author maria ladoue
 * @version 0.1, 06/08/02 
 */

package graphics2.panel.client;

import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;

import multimedia.model.*;
import utils.*;

public class ModelPanel extends JPanel implements ActionListener {

    protected JComboBox modelName, mginfcomplexmodelName;
    protected Hashtable listModel;
    protected Hashtable listMGInfModel;
    protected int nbParams = 2; // par defaut c'est 2 (i.e: le minimum)

    protected GenericAggregatedModel model;  // modif le 2/4/03
    protected InterneModelPanel imp;
    protected ClientPanel clientPanel;
    protected ModelNameListener cblModelName;
    protected MGInfComplexModelNameListener cblMGInfModelName;
    protected JPanel mPanel = new JPanel(), panel1 = new JPanel();


	/**
	 * @param clientPanel
	 * @param listModel an Hashtable
	 * @param listMGInfModel	an Hashtable
	 */
    public ModelPanel( ClientPanel clientPanel, Hashtable listModel, Hashtable listMGInfModel ) {
		super();
		this.clientPanel = clientPanel;
		this.listModel =  listModel;
		this.listMGInfModel = listMGInfModel;
		
		createComboBox();
		imp = new InterneModelPanel(this);

		setLayout( new BoxLayout(this,BoxLayout.Y_AXIS) );
		
		// Definition de l'interface graphique
		 mPanel.setLayout( new BoxLayout(mPanel,BoxLayout.Y_AXIS) );
		
	
	
		
	    panel1.setLayout( new BoxLayout(panel1,BoxLayout.X_AXIS) );
		panel1.add(Box.createRigidArea(new Dimension(10,0)));
		panel1.add(modelName);
		panel1.add(Box.createHorizontalGlue());
		panel1.add(mginfcomplexmodelName);
		panel1.add(Box.createRigidArea(new Dimension(40,0)));
		
		if( SpecificObjectCreation.createModelName(getModelName()).equals("MGInfComplex") ) {
			mginfcomplexmodelName.setEnabled(true);
		}
		else { mginfcomplexmodelName.setEnabled(false);	}
		
		mPanel.add(Box.createRigidArea(new Dimension(0,5)));
		mPanel.add(panel1);  // contient les ComboBox
		mPanel.add( Box.createRigidArea(new Dimension(0 ,5)));
		mPanel.add(imp);
		
		// definit pour etre utiliser dans le cas ou l'interface graphique du model est trop grosse
		JScrollPane scroller = new JScrollPane( mPanel  ); 
		scroller.setBorder( new TitledBorder("Model features"));
		
		this.add(scroller);
		
    } // fin du constructeur ModelPanel

public void createComboBox() {
	//	Liste des modeles disponibles
	 modelName = new JComboBox();
	
	// Cannot be done with an enumeration if we want the elements in the right order.
	 modelName.addItem("On-Off");
	 modelName.addItem("On-Off Heterogeneous Aggregated Flow");
	 modelName.addItem("MMPP-2");
	 modelName.addItem("MMPP-N");
	 modelName.addItem("M/G/Inf");
	 modelName.addItem("M/G/Inf Complex");
	 modelName.addItem("M/G/Inf Complex Aggregated Flows");
	 modelName.setEditable(false);
	 modelName.setSelectedIndex(0);
	 modelName.setMaximumRowCount(3);
	 modelName.setMaximumSize(new Dimension(modelName.getPreferredSize().width, 
																		   modelName.getPreferredSize().height));

	 modelName.addActionListener(this);
	 modelName.addItemListener(cblModelName = new ModelNameListener());

	 // Liste des modeles M/G/Inf pre-definis accessibles
	 mginfcomplexmodelName = new JComboBox();
	
	 Enumeration valMGInfModelName = listMGInfModel.elements();  // pour acceder aux elements de la HashTable 

	 while(valMGInfModelName.hasMoreElements()) {
		 mginfcomplexmodelName.addItem(valMGInfModelName.nextElement());
	 }

	 mginfcomplexmodelName.setEditable(false);
	 mginfcomplexmodelName.setSelectedIndex(0);
	 mginfcomplexmodelName.setMaximumRowCount(3);
	mginfcomplexmodelName.setMaximumSize(new Dimension(mginfcomplexmodelName.getPreferredSize().width, 
																							mginfcomplexmodelName.getPreferredSize().height));
	 mginfcomplexmodelName.addActionListener(cblMGInfModelName = new MGInfComplexModelNameListener());
}


/**
 * Method setModelTest.
 * @param modelName
 */
 public void setModelTest(String modelN) {
 	updateModel(modelN);
    //System.out.println("listModel.get(modelName.getClass().getName()) : " + listModel.get(model.getClass().getName()) );
    setModelTestName(modelN);
  	updateModelPanel(imp,SpecificObjectCreation.createModelName( getModelName()) );
  }
  
/**
 * Method setModelTestName.
 * @param modelName
 */
  public void setModelTestName(String modelN) throws IllegalArgumentException {
	modelName.setSelectedItem( SpecificObjectCreation.createInvModelName(modelN) );
  }
  
    // retourne le modelName choisi
    public GenericAggregatedModel getModelTest() { return model; }   // modifie le 02/04/03 : GenericModel => GenericAggregatedModel
   
    public ClientPanel getClientPanel() {	return clientPanel; }
    public InterneModelPanel getInterneModelPanel() { return imp; }
    

/**
 * Method updateModelPanel : used to update the model panel and the codec panel
 * @param imp
 * @param classModelName
 */
protected void updateModelPanel( InterneModelPanel imp, String classModelName) {
		imp.updateModelChoice( classModelName );
		imp.updateCodecChoice(); // Audio ou video
}

protected void updateModel(String classModelName) {
	
	if( classModelName.equals("MGInfComplex") ) {
		classModelName = new String("MGInf");
	}
	if( classModelName.equals("MGInfComplexAggregatedFlow") ) {
			classModelName = new String("MGInfAggregatedFlow");
	}
     try {
       //model = (GenericModel)Class.forName("multimedia.model." + classModelName +"Model").newInstance();  // modifie le 02/04/03
       model = (GenericAggregatedModel)Class.forName("multimedia.model." + classModelName +"Model").newInstance(); 
     }
    
    catch (IllegalAccessException iae) {
    		System.out.println(iae);
			System.out.println("ModelPanel.updateModel : pb");
    	}
     catch(InstantiationException ie) { 
     	System.out.println("ModelPanel.updateModel : the initialization provoked by this methode fails => Model Name : " + classModelName + " "  + ie);
     }
     catch (ClassNotFoundException cnfe) { 
     	System.out.println("ModelPanel.updateModel : The class cannot be located => Model Name : " + classModelName + " " + cnfe);
     	 }
  }


/**
 * @return String on the form : MMPP-2 et Jurrassik (Low Quality) si c'est du M/G/Inf
 */
public String getModelName() {
	String ans = "";
	if (  (modelName.getSelectedItem()).equals("MGInfComplex") ){
		if ( !(cblMGInfModelName.isNoneSelection()) ) {
		ans = getMGInfComplexModelName();
		}
		ans = "MGInf";
	}
	else {
		ans = modelName.getSelectedItem().toString();
	}
	return ans;
}

/**
 * @return String on the form : Jurrassik (Low Quality)....
 */
public String getMGInfComplexModelName() { return new String(mginfcomplexmodelName.getSelectedItem().toString()); }

public boolean isMGInfComplexSelection() {
	if ( (SpecificObjectCreation.createModelName(ModelPanel.this.getModelName())).equals("MGInfComplex") ) {
		return true;	
	}
	return false;	
} 


public void freezeInfo ( boolean b) {
	 modelName.setEnabled(!b);
     imp.freezeInfo(b);
     if( b && modelName.getSelectedItem().equals("MGInfComplex") ) {
     	mginfcomplexmodelName.setEnabled(!b);
     }
} // ajouter le 09/01/03

public void reset() {
	modelName.setEnabled(true);
	imp.reset();
}


 // change le nom du modelName choisi
  public void actionPerformed(ActionEvent e) {
  	
  	String modelCompleteName = SpecificObjectCreation.createModelName(getModelName());
  	updateModel( modelCompleteName );
  	mPanel.remove(imp);  // supprime InterneModelPanel de mPanel
  	if ( !(isMGInfComplexSelection()) ) {
 
  		//System.out.println("Avant: " + imp.getGenericModelPanel().getModelType());
  		imp = new InterneModelPanel(this);
  		//updateModelPanel(imp,  modelCompleteName);  //DEBUG 19/05/03
  		validate();
  		 //System.out.println("Apres: " +  imp.getGenericModelPanel().getModelType());
  	}
  
  	//else if ( isMGInfComplexSelection()) {
  	else {
    	imp = new InterneModelPanel( this, mginfcomplexmodelName.getSelectedItem().toString());
		validate();
  	}
  	//else { throw new IllegalArgumentException("ModelPanel.actionPerformed : There is a problem of selection!");}
     mPanel.add(imp);
     
 	clientPanel.freezeInfo(false);
	//clientPanel.getGestionTest().pack();
    validate();
	clientPanel.validate();
  }



// CLASSES INTERNES
// Listens to the comboBox. 

protected class ModelNameListener implements ItemListener {

	public ModelNameListener() {}
	
	public void itemStateChanged(ItemEvent e) {
     
        //Now that we know which model; was selected, find out
        //whether it was selected or deselected.
      		 if( SpecificObjectCreation.createModelName(modelName.getSelectedItem().toString()).equals("MGInfComplex") ) {
           		mginfcomplexmodelName.setEnabled(true);
				// //clientPanel.getGestionTest().getServerPanel().setScenarioFileName(mginfcomplexmodelName.getSelectedItem().toString());  // to update scenario file name if server features
        	}
        	else {
        		mginfcomplexmodelName.setEnabled(false);
        		// //clientPanel.getGestionTest().getServerPanel().setScenarioFileName(clientPanel.getModelTestName() + "_" + clientPanel.getDuration());  // to update scenario file name if server features
       		 }      	
		}	
}


protected class MGInfComplexModelNameListener implements ActionListener {
	
	public MGInfComplexModelNameListener() {}
	public void actionPerformed(ActionEvent e) {
  				ModelPanel.this.mPanel.remove(ModelPanel.this.imp);
  				imp = new InterneModelPanel(ModelPanel.this,mginfcomplexmodelName.getSelectedItem().toString());
  				ModelPanel.this.mPanel.add(imp);
  				ModelPanel.this.mPanel.validate(); 
	}
	
	public boolean isNoneSelection() {
		if ( mginfcomplexmodelName.getSelectedItem().equals("None") ) { return true; }
		return false;	
	} 
}	
	 
}
