package graphics2.panel.client;


import java.awt.*;
import java.awt.event.*;
import java.io.File;

import javax.swing.JPanel;
import javax.swing.*;
import javax.swing.tree.DefaultMutableTreeNode;


import graphics2.panel.FlowXmlTreePanel;
import utils.MessageManager;
import utils.ScenarioFileFilter;

/**
 * Here you define graphical interface of flows description
 * 
 * @author mladoue
 * @version 0.1
 */


public class ModelsListPanel extends JPanel {
	
	public final FlowXmlTreePanel fxtpClient;
	protected ClientPanel cp;
	
	private JButton addScenario = new JButton("Add >>");
	private JButton removeScenario = new JButton("Remove");
	private JButton clear = new JButton("Clear");
	//private JButton aggregation = new JButton("Aggregation");
	private JFileChooser chooser = new JFileChooser();

	
	public ModelsListPanel(ClientPanel cp) {
		super();
		this.cp = cp;
		
		fxtpClient = new FlowXmlTreePanel(cp, "Flows description/Scenario list");
		ScenarioFileFilter filter = new ScenarioFileFilter();
		chooser.setFileFilter(filter);
		chooser.setControlButtonsAreShown(false);	// pour cacher les boutons open et cancel 
		chooser.setMultiSelectionEnabled(false);
		chooser.setPreferredSize(new Dimension(400,200));
		this.setLayout(  new BoxLayout(this,BoxLayout.Y_AXIS) );
		
		JPanel mlPanel = new JPanel();
		mlPanel.setLayout( new BoxLayout(mlPanel,BoxLayout.Y_AXIS) );
		
		
		JPanel panel1 = new JPanel();
		panel1.setLayout( new BoxLayout(panel1,BoxLayout.X_AXIS) );
		
		
		JPanel panel12 = new JPanel();
		panel12.setLayout( new BoxLayout(panel12,BoxLayout.Y_AXIS) );
		
		panel12.add( Box.createRigidArea(new Dimension(0,20)) );
		panel12.add(addScenario);
		panel12.add( Box.createRigidArea(new Dimension(0,10)) );
		panel12.add(removeScenario);
		panel12.add( Box.createRigidArea(new Dimension(0,20)) );

		
		JPanel panel13 = new JPanel();
		panel13.setLayout( new BoxLayout(panel13,BoxLayout.Y_AXIS) );
		
		panel13.add(fxtpClient);
		panel13.add( Box.createRigidArea(new Dimension(0,5)) );
		
		JPanel panel132 = new JPanel();
		panel132.setLayout( new BoxLayout(panel132,BoxLayout.X_AXIS) );
		
		panel132.add( Box.createRigidArea(new Dimension(45,0)) );
		panel132.add(clear);
		panel132.add( Box.createRigidArea(new Dimension(45,0)) );
		
		panel13.add(panel132);
		
		panel1.add( Box.createRigidArea(new Dimension(1,0)) );
		panel1.add( chooser );
		panel1.add( Box.createRigidArea(new Dimension(5,0)) );
		panel1.add(panel12);
		panel1.add( Box.createRigidArea(new Dimension(5,0)) );
		panel1.add(panel13);
		panel1.add( Box.createRigidArea(new Dimension(1,0)) );
		
		makeButtons();
		
		mlPanel.add(panel1);
		add(mlPanel);	
	}
	
	/**
	 * Defines action of each buttons
	 */
	public void makeButtons() {
		addScenario.addActionListener( new ActionListener() {
			public void actionPerformed(ActionEvent e) {
   				if ( chooser.getSelectedFile() != null ) {
					String currentFile = chooser.getSelectedFile().getName();
					fxtpClient.addScenarioAsNode( currentFile );	
   				}
   				else {
   					MessageManager.warningMessage("You have to select a scenario file first!", true);
   				}
			}
		});
		
		removeScenario.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				//fxtpClient.removeCurrentNode();
				DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) fxtpClient.getTree().getLastSelectedPathComponent();
				if( selectedNode != null && selectedNode.getLevel() == 1  ) {
					fxtpClient.removeCurrentNode();
				}
				else {
					MessageManager.errorMessage("You don't have selected a flow or scenario definition!",true);
				}
			}
		});
		
		clear.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fxtpClient.clear();
			}
		});
	}
	
	public void freezeInfo(boolean b) {
		addScenario.setEnabled(!b);
		removeScenario.setEnabled(!b);
		clear.setEnabled(!b);
		chooser.setEnabled(!b);
		fxtpClient.setEnabled(!b);
	}
	
	public void reset() {
		addScenario.setEnabled(true);
		removeScenario.setEnabled(true);
		clear.setEnabled(true);
		chooser.setEnabled(true);
		chooser.setCurrentDirectory(new File(System.getProperty("user.dir")));
		fxtpClient.setEnabled(true);
		fxtpClient.clear();
	}
}
