package graphics2.panel.client.model;

/**
 * GenericModelPanel
 * 
 * @author mladoue
 * @version 0.1, 10/01/03 
 */


import javax.swing.*;
import javax.swing.border.*;
import graphics2.*;
import graphics2.panel.client.InterneModelPanel;

public abstract  class GenericModelPanel extends JPanel  {
	
	protected InterneModelPanel imp;
	protected int modelType;
	protected int modelParametersNumber;
	
	protected final static int AUDIO = 0;
 	protected final static int VIDEO = 1;
	
	protected GestionTest gt;
	
	
	public GenericModelPanel() {
		super();
		modelParametersNumber = 1;
	}
	
	public GenericModelPanel(InterneModelPanel imp) {
		super();
		gt = imp.getModelPanel().getClientPanel().getGestionTest();
	
		this.imp = imp;
		modelType = AUDIO;
	
 		this.setBorder( new TitledBorder("Model features"));
	}


public int getModelType() { return modelType; }
public void setModelType( int type) { this.modelType = type; }

public int getParametersNumber() { return modelParametersNumber;}
public void setParametersNumber(int number) { this.modelParametersNumber = number;}

/*
 * Method buildNewXmlNode Construct an XML node storing the information of the panel
 * about the model, the codec, the duration, and the number of source.
 */
    public abstract org.w3c.dom.Element buildNewXmlNode();

	public abstract void updateWithXmlNode(org.w3c.dom.Element elt);
	
	/**
	 * Check that all the parameters of a model are ok.
	 * @return true if parameters are ok, false otherwise.
	 */
	public abstract boolean isAValidModel();
	
	/**
	 * Check that a string is a valid integer
	 * Used by sub class to implemente isAValidModel functions
	 * @param str : the string to check
	 * @param str2 : the string used in the JOptionPane to inform the user.
	 * @return true if str is an integer
	 */
	protected boolean isAnInt(String str, String str2) {
		try {
			if (str==null)
				throw new NumberFormatException();
			int etat = Integer.parseInt(str);
		}
		catch (NumberFormatException nfe) {
			JOptionPane.showMessageDialog(this, "Wrong format for "+str2+". This is not an integer!","Integer format error",JOptionPane.ERROR_MESSAGE);
			// Please Insert a number !
			return false;
		}
		return true;
	}

	/**
	 * Check that str is a valid positive integer
	 * Used by sub class to implemente isAValidModel functions
	 * @param str : the string to check
	 * @param str2 : the string used in the JOptionPane to inform the user.
	 * @return true if str is an integer and >=0
	 */
	protected boolean isAPosInt(String str, String str2) {
		return (isAnInt(str,str2) && Integer.parseInt(str) >=0 );	
	}
	
	/**
	 * Check that a string is a valid float
	 * Used by sub class to implemente isAValidModel functions
	 * @return true if str is a float
	 */
	protected boolean isAFloat(String str, String str2) {
		try {
			if (str==null)
				throw new NumberFormatException();
			double value = Double.parseDouble(str);
		}
		catch (NumberFormatException nfe) {
			JOptionPane.showMessageDialog(this, "Wrong format for "+str2+". This is not a float!","Float format error",JOptionPane.ERROR_MESSAGE);
			// Please Insert a number !
			return false;
		}
		return true;
	}

public abstract void freezeInfo(boolean b);
public abstract void reset();

}
