package graphics2.panel.client.model;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

import org.w3c.dom.Element;

import multimedia.VideoDefinition;
import graphics2.panel.*;
import graphics2.panel.client.*;
import utils.*;
import utils.definitions.*;

/**
 * @author mladoue
 * @version 0.1, 16/05/03 
 */

public class MGInfComplexAggregatedFlowModelPanel extends GenericModelPanel implements DefaultObjectDefinitions {

	protected JComboBox mginfcomplexmodelName;
	protected Hashtable listMGInfModel;
	
	protected JButton addNb = new JButton("Add Flow");
	protected	JButton removeNb = new JButton("Remove Flow");
	protected	JButton clearFlow = new JButton("Clear All");
	protected	JLabel srcNb = new JLabel("Number of sources for this flow: ");
	protected	final JTextField nb = new JTextField("",3);


	protected FlowTreePanel mdt = new FlowTreePanel("Flows description list");
	private int flowsNumber = 0;  // nb de flows existants
	private int nodeNumber = 0;  // numero du prochain flot pouvant etre cree
	
	private Toolkit toolkit = Toolkit.getDefaultToolkit();
	private TreeSet videoValue = new TreeSet();  // savoir s'il y a ou non un model definit pour le flow correspondant

	
	public MGInfComplexAggregatedFlowModelPanel() {
		super();
		modelType = VIDEO;
		modelParametersNumber = 1;   // minumum number of parameters
	}
	
	
	public MGInfComplexAggregatedFlowModelPanel( InterneModelPanel imp ) {
		super(imp);
		modelType = VIDEO;
		modelParametersNumber = 1;   // minumum number of parameters
	
		this.listMGInfModel = (imp.getModelPanel().getClientPanel()).getListVideo();
	
		this.setLayout( new BoxLayout(this,BoxLayout.X_AXIS) );
		
		createComboBox();
		
		JPanel videoaggPanel = new JPanel();
		videoaggPanel.setLayout( new BoxLayout(videoaggPanel,BoxLayout.X_AXIS) );
	
		JPanel panel1 = new JPanel();
		panel1.setLayout( new BoxLayout( panel1, BoxLayout.Y_AXIS) );
	
			// Set flow parameters
		JPanel panel11 = new JPanel();
		panel11.setLayout( new BoxLayout( panel11, BoxLayout.X_AXIS) );
		
		panel11.add(Box.createRigidArea(new Dimension(50,0)));	
		panel11.add(srcNb);
		panel11.add(Box.createRigidArea(new Dimension(20,0)));	
		panel11.add(nb);
		nb.setEditable(true);
		nb.setMaximumSize(new Dimension(nb.getPreferredSize().width, 
																nb.getPreferredSize().height));
		panel11.add(Box.createRigidArea(new Dimension(60,0)));
		panel11.add(mginfcomplexmodelName);
		
			// Add buttons
		JPanel panel13 = new JPanel();
		panel13.setLayout(new BoxLayout(panel13,BoxLayout.X_AXIS));

		panel13.add(Box.createRigidArea(new Dimension(50,0)));	
		panel13.add(addNb);
		panel13.add(Box.createRigidArea(new Dimension(20,0)));	
		panel13.add(removeNb);
		panel13.add(Box.createRigidArea(new Dimension(50,0)));
		panel13.add(clearFlow);
		panel13.add(Box.createRigidArea(new Dimension(100,0)));
				
		panel1.add(panel11);
		panel1.add(Box.createRigidArea( new Dimension(0,50)));
		panel1.add(panel13);
		panel1.add(Box.createRigidArea(new Dimension(0,5)));
		
		JPanel panel2 = new JPanel();
		panel2.setLayout( new BoxLayout(panel2,BoxLayout.Y_AXIS));
		panel2.add(mdt);
	
		videoaggPanel.add(panel1);
		videoaggPanel.add(Box.createRigidArea(new Dimension(0,5)));
		videoaggPanel.add(panel2);
		
		makeButtons();
		
		add(videoaggPanel);
		
		//Positionner le nb de parametre
		 setModelParametersNumber();
		
		// Set the Border
		this.setBorder( new TitledBorder("Aggregated Video Flows features") );
	}
	
	
	public void createComboBox() {
		//	Liste des modeles M/G/Inf pre-definis accessibles
			 mginfcomplexmodelName = new JComboBox();
	
			 Enumeration valMGInfModelName = listMGInfModel.elements();  // pour acceder aux elements de la HashTable 

			 while(valMGInfModelName.hasMoreElements()) {
				 mginfcomplexmodelName.addItem( valMGInfModelName.nextElement() );
			 }
		 
			 mginfcomplexmodelName.removeItem( new String("None") );
			 mginfcomplexmodelName.setEditable( false );
			 mginfcomplexmodelName.setSelectedIndex( 0 );
			 mginfcomplexmodelName.setMaximumSize(new Dimension(mginfcomplexmodelName.getPreferredSize().width, 
																										mginfcomplexmodelName.getPreferredSize().height));

			 mginfcomplexmodelName.setMaximumRowCount( 3 );
	}
	
	/**
	 * Make the buttons to add/remove a flow, a video and clear all
	 */
	protected void makeButtons() {
		addNb.addActionListener( new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					// First, check that the model is ok.
					if (isAPosInt(nb.getText(), "flow numbers")) {
						flowsNumber++;
						nodeNumber++;
	
						// Add the flow
						setSpecificFlowDefinition( nodeNumber, flowsNumber, Integer.parseInt(nb.getText()));
						// Add  the video type.
						String videoName = mginfcomplexmodelName.getSelectedItem().toString();
						videoName = VideoDefinition.getSimplifiedName(videoName);
						mdt.addSpecificChild( flowsNumber-1, "Video: " + videoName);
					}
				}
		});	
		
		removeNb.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if ( mdt.hasSelectedNode() ) {
						if ( mdt.isAFirstLevelParent() ) {
							videoValue.remove( new Integer(mdt.getCurrentFlowSelection()+1) );
							mdt.removeCurrentNode(true);
							flowsNumber--;							
						}
						else {
							MessageManager.errorMessage("You must select a flow first!",true);
						}
					}
					else {
						toolkit.beep();
					}
				}
		});
		
		clearFlow.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						videoValue.clear();
						mdt.clear();
						nodeNumber =0;
						flowsNumber = 0;
					}
			});	
	}
	
	/**
	 * @param idx between 1 et nbDifferentFlow
	 * @return a VideoDefinition which belongs to VideoExample<BR>
	 *  Format of returned String : "Bean (Medium Quality)"
	 */
	public VideoDefinition getFlowVideoDefinition( int idx ) throws IllegalArgumentException {
		if( idx>0 && idx<=flowsNumber ) {
			String name =  this.getSpecificVideoDefinition(idx);
			name = VideoDefinition.getCompleteName( name.split(": ",2)[1]);
		
			return SpecificObjectCreation.createVideoExample(name);
		}
		else { throw MyExceptionDefinitions.BAD_INDEX_ACCESS; }
	}
	
	/**
	 * @param idx between 1 and nbDifferentFlow
	 * @return the number of source for the specified flow
	 */
	public int getFlowSourceNumber( int idx) {
		if( idx>0 && idx<=flowsNumber ) {
			String flow = getSpecificFlowDefinition(idx);
			int number = Integer.parseInt( flow.split(" ", 5)[3] );	//"Flow " + flowNumber + " : " + sourcesNumber + " source(s).
			return number;
		}
		else { throw MyExceptionDefinitions.BAD_INDEX_ACCESS; }
	}
	
	public int getFlowTypeNumber() { return mdt.getFlowNumber();	}
	public void setFlowNumber( int flowsNumber ) { this.flowsNumber = flowsNumber; }
	public int getCurrentFlowSelection() { return (mdt.getCurrentFlowSelection()+1); }
	
	/**
	 * @param flowNumber  it's an int between 1 and N
	 * @param sourcesNumber
	 */
	public void setSpecificFlowDefinition(int nodeNumber, int flowNumber, int sourcesNumber) {
		if( mdt.addChild( true,"Flow " + nodeNumber + " : " + sourcesNumber + " source(s).") == 1) {
			flowsNumber--;
			nodeNumber--;
			MessageManager.errorMessage( "You could not add a flow to another flow", true);
		}
	}
	
	/**
	 * Returns a definition of the specified flow type 
	 * @param flowNumber	 it's an int between 1 and N
	 * @return String like: "Flow 1 : 10 source(s)."
	 */		
	public String getSpecificFlowDefinition( int flowNumber ) {
		flowsNumber = getFlowTypeNumber();
		if( flowsNumber >= flowNumber && flowNumber> 0) {
			return mdt.getSpecificFlow(flowNumber-1);
		}
		else { throw MyExceptionDefinitions.FLOW_EXISTENCE; }
	}
	
	/**
	 * @param flowNumber	flow type number between 1 and nbDifferentFlow
	 * @param videoName a String like:	 "Video: Bean (Low Quality)"
	 */
	public void setSpecificVideoDefinition( int flowNumber, String videoName) {
			String name = videoName.split(": ",2)[1];
			setVideoObject( flowNumber-1, name);
	}
	
	
	/**
	 * Returns video description of the specified flow type
	 * @param flowNumber	it's an int between 1 and N 
	 * @return a String like:  "Video: Bean (Low Quality)"
	 */
	public String getSpecificVideoDefinition( int flowNumber ) {
		flowsNumber = getFlowTypeNumber();
		if( (flowsNumber >= flowNumber) && flowNumber> 0 ) {
			if( !videoValue.contains( new Integer(flowNumber) ) ) {
				System.out.println("MGInfComplexAggregatedFlowModelPanel.getSpecificVideoDefinition:  Error:  No video is defined for this flow: " +  flowNumber +" !");
				setVideoObject( flowNumber-1, DEFAULT_VIDEO_DESCRIPTION);
			}
			//System.out.println("Model definition: " + mdt.getChildDefinition( flowNumber-1, 0, modelValue));
			return mdt.getChildDefinition( flowNumber-1, 0);	
		}
		else { throw MyExceptionDefinitions.FLOW_EXISTENCE; }
	}
	
	/**
	 * Method setModelObject
	 * @param flowNumber it's an int between 0 and N
	 * @param videoName : String as: "Bean (Medium Quality)"
	 */
		public void setVideoObject( int flowNumber, String videoName) throws IllegalArgumentException {
			if( flowNumber>=0 && !( videoValue.contains(new Integer(flowNumber+1)) ) ) {
				mdt.addSpecificChild( flowNumber, "Video: " + videoName);
				videoValue.add( new Integer(flowNumber+1));
			}
			else {
				toolkit.beep();
				System.out.println("Flow number: " + flowNumber + " params: " + videoName);	 //DEBUG
				MessageManager.errorMessage( "You must specify only one model for a flow!!!", true);
			 }	
		}
	
	/**
	 * Computes the parameter number:
	 * 	flow type number
	 * 	+ for each flow type:
	 * 			- number of sources
	 * 			- a video definition
	 */
	protected void setModelParametersNumber() {
			this.modelParametersNumber =1; // pour le nb de flux different
			flowsNumber = getFlowTypeNumber();	// correspond au nombre de flots
			this.modelParametersNumber += flowsNumber*2; 
	}
	
	public int getParametersNumber() {
		setModelParametersNumber();
		return modelParametersNumber;	
	}
	
	public void freezeInfo(boolean b) {
		nb.setEnabled(!b);
		addNb.setEnabled(!b);
		removeNb.setEnabled(!b);
		clearFlow.setEnabled(!b);
	  	mginfcomplexmodelName.setEnabled(!b);
   }
	
   public void reset() {
		nb.setEnabled(true);
		addNb.setEnabled(true);
		removeNb.setEnabled(true);
		clearFlow.setEnabled(true);
		mginfcomplexmodelName.setEnabled(true);
		mginfcomplexmodelName.setSelectedIndex( 0 );
   }


	public boolean isAValidModel() {
		if (flowsNumber > 0)
			return true;
		else {
			JOptionPane.showMessageDialog(this, "Empty flow list. First add a flow before adding it to the scenario!","Empty flow list",JOptionPane.ERROR_MESSAGE);
			return false;
		}
	}
	
/* (non-Javadoc)
 * @see graphics2.panel.models.GenericModelPanel#buildNewXmlNode()
 */
	public Element buildNewXmlNode() {
		graphics2.ConfigurationXMLFile conftree = imp.getModelPanel().getClientPanel().getListScenario();
		org.w3c.dom.Element elem = conftree.createNewXMLElement("scenario_mginfaggregatedflow");
        
			// Add the model.
		org.w3c.dom.Element modelchild = conftree.createElement("model");
		modelchild.setAttribute("name", "MGInfComplexAggregatedFlow");
		modelchild.setAttribute("aggregated", "true");
		elem.appendChild(modelchild);
		
			// This is an aggregated model
			// iterate the list of models...
		for (int i=1; i <= flowsNumber ; i++) {
			org.w3c.dom.Element agreg = conftree.createElement("submodel");
			agreg.setAttribute("video", ""+getSpecificVideoDefinition(i).split(" ")[1]);
			agreg.setAttribute("source", ""+getFlowSourceNumber(i));
			modelchild.appendChild(agreg);
		}
		return elem;
	}

	public void updateWithXmlNode(org.w3c.dom.Element elt) {
		graphics2.ConfigurationXMLFile conftree = imp.getModelPanel().getClientPanel().getListScenario();

		if (! elt.getAttribute("name").equals("MGInfComplexAggregatedFlow")) {
			// Error
			System.out.println("Wrong model name: "+elt.getAttribute("name")+" in MGInfComplexAggregatedFlowPanel");	
		}
		else {
			// Erage the tree.
			mdt.clear();
			nodeNumber =0;
			flowsNumber = 0;

			org.w3c.dom.NodeList lnodes = elt.getElementsByTagName("submodel");
			for(int i=0; i <lnodes.getLength(); i++) {
					String videoType = (((org.w3c.dom.Element)lnodes.item(i)).getAttribute("video"));
					int sourceNb = Integer.parseInt (((org.w3c.dom.Element)lnodes.item(i)).getAttribute("source"));
					flowsNumber++;
					nodeNumber++;
					// Add the flow
					setSpecificFlowDefinition(nodeNumber, nodeNumber, sourceNb);
					// Add  the video type.
					setVideoObject( nodeNumber-1, videoType);
			}
		}
	}
}
