package graphics2.panel.client.model;

import graphics2.panel.client.*;
import multimedia.*;
import statistic.distribution.*;
import statistic.function.*;
import utils.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * MGInfComplexModelPanel
 *
 * @author mladoue
 * @version
 */
public class MGInfComplexModelPanel extends GenericModelPanel {
    protected MixedDistributionPanel[] mixedDistPanel = new MixedDistributionPanel[3];
    protected JLabel lambdaL = new JLabel("Lambda : ");
    protected JTextField lambdaF = new JTextField("1.0", 7);
    public MixedFunctionPanel mixedFunctionPanel;
    protected Hashtable listFunction;
    protected Hashtable listContinuousDistribution;
	protected JTabbedPane jTabbedpanel;
	
    public MGInfComplexModelPanel() {
        super();
        modelType = VIDEO;
        modelParametersNumber = 4; // minumum number of parameters
    }

    public MGInfComplexModelPanel(InterneModelPanel imp, int[] panelNb) {
        super(imp);
        modelType = VIDEO;
        modelParametersNumber = 4; // minumum number of parameters

        listFunction = imp.getListFunction();
        listContinuousDistribution = imp.getListContinuousDistribution();

		jTabbedpanel =new JTabbedPane(); 
        mixedDistPanel[0] = new MixedDistributionPanel("Amount of data per slot",
                listContinuousDistribution, panelNb[0]);
        mixedDistPanel[1] = new MixedDistributionPanel("Data distribution in slot",
                listContinuousDistribution, panelNb[1]);
        mixedDistPanel[2] = new MixedDistributionPanel("Packet size distribution",
                listContinuousDistribution, panelNb[2]);
		jTabbedpanel.add("Amount of data per slot",mixedDistPanel[0]);
		jTabbedpanel.add("Data distribution in slot",mixedDistPanel[1]);
		jTabbedpanel.add("Packet size distribution",mixedDistPanel[2]);
		
        mixedFunctionPanel = new MixedFunctionPanel(listFunction, panelNb[3]);

        this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        JPanel mginfComplexPanel = new JPanel();
        mginfComplexPanel.setLayout(new BoxLayout(mginfComplexPanel,
                BoxLayout.Y_AXIS));

        JPanel panel1 = new JPanel();
        panel1.setLayout(new BoxLayout(panel1, BoxLayout.X_AXIS));

		panel1.add(Box.createRigidArea(new Dimension(5, 0)));
		panel1.add(jTabbedpanel);
        panel1.add(Box.createRigidArea(new Dimension(5, 0)));
        panel1.add(MessageManager.createHelpMessageDialogButton(
                "Help window...to do"));
        panel1.add(Box.createRigidArea(new Dimension(5, 0)));

        JPanel panel2 = new JPanel();
        panel2.setLayout(new BoxLayout(panel2, BoxLayout.X_AXIS));

        JPanel panel21 = new JPanel();
        panel21.setLayout(new BoxLayout(panel21, BoxLayout.X_AXIS));

        panel21.add(lambdaL);
        panel21.add(Box.createRigidArea(new Dimension(10, 0)));
        panel21.add(lambdaF);
		lambdaF.setMaximumSize(new Dimension(lambdaF.getPreferredSize().width, 
																		lambdaF.getPreferredSize().height));
       panel21.add(Box.createRigidArea(new Dimension(10, 0)));

        panel2.add(mixedFunctionPanel);
        panel2.add(Box.createRigidArea(new Dimension(0, 5)));
        panel2.add(panel21);

        mginfComplexPanel.add(panel1);
        mginfComplexPanel.add(Box.createRigidArea(new Dimension(0, 5)));
        mginfComplexPanel.add(panel2);

        add(mginfComplexPanel);

        //Positionner le nb de parametre
        setModelParametersNumber();

        // Set the Border
        setBorder(new TitledBorder("M/G/Infinity Complex Model features"));
    }

    public void freezeInfo(boolean b) {
        for (int i = 0; i < 3; i++) {
            mixedDistPanel[i].freezeInfo(b);
        }

        mixedFunctionPanel.freezeInfo(b);
        lambdaF.setEditable(!b);
    }

    public void reset() {
        for (int i = 0; i < 3; i++) {
            mixedDistPanel[i].reset();
        }

        mixedFunctionPanel.reset();
        lambdaF.setText("");
    }

    public int getDistributionsNumber(int idx) {
        return mixedDistPanel[idx].getPanelNumber();
    }

    protected int getRealMixedDistributionNumber() {
        int result = 0;

        for (int i = 0; i < 3; i++) {
            if (mixedDistPanel[i].getPanelNumber() == 2) {
                result++;
            }
        }

        return result;
    }

    /**
     * Method getMixedDistributionPanel.
     * @param idx
     * @return MixedDistributionPanel 0: amount of data, 1: data distribution, 2: packet size distribution
     * @throws IllegalArgumentException
     */
    public MixedDistributionPanel getSpecifiedMixedDistributionPanel(int idx)
        throws IllegalArgumentException {
        return mixedDistPanel[idx];
    }

    public MixedDistributionPanel[] getMixedDistributionPanel() {
        return mixedDistPanel;
    }

    public void setMixedDistributionPanels(
        MixedDistributionPanel[] mixedDistPanel) {
        this.mixedDistPanel = mixedDistPanel;
    }

    public void setMixedDistributionPanel(int idx,
        MixedDistributionPanel mixedDistPanel) {
        this.mixedDistPanel[idx] = mixedDistPanel;
    }

    // Distribution name
    public void setDistributionNames(int panelNb, String[] names)
        throws IllegalArgumentException {
        mixedDistPanel[panelNb].setDistributionsNames(names);
    }

    public String getDistributionName(int panelNb, int distNb) {
        return mixedDistPanel[panelNb].getDistributionName(distNb);
    }

    public void setDistributionsName(String[] distName)
        throws IllegalArgumentException {
        for (int i = 0; i < 3; i++) {
            for (int j = 0; j < getRealMixedDistributionNumber(); j++) {
                mixedDistPanel[i].setDistributionName(j, distName[i]);
            }
        }
    }

    // Distribution weight
    public void setDistributionWeights(int panelNb, double[] weights) {
        mixedDistPanel[panelNb].setDistributionsWeight(weights);
    }

    public double[] getDistributionsWeight(int panelNb) {
        return mixedDistPanel[panelNb].getDistributionsWeight();
    }

    public double getDistributionWeight(int panelNb, int distNb)
        throws IllegalArgumentException {
        return mixedDistPanel[panelNb].getDistributionWeight(distNb);
    }

    // Distribution parameter
    public void setDistributionParameters(int panelNb, int distribNb,
        String[] paramsV) {
        mixedDistPanel[panelNb].setParams(distribNb, paramsV);
    }

    /**
     * Method getDistributionParameters. Return all the parameters of the distribution(s)
     * @param panelNb
     * @return String[]
     */
    public String[] getDistributionParameters(int panelNb) {
        return mixedDistPanel[panelNb].getParams();
    }

    // To put if it's a mixed distribution or not 
    public void setDistributionType(int idx, boolean type) {
        if (type) {
            mixedDistPanel[idx].setPanelNumber(2);
        } else {
            mixedDistPanel[idx].setPanelNumber(1);
        }
    }

    public boolean getDistributionType(int idx) {
        return mixedDistPanel[idx].isMixedDist;
    }

    public boolean[] getDistributionsTypes() {
        boolean[] ans = new boolean[3];

        for (int i = 0; i < 3; i++) {
            ans[i] = mixedDistPanel[i].isMixedDist;
        }

        return ans;
    }

    // Function
    public MixedFunction getMixedFunction() {
        return mixedFunctionPanel.getMixedFunction();
    }

//    public void setMixedFunction(MixedFunction mf) {
//        mixedFunctionPanel.setMixedFunction(mf);
//   }

    public GenericFunction getSpecificFunction(int idx) {
        return mixedFunctionPanel.getSpecificFunction(idx);
    }

    public void setSpecificFunction(int idx, GenericFunction gf) {
        mixedFunctionPanel.setSpecificFunction(idx, gf);
    }

    // Function panel
    public MixedFunctionPanel getMixedFunctionPanel() {
        return mixedFunctionPanel;
    }

    public void setMixedFunctionPanel(MixedFunctionPanel mfp) {
        this.mixedFunctionPanel = mfp;
    }

    // Function name
    public void setMixedFunctionName(int functionNb, String name) {
        mixedFunctionPanel.setFunctionName(functionNb, name);
    }

    public String getMixedFunctionName(int funcNb) {
        return mixedFunctionPanel.getFunctionName(funcNb);
    }

    // Function weight
    public void setMixedFunctionWeight(int panelNb, double weight) {
        mixedFunctionPanel.setWeight(panelNb, weight);
    }

    public double[] getMixedFunctionWeight() {
        return mixedFunctionPanel.getWeights();
    }

    // Function parameter
    public void setMixedFunctionParameter(int functionNb, double value) {
        mixedFunctionPanel.setFunctionParameter(functionNb, value);
    }

    public String getMixedFunctionParameter(int idx)
        throws IllegalArgumentException {
        return Double.toString(mixedFunctionPanel.getFunctionParameter(idx));
    }

    // Function type
    public void setIsMixedFunction(boolean value) {
        mixedFunctionPanel.setMixedFunction(value);
    }

    public boolean getIsMixedFunction() {
        return mixedFunctionPanel.isMixedFunction();
    }

    // Lambda
    public void setLambda(double value) {
        lambdaF.setText("" + value);
    }

    public double getLambda() {
        return Double.parseDouble(lambdaF.getText());
    }

    protected void setModelParametersNumber() {
        modelParametersNumber = 5; // mixedDist?x3 + mixedFunc? + lambda

        for (int i = 0; i < 3; i++) {
            if (mixedDistPanel[i].isMixedDist) {
                modelParametersNumber += 4; // +2 nom de distrib, + 2 valeurs de weight
            } else {
                modelParametersNumber += 2;
            }
             // + 1nom + 1 valeur de poids 

            for (int j = 0; j < mixedDistPanel[i].getPanelNumber(); j++) {
                modelParametersNumber += mixedDistPanel[i].getDistribution(j)
                                                          .getParamsCount(); // +nbParametre de chaque distrib
            }
        }

        if (mixedFunctionPanel.isMixedFunc) {
            modelParametersNumber += 6; // +2nom de func + 2 params de func + 2weight
        } else {
            modelParametersNumber += 3;
        }
    }

    public int getParametersNumber() {
        setModelParametersNumber();

        return modelParametersNumber;
    }

    // Methodes definies dans le cas d'un utilisation des MGInfModel pre-definis
    public void setDistributions(VideoDefinition vd) {
        GenericContinuousDistribution distributionTest = vd.getModel()
                                                           .getDistributionTest();

        if (!distributionTest.getName().equals("Mixed")) {
            mixedDistPanel[0].setDistribution(0, distributionTest);
        } else {
            mixedDistPanel[0].setDistributionsWeight(((MixedDistribution) distributionTest).getWeights());
            mixedDistPanel[0].setDistributions(((MixedDistribution) distributionTest).getSpecificDistributions());

            //System.out.println( SpecificObjectCreation.transformSTS(SpecificObjectCreation.transformDTST(((MixedDistribution)distributionTest).getDistributionsParameters())));
        }

        GenericContinuousDistribution distributionSlot = (GenericContinuousDistribution) vd.getModel()
                                                                                           .getDistributionSlot();

        if (!distributionSlot.getName().equals("Mixed")) {
            mixedDistPanel[1].setDistribution(0, distributionSlot);
        } else {
            mixedDistPanel[1].setDistributionsWeight(((MixedDistribution) distributionSlot).getWeights());
            mixedDistPanel[1].setDistributions(((MixedDistribution) distributionSlot).getSpecificDistributions());
        }

        GenericContinuousDistribution distributionPacketSize = (GenericContinuousDistribution) vd.getModel()
                                                                                                 .getDistributionPacketSize();

        if (!distributionPacketSize.getName().equals("Mixed")) {
            mixedDistPanel[2].setDistribution(0, distributionPacketSize);
        } else {
            mixedDistPanel[2].setDistributionsWeight(((MixedDistribution) distributionPacketSize).getWeights());
            mixedDistPanel[2].setDistributions(((MixedDistribution) distributionPacketSize).getSpecificDistributions());
        }
    }

    public void setMixedFunctionsName(String[] names) {
        mixedFunctionPanel.setFunctionsName(names);
    }

    public void setMixedFunctionsWeight(double[] weight) {
        mixedFunctionPanel.setWeights(weight);
    }

    public void setMixedFunctionsParameter(double[] values) {
        mixedFunctionPanel.setFunctionsParameter(values);
    }

    public void setMixedFunction(String[] names, double[] values,
        double[] weight) {
        mixedFunctionPanel.updateFunction(names, values);
        mixedFunctionPanel.setWeights(weight);
    }

    public void updateParam(VideoDefinition vd) {
        setDistributions(vd);

        if (mixedFunctionPanel.isMixedFunction()) {
            setMixedFunctionsName(MixedFunction.getNames(
                    ((MixedFunction) vd.getModel().getAutoCorr()).getSpecificFunctions()));
            setMixedFunctionsWeight(((MixedFunction) vd.getModel().getAutoCorr()).getWeight());
            setMixedFunctionsParameter(vd.getModel().getFunctionsParameter());
            setMixedFunction(MixedFunction.getNames(
                    ((MixedFunction) vd.getModel().getAutoCorr()).getSpecificFunctions()),
                vd.getModel().getFunctionsParameter(),
                ((MixedFunction) vd.getModel().getAutoCorr()).getWeight());
        } else {
            setMixedFunctionName(0, vd.getModel().getAutoCorr().getName());
            setMixedFunctionParameter(0, vd.getModel().getAutoCorr().getParam());
            setSpecificFunction(0, vd.getModel().getAutoCorr());
        }

        setLambda(vd.getModel().getLambda());

        //System.out.println(panelNb[0] + " "  + panelNb[1]  + " " + panelNb[2] + " " + panelNb[3] );
        //System.out.println( "Functions weight " + SpecificObjectCreation.transformSTS(SpecificObjectCreation.transformDTST(((MixedDistribution)vd.getModel().getDistributionTest()).getWeights()))); 
        //System.out.println( "FunctionsParameter: " + SpecificObjectCreation.transformSTS( TestParameters.transformDTST(vd.getModel().getFunctionsParameter())) );
        //System.out.println( "True FunctionsParameter: " + SpecificObjectCreation.transformSTS( TestParameters.transformDTST(getMixedFunctionPanel().getFunctionsParameter()) ));
    }

	public boolean isAValidModel() {
		if (isAFloat(this.lambdaF.getText(), "Lambda") &&
		mixedFunctionPanel.isAValidPanel()) {
			// Check the 3 distributions
			for (int j = 0; j < mixedDistPanel.length; j++) {
 				if (!mixedDistPanel[j].isAValidPanel())
 					return false;
			}
		}
		else
			return false;
		return true;
	}
    public org.w3c.dom.Element buildNewXmlNode() {
        // data values in the Panel should be ok
        // Access Configuration Tree.
        graphics2.ConfigurationXMLFile conftree = imp.getModelPanel()
                                                     .getClientPanel()
                                                     .getListScenario();
        org.w3c.dom.Element elem = conftree.createNewXMLElement(
                "scenario_mginfcomplex");

        // Add the model.
        org.w3c.dom.Element modelchild = conftree.createElement("model");
        modelchild.setAttribute("name", "MGInfComplex");
        elem.appendChild(modelchild);

        // Add the Function
        org.w3c.dom.Element fctschild = conftree.createElement("functions");

        int nbFunc = 1;

        if (getIsMixedFunction()) {
            nbFunc = 2;
        }

        for (int i = 0; i < nbFunc; i++) {
            org.w3c.dom.Element fctchild = conftree.createElement("function");
            fctchild.setAttribute("name", getMixedFunctionName(i));
            fctchild.setAttribute("param1", getMixedFunctionParameter(i));

            if (getIsMixedFunction()) {
                fctchild.setAttribute("weight",
                    String.valueOf(mixedFunctionPanel.getWeight(i)));
            }

            fctschild.appendChild(fctchild);
        }

        modelchild.appendChild(fctschild);

        // Add lambda1 value node to the model
        org.w3c.dom.Element lambdachild = conftree.createElement("lambda1");
        lambdachild.setAttribute("value", String.valueOf(getLambda()));
        modelchild.appendChild(lambdachild);

        // Add the 3 distributions...
        for (int j = 0; j < mixedDistPanel.length; j++) {
            org.w3c.dom.Element dischild = conftree.createElement(
                    "distribution" + (j + 1));

            if (getDistributionType(j)) { // Mixed distribution 
                dischild.setAttribute("mixed", "true");
            }

            dischild.setAttribute("name1", getDistributionName(j, 0));

            for (int i = 0;
                    i < mixedDistPanel[j].getPanel()[0].getParams().length;
                    i++)
                dischild.setAttribute("param1" + (i + 1),
                    mixedDistPanel[j].getPanel()[0].getParams()[i]);

            dischild.setAttribute("weight1",
                String.valueOf(getDistributionWeight(j, 0)));

            if (getDistributionType(j)) { // Mixed distribution 
                dischild.setAttribute("name2", getDistributionName(j, 1));

                for (int i = 0;
                        i < mixedDistPanel[j].getPanel()[1].getParams().length;
                        i++)
                    dischild.setAttribute("param2" + (i + 1),
                        mixedDistPanel[j].getPanel()[1].getParams()[i]);

                dischild.setAttribute("weight2",
                    String.valueOf(getDistributionWeight(j, 1)));
            }

            modelchild.appendChild(dischild);
        }

        return elem;
    }

    public void updateWithXmlNode(org.w3c.dom.Element elt) {
        org.w3c.dom.NodeList nlist = elt.getChildNodes();

        for (int i = 0; i < nlist.getLength(); i++) {
            String name = nlist.item(i).getNodeName();

            if (name.equals("functions")) {
                // Get all function nodes...
                org.w3c.dom.NodeList fctlist = nlist.item(i).getChildNodes();
                int cmp = 0;

                for (int j = 0; j < fctlist.getLength(); j++) {
                    if (fctlist.item(j).getNodeName().equals("function")) {
                        this.setMixedFunctionName(cmp,
                            ((org.w3c.dom.Element) fctlist.item(j)).getAttribute(
                                "name"));
                        this.setMixedFunctionParameter(cmp,
                            Double.valueOf(((org.w3c.dom.Element) fctlist.item(
                                    j)).getAttribute("param1")).doubleValue());

                        if (!((org.w3c.dom.Element) fctlist.item(j)).getAttribute(
                                    "weight").equals("")) {
                            this.setMixedFunctionWeight(cmp,
                                Double.valueOf(((org.w3c.dom.Element) fctlist.item(
                                        j)).getAttribute("weight")).doubleValue());
                        }

                        cmp++; // count the number of function						
                    }
                }

                if (cmp == 2) {
                    this.setIsMixedFunction(true);
                } else {
                    this.setIsMixedFunction(false);
                }
            } else if (name.equals("lambda1")) {
                setLambda(Double.valueOf(((org.w3c.dom.Element) nlist.item(i)).getAttribute(
                            "value")).doubleValue());
            } else if (name.equals("distribution1") ||
                    name.equals("distribution2") ||
                    name.equals("distribution3")) {
                int distnumber;

                if (name.equals("distribution1")) {
                    distnumber = 0;
                } else if (name.equals("distribution2")) {
                    distnumber = 1;
                } else {
                    distnumber = 2;
                }

                if (((org.w3c.dom.Element) nlist.item(i)).getAttribute("mixed")
                         .equals("true")) {
                    this.setDistributionType(distnumber, true);
                    getSpecifiedMixedDistributionPanel(distnumber)
                        .setDistributionName(1,
                        ((org.w3c.dom.Element) nlist.item(i)).getAttribute(
                            "name2"));
                    mixedDistPanel[distnumber].updateDistribution(1,
                        ((org.w3c.dom.Element) nlist.item(i)).getAttribute(
                            "name2"));

                    String par21 = ((org.w3c.dom.Element) nlist.item(i)).getAttribute(
                            "param21");
                    String par22 = ((org.w3c.dom.Element) nlist.item(i)).getAttribute(
                            "param22");
                    String[] param2;
                    System.out.println("--> " + par21 + " " + par22);

                    if (par22 != null) {
                        param2 = new String[2];
                        param2[0] = par21;
                        param2[1] = par22;
                    } else {
                        param2 = new String[1];
                        param2[0] = par21;
                    }

                    this.setDistributionParameters(distnumber, 1, param2);

                    mixedDistPanel[distnumber].setDistributionWeight(0,
                        Double.valueOf(((org.w3c.dom.Element) nlist.item(i)).getAttribute(
                                "weight1")).doubleValue());
                    mixedDistPanel[distnumber].setDistributionWeight(1,
                        Double.valueOf(((org.w3c.dom.Element) nlist.item(i)).getAttribute(
                                "weight2")).doubleValue());
                } else {
                    this.setDistributionType(distnumber, false);
                }

                getSpecifiedMixedDistributionPanel(distnumber)
                    .setDistributionName(0,
                    ((org.w3c.dom.Element) nlist.item(i)).getAttribute("name1"));
                mixedDistPanel[distnumber].updateDistribution(0,
                    ((org.w3c.dom.Element) nlist.item(i)).getAttribute("name1"));

                String par11 = ((org.w3c.dom.Element) nlist.item(i)).getAttribute(
                        "param11");
                String par12 = ((org.w3c.dom.Element) nlist.item(i)).getAttribute(
                        "param12");
                String[] param;

                if (par12 != null) {
                    param = new String[2];
                    param[0] = par11;
                    param[1] = par12;
                } else {
                    param = new String[1];
                    param[0] = par11;
                }

                this.setDistributionParameters(distnumber, 0, param);
            }
        }
    }
}
