package graphics2.panel.experiment;

import graphics2.*;
import graphics2.info.*;
import utils.MessageManager;
import java.awt.*;
import java.awt.event.*;

/**
 * ExperimentPanel : interface graphique de l'onglet concernant: experiment configuration
 *
 * @author mladoue
 * @version 0.1, 20/01/03
 */
import javax.swing.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
// Pour la sortie.
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult; 
import java.io.*;


public class ExperimentPanel extends JPanel implements ExperimentInfo {
    // JFrame auquel sera attache le JPanel
    protected GestionTest gestionTest;
    protected MachinesPanel mlpServer;
    protected MachinesPanel mlpClient;
    protected MachinesTreePanel mtp;

    protected JTextField name;
    protected JTextField ipAddress;
    protected JTextField srcPort;
    protected JTextField resultDirectory;
    protected JLabel nameL;
    protected JLabel ipL;
    protected JLabel srcPortL;
    private JButton clear = new JButton("Clear");
    private JButton save = new JButton("Save");

    // Store informations for the hostinfo.xml file.
    org.w3c.dom.Document document;

    //Create a file chooser
    final JFileChooser fc = new JFileChooser();

    public ExperimentPanel(GestionTest gestionTest) {
        this.gestionTest = gestionTest;

        // Construct the XML Tree.
        try {
            DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
            document = docBuilder.newDocument();
        } catch (Throwable t) {
            t.printStackTrace();
        }
        org.w3c.dom.Element configelt = document.createElement("Configuration");
        document.appendChild(configelt);
		org.w3c.dom.Element servers = document.createElement("Servers");
        configelt.appendChild(servers);

        this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        mtp = new MachinesTreePanel(servers, "Network description");

	        // define server panel
        mlpServer = new MachinesPanel(this, mtp, true);
 
        	// define client panel
        mlpClient = new MachinesPanel(this, mtp, false);

        JPanel ePanel = new JPanel();
        ePanel.setLayout(new BoxLayout(ePanel, BoxLayout.X_AXIS));

        // Partie representant server, client/scenario conf
        JPanel panel1 = new JPanel();
        panel1.setLayout(new BoxLayout(panel1, BoxLayout.Y_AXIS));

        panel1.add(mlpServer);
        panel1.add(Box.createRigidArea(new Dimension(0, 5)));
        panel1.add(mlpClient);
        panel1.add(Box.createRigidArea(new Dimension(0, 5)));

        // Partie representant l'arbre graphique		
        JPanel panel2 = new JPanel();
        panel2.setLayout(new BoxLayout(panel2, BoxLayout.Y_AXIS));

        JPanel panel21 = new JPanel();
        panel21.setLayout(new BoxLayout(panel21, BoxLayout.X_AXIS));

        panel21.add(Box.createRigidArea(new Dimension(5, 0)));
        panel21.add(clear);
        panel21.add(Box.createRigidArea(new Dimension(5, 0)));
        panel21.add(save);
        panel21.add(Box.createRigidArea(new Dimension(10, 0)));
        panel21.add(MessageManager.createHelpMessageDialogButton(
                "Help window...to do"));
        panel21.add(Box.createRigidArea(new Dimension(5, 0)));

        panel2.add(mtp);
        panel2.add(Box.createRigidArea(new Dimension(0, 5)));
        panel2.add(panel21);
        panel2.add(Box.createRigidArea(new Dimension(0, 5)));

        ePanel.add(panel1);
        ePanel.add(Box.createRigidArea(new Dimension(5, 0)));
        ePanel.add(panel2);

        makeButtons();

        JScrollPane scroller = new JScrollPane(ePanel);
        this.add(scroller);
    }

	public org.w3c.dom.Document getDocument() {
		return document;
	}

    public void makeButtons() {
        // clear button
        clear.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    mtp.clear();
                }
            });

        // save button
        save.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
					saveFile("/tmp/toto.xml");
                }
            });
    }

	public void openFile(String filename) {
		// Parse the xml file only if filname !=NULL and it exists.
		try {
			if (filename.length()!=0 && utils.FileTest.isExistent(filename)) {
				DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
				DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
				document = docBuilder.parse (new File(filename));
                
					// normalize text representation
				document.getDocumentElement ().normalize ();
				
					// Update the XML and JTree trees in MachinesTreePanel
				mtp.updateXMLTree( (org.w3c.dom.Element)(document.getElementsByTagName("Servers")).item( 0));
			}
		}
		catch (Throwable t) {
			t.printStackTrace();
		}
	}
	
	/**
	 * check that all scenario files are found on disk.
	 * @return true if all the files were found.
	 */
	public boolean allFileExist() {
		org.w3c.dom.NodeList clients = document.getElementsByTagName("Client");
		for(int i=0; i < clients.getLength(); i++) {
			org.w3c.dom.NamedNodeMap attr = clients.item(i).getAttributes();
			String filename= attr.getNamedItem("scfile").getNodeValue();
			if (!utils.FileTest.isExistent(filename)) {
				System.err.println("Erreur, unable to find file "+filename);
				return false;
			}
		}
		return true;
	}

	public boolean saveFile(String filename) {
		try {
				 // Use a Transformer for output
			 TransformerFactory tFactory = TransformerFactory.newInstance();
			 Transformer transformer = tFactory.newTransformer();
            
			 DOMSource source = new DOMSource(document);
			 File xmloutputfile = new File(filename);
			 StreamResult result = new StreamResult(xmloutputfile);
			 transformer.transform(source, result);
			 return true;
		 } 
		 catch (TransformerConfigurationException tce) {
				 // Error generated by the parser
			 System.out.println ("\n** Transformer Factory error");
			 System.out.println("   " + tce.getMessage() );
            
				 // Use the contained exception, if any
			 Throwable x = tce;
			 if (tce.getException() != null)
				 x = tce.getException();
			 x.printStackTrace();
			 return false;
		 } 
		 catch (TransformerException te) {
				 // Error generated by the parser
			 System.out.println ("\n** Transformation error");
			 System.out.println("   " + te.getMessage() );
            
				 // Use the contained exception, if any
			 Throwable x = te;
			 if (te.getException() != null)
				 x = te.getException();
			 x.printStackTrace();
			 return false;
		 }
	}
	
    // Getters of panels associated panels of experimSent panel ;o)
    public GestionTest getGestionPanel() {
        return gestionTest;
    }

    public MachinesPanel getMachinesPanel(boolean server) {
        if (server) {
            return mlpServer;
        } else {
            return mlpClient;
        }
    }

	/**
	 *  For the moment, only check if there is at least two nodes in the tree.
	 * TODO: improve this checking to verify that each server has, at least, on client.
	 */
	public boolean hasMachineDescription() {
		if (mtp.getTree().getRowCount() >=3)
			return true;
		else
			return false;
	}
	
    public void freezeInfo(boolean b) {
        mlpClient.freezeInfo(b);
        mlpServer.freezeInfo(b);
        mtp.freezeInfo(b);
        clear.setEnabled(!b);
        save.setEnabled(!b);
    }

    public void reset() {
        mlpClient.reset();
        mlpServer.reset();
        mtp.reset();
        clear.setEnabled(true);
        save.setEnabled(true);
    }

    public void setClientName(String clientName) {
        name.setText(clientName);
    }

    public void setClientPort(int port) {
        srcPort.setText("" + port);
    }

    public String getClientName() {
        return name.getText();
    }

    public int getClientPort() {
        return Integer.parseInt(srcPort.getText());
    }

    /**
      * Methode main destine a tester la classe
      */
    public static void main(String[] args) {
        GestionTest gt = new GestionTest();
        ExperimentPanel ep = new ExperimentPanel(gt);
    }
}
