package graphics2.panel.experiment;

import utils.MessageManager;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.tree.DefaultMutableTreeNode;


/**
 * This is used to defined common parameters of server and client configuration part
 * This includes a DescriptionPanel which is function of the case: if it's a server or a client
 *
 * @author mladoue
 * @version 0.1, Jun 13, 2003
 */
public class MachinesPanel extends JPanel {
    protected ExperimentPanel ep;
    protected MachinesTreePanel mtp;
    protected DescriptionPanel descriptionPanel;
    protected JButton addMachine = new JButton("Add machine");
    protected JButton removeMachine = new JButton("Remove machine");
    protected JButton reset = new JButton("Reset");
    protected String title;
    protected boolean server;

    /**
     * @param ep
     * @param mtp
     * @param server
     */
    public MachinesPanel(ExperimentPanel ep, MachinesTreePanel mtp,
        boolean server) {
        this.ep = ep;
        this.mtp = mtp;
        this.server = server;

        if (server) {
            title = "Server machine specification";
            descriptionPanel = new ServerDescriptionPanel(this);
        } else {
            title = "Client machine specification";
            descriptionPanel = new ClientDescriptionPanel(this);
        }

        JPanel mlPanel = new JPanel();
        mlPanel.setLayout(new BoxLayout(mlPanel, BoxLayout.X_AXIS));

        JPanel panel1 = new JPanel();
        panel1.setLayout(new BoxLayout(panel1, BoxLayout.Y_AXIS));

        JPanel panel12 = new JPanel();
        panel12.setLayout(new BoxLayout(panel12, BoxLayout.X_AXIS));

        panel12.add(descriptionPanel);

        JPanel panel13 = new JPanel();
        panel13.setLayout(new BoxLayout(panel13, BoxLayout.X_AXIS));

        panel13.add(Box.createRigidArea(new Dimension(30, 0)));
        panel13.add(addMachine);
        panel13.add(Box.createRigidArea(new Dimension(20, 0)));
        panel13.add(removeMachine);
        panel13.add(Box.createRigidArea(new Dimension(20, 0)));
        panel13.add(reset);
        panel13.add(Box.createRigidArea(new Dimension(30, 0)));

        panel1.add(Box.createRigidArea(new Dimension(0, 5)));
        panel1.add(panel12);
        panel1.add(Box.createRigidArea(new Dimension(0, 5)));
        panel1.add(panel13);

        mlPanel.add(panel1);

        makeButtons();

        add(mlPanel);
        this.setBorder(new TitledBorder(title));
    }

    public void makeButtons() {
        addMachine.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    String machineD = "";
					org.w3c.dom.Element newelt=null;
                    if (descriptionPanel.machine.getText() != "") {
                        if (server) {
								//Create XML Node for the server
							newelt = ep.getDocument().createElement("Server");
							newelt.setAttribute( "name", descriptionPanel.machine.getText());
							newelt.setAttribute( "ipadr",descriptionPanel.ipaddress.getText() );
							newelt.setAttribute( "port", descriptionPanel.port.getText());
							newelt.setAttribute( "accesstype", descriptionPanel.accessType.getSelectedItem().toString());
							if (descriptionPanel.ipv6.isSelected())
								newelt.setAttribute( "ipprot", "ipv6");
							else
								newelt.setAttribute( "ipprot", "ipv4");
							// TODO: javacmd attribute is missing. Hardcoded
							newelt.setAttribute( "javacmd","java");                       
                        } else {
								// Create XML Node for the client
							newelt = ep.getDocument().createElement("Client");
							newelt.setAttribute( "name", descriptionPanel.machine.getText());
							newelt.setAttribute( "port", descriptionPanel.port.getText());
							newelt.setAttribute( "scfile", descriptionPanel.scenarioName.getSelectedItem().toString());
							newelt.setAttribute( "accesstype", descriptionPanel.accessType.getSelectedItem().toString());
							if (descriptionPanel.cosC.isSelected())
								newelt.setAttribute("cos", descriptionPanel.cos.getText()); //TODO: A voir la valeur, l'IHM semble plus complete.
							else
								newelt.setAttribute("cos","0");
							// TODO: javacmd attribute is missing. Hardcoded
							newelt.setAttribute( "javacmd","java");                       
                         }
                        mtp.addMachine(newelt, server);
                    }
                }
            });

        removeMachine.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) mtp.getTree()
                                                                                      .getLastSelectedPathComponent();

                    if ((selectedNode != null) &&
                            (selectedNode.getLevel() < 3)) {
                        mtp.removeCurrentNode();
                    } else {
                        MessageManager.errorMessage("You don't have selected a server or a client definition!",
                            true);
                    }
                }
            });

        reset.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    reset();
                }
            });
    }

    public DescriptionPanel getDescriptionPanel() {
        return descriptionPanel;
    }

    public void freezeInfo(boolean b) {
		descriptionPanel.machine.setEnabled(!b);
        descriptionPanel.freezeInfo(b);
		descriptionPanel.port.setEnabled(!b);
        addMachine.setEnabled(!b);
        removeMachine.setEnabled(!b);
        reset.setEnabled(!b);
    }

    public void reset() {
		descriptionPanel.machine.setText("");
		descriptionPanel.machine.setEnabled(true);
        descriptionPanel.reset();
		descriptionPanel.port.setText("5000");
		descriptionPanel.port.setEnabled(true);
        addMachine.setEnabled(true);
        removeMachine.setEnabled(true);
        reset.setEnabled(true);
    }
}
