package graphics2.panel.experiment;

import utils.MessageManager;
import utils.MyDynamicTree;
import utils.definitions.ImageList;
import java.awt.Component;
import javax.swing.*;
import javax.swing.tree.*;


/**
 * Used in Experiment panel to describe relations between server/client/scenario-file
 * @author mladoue
 * @version 0.1, Jun 13, 2003
 */
public class MachinesTreePanel extends MyDynamicTree {
    protected MyComputerRenderer mr = new MyComputerRenderer();
	org.w3c.dom.Element servers;
		
    public MachinesTreePanel(org.w3c.dom.Element servers, String rootTitle) {
        super(rootTitle);
		this.servers=servers;
		
        tree.setEditable(true);
        tree.setRootVisible(true);

        //	Set the icon for leaf nodes
        tree.setCellRenderer(mr);
    }

	void updateXMLTree(org.w3c.dom.Element servers) {
		this.servers=servers;
		org.w3c.dom.NodeList nl= servers.getElementsByTagName("Server");
		for(int i=0;i<nl.getLength();i++) {
				// Display them in the tree.
			org.w3c.dom.Element server = (org.w3c.dom.Element)(nl.item(i));
			DefaultMutableTreeNode machine = new DefaultMutableTreeNode(new MachineDescription(server));
			treeModel.insertNodeInto(machine, rootNode, rootNode.getChildCount());
				
				// Get the clients
			org.w3c.dom.NodeList clist= server.getElementsByTagName("Client");
			for(int j=0;j<clist.getLength();j++) {
					// Display them in the tree.
				org.w3c.dom.Element client = (org.w3c.dom.Element)(clist.item(j));
				DefaultMutableTreeNode clientmachine = new DefaultMutableTreeNode(new MachineDescription(client));
				treeModel.insertNodeInto(clientmachine, machine,machine.getChildCount());
			}
		}
	}

		
   /**
     *  To add in the JTree a machine with its description
     * XML tree is not altered.
     * @param child
     * @param server if it's a server or not
     * @return int 0 if ok and 1 otherwise
     */
    public int addMachine(org.w3c.dom.Element elt, boolean server) {
        DefaultMutableTreeNode parentNode = null;
        TreePath parentPath = tree.getSelectionPath();

		if (parentPath == null) { // si pas de selection
		   tree.setSelectionRow(0);
		   parentPath=tree.getSelectionPath();
		   if (parentPath==null)
			   System.out.println("PB");
		}
       
        if (server) {
                parentNode = rootNode;
            } else {
                if (parentPath.getLastPathComponent() == rootNode) {
                    MessageManager.errorMessage("You have to add a client to a server",
                        true);

                    return 1;
                } else {
                    parentNode = (DefaultMutableTreeNode) (parentPath.getLastPathComponent());

                    if (parentNode.getLevel() != 1) { // distance au noeud parent a root => doit etre de 1
                        MessageManager.errorMessage("You can not add a client to another client!",
                            true);

                        return 1;
                    }
                }
            }
			System.out.println("on ajoute...");
            DefaultMutableTreeNode machine = null;
            MachineDescription machineD = new MachineDescription(elt);

            machine = new DefaultMutableTreeNode(machineD);
            treeModel.insertNodeInto(machine, parentNode, parentNode.getChildCount());

			if (parentNode.getLevel() == 1) {
				// a client... get XML parent node...
				org.w3c.dom.Element parentxml = ((MachineDescription)parentNode.getUserObject()).elt;
				parentxml.appendChild(elt);				
			}
			else if (parentNode.getLevel()==0) {
				servers.appendChild(elt);
			}
			else {
				System.err.println("Not sure we should be here...");
			}
	
            //	Set the icon for some leaf nodes
            tree.setCellRenderer(mr);

            //	now display new node
            TreeNode[] nodes = treeModel.getPathToRoot(machine);
            TreePath path = new TreePath(nodes);
            tree.scrollPathToVisible(path);
            machine.setAllowsChildren(true);

            return 0;
       
    }

	/* Overloaded  function to remove also the XML node
	 * 
	 * @see utils.MyDynamicTree#removeCurrentNode()
	 */
	public int removeCurrentNode() {
			// First, delete the xml node...
		DefaultMutableTreeNode selectedNode =  (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();
		org.w3c.dom.Element nodetoremove = ((MachineDescription)selectedNode.getUserObject()).elt;
		System.out.println(nodetoremove);
			// Find its parent.
		nodetoremove.getParentNode().removeChild(nodetoremove);
		return super.removeCurrentNode();
	}
	
	/* Overloaded  function to remove all the child of Servers XML node.
	 *  (non-Javadoc)
	 * @see utils.MyDynamicTree#clear()
	 */
	public void clear() {
		org.w3c.dom.NodeList nl = servers.getChildNodes();
		while (servers.hasChildNodes()) {
			servers.removeChild(servers.getFirstChild());
		}
		super.clear();
	}

    /**
     * Permits to add a flow definition from FlowXmlTreePanel (in ExperimentPanel) in MachinesTreePanel
     * @param child
     * @return
     * int
     */
    public int addFlow(Object child) { //TODO: Verifier si toujours d'actualite...

        DefaultMutableTreeNode parentNode = null;
        DefaultMutableTreeNode flow = null;

        TreePath parentPath = tree.getSelectionPath();

        if (parentPath == null) { // si pas de selection

            return 1;
        } else {
            parentNode = (DefaultMutableTreeNode) (parentPath.getLastPathComponent());

            // on ne peut ajouter un flow qu'a un client
            if (parentNode.getLevel() != 2) {
                MessageManager.errorMessage("You could only add a flow to a client",
                    true);

                return 1;
            }
        }

        String flowD = child.toString();
        flow = new DefaultMutableTreeNode(flowD);

        treeModel.insertNodeInto(flow, parentNode, parentNode.getChildCount());

        //	Set the icon for some leaf nodes
        //tree.setCellRenderer( mr );
        //	now display new node
        TreeNode[] nodes = treeModel.getPathToRoot(flow);
        TreePath path = new TreePath(nodes);
        tree.scrollPathToVisible(path);
        flow.setAllowsChildren(false);

        return 0;
    }

    public void freezeInfo(boolean b) {
        tree.setEnabled(!b);
    }

    public void reset() {
        tree.setEnabled(true);
        this.clear();
    }

    private class MyComputerRenderer extends DefaultTreeCellRenderer {
        ImageIcon computerIcon;

        public MyComputerRenderer() {
            computerIcon = ImageList.clientIcon;
        }

        public Component getTreeCellRendererComponent(JTree tree, Object value,
            boolean sel, boolean expanded, boolean leaf, int row,
            boolean hasFocus) {
            super.getTreeCellRendererComponent(tree, value, sel, expanded,
                leaf, row, hasFocus);

            if (value == rootNode) {
                setIcon(ImageList.computerListIcon);
                setToolTipText(
                    "This is a description of relations between servers<=>clients<=>scenarii");
            } else if (isComputer(value)) {
                setIcon(computerIcon);
                setToolTipText("This is a computer name");
            } else {
                setToolTipText(null);
            }
            return this;
        }

        protected boolean isComputer(Object value) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;

            if ((node != null) && (node != rootNode) && (node.getLevel() < 3)) {
//                MachineDescription nodeInfo = (MachineDescription) (node.getUserObject());
 //               String title2 = nodeInfo.machineDescription;

                return true;
            }
            return false;
        }
    }
}
