/**
 * VideoExample : Definition de plusieurs types de video standards: films, dessins animes...
 * 
 * @author maria ladoue
 * @version 0.1, 06/11/02 
 */

package multimedia;

import statistic.distribution.*;
import statistic.function.*;
import multimedia.model.MGInfModel;
import multimedia.codec.*;

public interface VideoExample extends MovieExample { 
	
    public final GenericDistribution dSlot = new ConstantDistribution(1d);
    public final GenericDistribution dPacketSize = new ConstantDistribution(1000d);
}  


/**
 * MovieExample Examples from statistical analysis
 *  Film duration: 60 minutes,
 *  Video codec: MPEG4 => slot duration = 0.48 sec
 * 
 * <a href="http://www-tkn.ee.tu-berlin.de/research/trace/stat.html">web site</a>
 */

interface MovieExample {
 
    /**
     * Characteristics of JurassicPark- low quality
     *
     *<center><table BORDER NOSAVE><tr><td width="40%">Frame statistics</td>
     * <td width="20%"></td><td width="40%"></td></tr><tr> <td> compression ratio </td>
     * <td> YUV:MP4</td><td ALIGN=RIGHT NOSAVE>49.46</td></tr>
     * <tr> <td>File Size</td><td>byte</td><td ALIGN=RIGHT NOSAVE>6.9e+07</td></tr>
     * <tr> <td> Video run time</td><td> msec </td><td ALIGN=RIGHT NOSAVE>3.6e+06</td></tr>
     * <tr> <td> # of Frames </td><td>-</td><td ALIGN=RIGHT NOSAVE>89998</td></tr>
     * <tr> <td> mean frame size</td><td> byte </td><td ALIGN=RIGHT NOSAVE>7.7e+02</td></tr>
     * <tr> <td> var frame size</td><td>-</td><td ALIGN=RIGHT NOSAVE>1.1e+06</td></tr>
     * <tr> <td> CoV of frame size</td><td>-</td><td ALIGN=RIGHT NOSAVE>1.39</td></tr>
     * <tr> <td> min frame size</td><td>byte</td><td ALIGN=RIGHT NOSAVE>26</td></tr>
     * <tr> <td> max frame size</td><td>byte</td><td ALIGN=RIGHT NOSAVE> 8154</td></tr>
     * <tr> <td> Mean bit rate</td><td> bit/sec </td><td ALIGN=RIGHT NOSAVE>1.5e+05</td></tr>
     * <tr> <td> Peak bit rate</td><td> bit/sec </td><td ALIGN=RIGHT NOSAVE>1.6e+06</td></tr>
     * <tr> <td> Peak/Mean of bit rate</td><td>-</td><td ALIGN=RIGHT NOSAVE>10.61</td></tr></table>
     * </center><br><br> <center><table BORDER NOSAVE><tr><td width="40%">GoP statistics</td><td width="20%"></td><td width="40%"></td></tr>
     * <tr> <td> # of GoPs </td><td>-</td><td ALIGN=RIGHT NOSAVE>7499</td></tr>
     * <tr> <td> mean GoP size</td><td> byte </td><td ALIGN=RIGHT NOSAVE>9.2e+03</td></tr>
     * <tr> <td> var GoP size</td><td>-</td><td ALIGN=RIGHT NOSAVE>2.4e+07</td></tr>
     * <tr> <td> CoV of GoP size</td><td>-</td><td ALIGN=RIGHT NOSAVE>0.53</td></tr>
     * <tr> <td> min GoP size</td><td>byte</td><td ALIGN=RIGHT NOSAVE>839</td></tr>
     * <tr> <td> max GoP size</td><td>byte</td><td ALIGN=RIGHT NOSAVE> 37342</td></tr>
     * <tr> <td> Mean bit rate</td><td> bit/sec </td><td ALIGN=RIGHT NOSAVE>1.5e+05</td></tr>
     * <tr> <td> Peak bit rate</td><td> bit/sec </td><td ALIGN=RIGHT NOSAVE>6.2e+05</td></tr>
     * <tr> <td> Peak/Mean of bit rate</td><td>-</td><td ALIGN=RIGHT NOSAVE>4.05</td></tr></table></center>
     */
 // Low quality       
double[] coeffFuncjpl = { 0.443, 0.557};
GenericFunction[] gftjpl = { new MarkovianFunction(0.02), new SRDFunction(0.333)};
GenericFunction gfjpl = new MixedFunction(gftjpl ,coeffFuncjpl  );
MGInfModel mginfjpl = new MGInfModel(gfjpl , new LogNormalDistribution(9.0002, 0.5127),VideoExample.dSlot , VideoExample.dPacketSize);

final VideoDefinition JurassicParkLow = new VideoDefinition ( mginfjpl ,VideoCodecExample.MPEG4  , 60 , "Jurassic Park (Low Quality)");
//  final VideoDefinition JurassicParkLow = new VideoDefinition ( mginfjpl ,VideoCodecExample.MPEG4  , 1 , "Jurassic Park (Low Quality)");

    
// Medium quality    
double[] coeffFuncjpm = { 0.3343, 0.6657 };
GenericFunction[] gftjpm = { new MarkovianFunction(0.0207), new SRDFunction(0.269) };
double[] coeffDistjpm = { 0.0413, 0.9587 };
GenericContinuousDistribution[] gdjpm = { new GammaDistribution(19.1288,1499.9), new LogNormalDistribution(9.5081,0.5425)};
GenericContinuousDistribution djpm = new MixedDistribution(gdjpm,coeffDistjpm);
MGInfModel mginfjpm = new MGInfModel(gftjpm,coeffFuncjpm, 1.0 , djpm, VideoExample.dSlot , VideoExample.dPacketSize);

final VideoDefinition JurassicParkMedium = new VideoDefinition( mginfjpm, VideoCodecExample.MPEG4, 60, "Jurassic Park (Medium Quality)");

// High quality
double[] coeffFuncjph = { 0.221, 0.779 };
GenericFunction[] gftjph = { new MarkovianFunction(0.026), new SRDFunction(0.218) };
double[] coeffDistjph = { 0.286, 0.714 };
GenericContinuousDistribution[] gdjph = { new GammaDistribution(5.879,9484.9), new LogNormalDistribution(10.547,0.461)};
GenericContinuousDistribution djph = new MixedDistribution(gdjph,coeffDistjph);
MGInfModel mginfjph = new MGInfModel(gftjph, coeffFuncjph,1.0,djph, VideoExample.dSlot, VideoExample.dPacketSize);

final VideoDefinition JurassicParkHigh = new VideoDefinition( mginfjph, VideoCodecExample.MPEG4, 60, "Jurassic Park (High Quality)");
//final VideoDefinition JurassicParkHigh = new VideoDefinition( mginfjph, VideoCodecExample.MPEG4, 30, "Jurassic Park (High Quality)");

 /**
  * Characteristics of BEAN 
  */

// Low quality
double[] coeffFuncbl = { 0.489, 0.511};
GenericFunction[] gftbl = { new MarkovianFunction(0.143), new SRDFunction(0.126)};
MGInfModel mginfbl = new MGInfModel(gftbl, coeffFuncbl, 1.0 , new LogNormalDistribution(8.74, 0.4274),VideoExample.dSlot , VideoExample.dPacketSize);

final VideoDefinition BeanLow = new VideoDefinition ( mginfbl ,VideoCodecExample.MPEG4  , 60 , "Bean (Low Quality)");
//final VideoDefinition BeanLow = new VideoDefinition ( mginfbl ,VideoCodecExample.MPEG4, 30, "Bean (Low Quality)");



// Medium quality
double[] coeffFuncbm = { 0.229, 0.771 };
GenericFunction[] gftbm = { new MarkovianFunction(0.005), new SRDFunction(0.655) };
double[] coeffDistbm = { 0.083, 0.917 };
GenericContinuousDistribution[] gdbm = { new GammaDistribution(21.387,501.472), new LogNormalDistribution(9.194,0.483) };
GenericContinuousDistribution dbm = new MixedDistribution(gdbm, coeffDistbm);
MGInfModel mginfbm = new MGInfModel(gftbm, coeffFuncbm, 1.0 ,dbm ,VideoExample.dSlot , VideoExample.dPacketSize);

final VideoDefinition BeanMedium = new VideoDefinition ( mginfbm ,VideoCodecExample.MPEG4  , 60 , "Bean (Medium Quality)");
//final VideoDefinition BeanMedium = new VideoDefinition ( mginfbm ,VideoCodecExample.MPEG4  , 30, "Bean (Medium Quality)");



// High quality
double[] coeffFuncbh = { 0.451, 0.549 };
GenericFunction[] gftbh = { new MarkovianFunction(0.129), new SRDFunction(0.108) };
GenericFunction gfbh = new MixedFunction(gftbh ,coeffFuncbh);
double[] coeffDistbh = { 0.309, 0.691 };
GenericContinuousDistribution[] gdbh = { new GammaDistribution(18.212,1998), new LogNormalDistribution(10.347,0.441) };
GenericContinuousDistribution dbh = new MixedDistribution(gdbh, coeffDistbh);
MGInfModel mginfbh = new MGInfModel(gftbh, coeffFuncbh, 1.0,dbh ,VideoExample.dSlot , VideoExample.dPacketSize);

final VideoDefinition BeanHigh = new VideoDefinition ( mginfbh ,VideoCodecExample.MPEG4  , 60 , "Bean (High Quality)");


/**
  * Characteristics of
  */

}

interface CartoonsExample {
		
	/**
  	* Characteristics of Aladdin
   */
	
	 /**
     * Characteristics of aladdin- low quality
     * <table BORDER NOSAVE><tr><td width="40%">Frame statistics</td><td width="20%"></td><td width="40%"></td></tr>
     * <tr> <td> compression ratio </td><td> YUV:MP4</td><td ALIGN=RIGHT NOSAVE>75.53</td></tr>
     * <tr> <td>File Size</td><td>byte</td><td ALIGN=RIGHT NOSAVE>4.5e+07</td></tr>
     * <tr> <td> Video run time</td><td> msec </td><td ALIGN=RIGHT NOSAVE>3.6e+06</td></tr>
     * <tr> <td> # of Frames </td><td>-</td><td ALIGN=RIGHT NOSAVE>89998</td></tr>
     * <tr> <td> mean frame size</td><td> byte </td><td ALIGN=RIGHT NOSAVE>5e+02</td></tr>
     * <tr> <td> var frame size</td><td>-</td><td ALIGN=RIGHT NOSAVE>4.6e+05</td></tr>
     * <tr> <td> CoV of frame size</td><td>-</td><td ALIGN=RIGHT NOSAVE>1.35</td></tr>
     * <tr> <td> min frame size</td><td>byte</td><td ALIGN=RIGHT NOSAVE>26</td></tr>
     * <tr> <td> max frame size</td><td>byte</td><td ALIGN=RIGHT NOSAVE> 6241</td></tr>
     * <tr> <td> Mean bit rate</td><td> bit/sec </td><td ALIGN=RIGHT NOSAVE>1e+05</td></tr>
     * <tr> <td> Peak bit rate</td><td> bit/sec </td><td ALIGN=RIGHT NOSAVE>1.2e+06</td></tr>
     * <tr> <td> Peak/Mean of bit rate</td><td>-</td><td ALIGN=RIGHT NOSAVE>12.40</td></tr></table>
     * <table BORDER NOSAVE><tr><td width="40%">GoP statistics</td><td width="20%"></td><td width="40%"></td></tr>
     * <tr> <td> # of GoPs </td><td>-</td><td ALIGN=RIGHT NOSAVE>7499</td></tr><tr>
     *  <td> mean GoP size</td><td> byte </td><td ALIGN=RIGHT NOSAVE>6e+03</td></tr>
     * <tr> <td> var GoP size</td><td>-</td><td ALIGN=RIGHT NOSAVE>1.1e+07</td></tr>
     * <tr> <td> CoV of GoP size</td><td>-</td><td ALIGN=RIGHT NOSAVE>0.55</td></tr>
     * <tr> <td> min GoP size</td><td>byte</td><td ALIGN=RIGHT NOSAVE>740</td></tr>
     * <tr> <td> max GoP size</td><td>byte</td><td ALIGN=RIGHT NOSAVE> 29506</td></tr>
     * <tr> <td> Mean bit rate</td><td> bit/sec </td><td ALIGN=RIGHT NOSAVE>1e+05</td></tr>
     * <tr> <td> Peak bit rate</td><td> bit/sec </td><td ALIGN=RIGHT NOSAVE>4.9e+05</td></tr>
     * <tr> <td> Peak/Mean of bit rate</td><td>-</td><td ALIGN=RIGHT NOSAVE>4.88</td></tr></table>
	*/
// Low quality
double[] coeffFuncal = { 0.292, 0.708};
GenericFunction[] gftal = { new MarkovianFunction(0.003), new LRDFunction(0.651) };
double[] coeffDistal= { 0.118, 0.882 };
GenericContinuousDistribution[] gdal = { new GammaDistribution(15.763,496.041), new LogNormalDistribution(8.512,0.559) };
GenericContinuousDistribution dal = new MixedDistribution(gdal, coeffDistal);
MGInfModel mginfal = new MGInfModel(gftal, coeffFuncal, 1.0, dal,VideoExample.dSlot , VideoExample.dPacketSize);

final VideoDefinition AladdinLow = new VideoDefinition ( mginfal , VideoCodecExample.MPEG4  , 60 , "Aladdin (Low Quality)");

// Medium quality
double[] coeffFuncam = { 0.311, 0.689};
GenericFunction[] gftam = { new MarkovianFunction(0.003), new LRDFunction(0.647)};
MGInfModel mginfam = new MGInfModel(gftam,coeffFuncam, 1.0, new GammaDistribution(2.7872, 3335.9),VideoExample.dSlot , VideoExample.dPacketSize);

final VideoDefinition AladdinMedium = new VideoDefinition ( mginfam, VideoCodecExample.MPEG4  , 60 , "Aladdin (Medium Quality)");

// Medium quality
double[] coeffFuncah = { 0.337, 0.663 };
GenericFunction[] gftah = { new MarkovianFunction(0.004), new LRDFunction(0.665)};
MGInfModel mginfah = new MGInfModel( gftah, coeffFuncah,1.0, new GammaDistribution(3.1079, 8405.6),VideoExample.dSlot , VideoExample.dPacketSize);

final VideoDefinition AladdinHigh = new VideoDefinition ( mginfah, VideoCodecExample.MPEG4  , 60 , "Aladdin (High Quality)");


}


interface SetTopExample {
	/**
  	* Characteristics of an Office Camera
   */
	
	 /**
     * Characteristics of Office Camera - low quality
     * <table BORDER NOSAVE><tr><td width="40%">Frame statistics</td><td width="20%"></td><td width="40%"></td></tr>
     * <tr> <td> compression ratio </td><td> YUV:MP4</td><td ALIGN=RIGHT NOSAVE>84.20</td></tr>
     * <tr> <td>File Size</td><td>byte</td><td ALIGN=RIGHT NOSAVE>3e+07</td></tr>
     * <tr> <td> Video run time</td><td> msec </td><td ALIGN=RIGHT NOSAVE>2.7e+06</td></tr>
     * <tr> <td> # of Frames </td><td>-</td><td ALIGN=RIGHT NOSAVE>67498</td></tr>
     * <tr> <td> mean frame size</td><td> byte </td><td ALIGN=RIGHT NOSAVE>4.5e+02</td></tr>
     * <tr> <td> var frame size</td><td>-</td><td ALIGN=RIGHT NOSAVE>1.7e+06</td></tr>
     * <tr> <td> CoV of frame size</td><td>-</td><td ALIGN=RIGHT NOSAVE>2.87</td></tr>
     * <tr> <td> min frame size</td><td>byte</td><td ALIGN=RIGHT NOSAVE>26</td></tr>
     * <tr> <td> max frame size</td><td>byte</td><td ALIGN=RIGHT NOSAVE> 5184</td></tr>
     * <tr> <td> Mean bit rate</td><td> bit/sec </td><td ALIGN=RIGHT NOSAVE>9e+04</td></tr>
     * <tr> <td> Peak bit rate</td><td> bit/sec </td><td ALIGN=RIGHT NOSAVE>1e+06</td></tr>
     * <tr> <td> Peak/Mean of bit rate</td><td>-</td><td ALIGN=RIGHT NOSAVE>11.48</td></tr></table>
     * <table BORDER NOSAVE><tr><td width="40%">GoP statistics</td><td width="20%"></td><td width="40%"></td></tr>
     * <tr> <td> # of GoPs </td><td>-</td><td ALIGN=RIGHT NOSAVE>5624</td></tr>
     * <tr> <td> mean GoP size</td><td> byte </td><td ALIGN=RIGHT NOSAVE>5.4e+03</td></tr>
     * <tr> <td> var GoP size</td><td>-</td><td ALIGN=RIGHT NOSAVE>2.4e+05</td></tr>
     * <tr> <td> CoV of GoP size</td><td>-</td><td ALIGN=RIGHT NOSAVE>0.09</td></tr>
     * <tr> <td> min GoP size</td><td>byte</td><td ALIGN=RIGHT NOSAVE>4666</td></tr>
     * <tr> <td> max GoP size</td><td>byte</td><td ALIGN=RIGHT NOSAVE> 9749</td></tr>
     * <tr> <td> Mean bit rate</td><td> bit/sec </td><td ALIGN=RIGHT NOSAVE>9e+04</td></tr>
     * <tr> <td> Peak bit rate</td><td> bit/sec </td><td ALIGN=RIGHT NOSAVE>1.6e+05</td></tr>
     * <tr> <td> Peak/Mean of bit rate</td><td>-</td><td ALIGN=RIGHT NOSAVE>1.80</td></tr></table>
     */

// Low quality
double[] coeffFuncocl = { 0.669, 0.331};
GenericFunction[] gftocl = { new MarkovianFunction(0.254), new SRDFunction(0.110)};
GenericFunction gfocl = new MixedFunction(gftocl ,coeffFuncocl  );
double[] coeffDistocl = { 0.053, 0.947 };
GenericContinuousDistribution[] gdocl = { new GammaDistribution(5.623,1344.5), new LogNormalDistribution(8.578,0.052) };
GenericContinuousDistribution docl = new MixedDistribution(gdocl, coeffDistocl);
MGInfModel mginfocl = new MGInfModel(gfocl , docl,VideoExample.dSlot , VideoExample.dPacketSize);

final VideoDefinition OfficeCameraLow = new VideoDefinition ( mginfocl , VideoCodecExample.MPEG4  , 60 , "Office Camera (Low Quality)");

//Medium quality
double[] coeffFuncocm = { 0.756, 0.244};
GenericFunction[] gftocm = { new MarkovianFunction(0.218), new SRDFunction(0.125)};
GenericFunction gfocm = new MixedFunction(gftocm ,coeffFuncocm);
double[] coeffDistocm = { 0.5, 0.5};
GenericContinuousDistribution[] gdocm = { new ConstantDistribution(5510), new LogNormalDistribution(6.83, 0.63) };
GenericContinuousDistribution docm = new MixedDistribution(gdocm, coeffDistocm);
MGInfModel mginfocm = new MGInfModel(gfocm , docm,VideoExample.dSlot , VideoExample.dPacketSize);

final VideoDefinition OfficeCameraMedium = new VideoDefinition ( mginfocm , VideoCodecExample.MPEG4  , 60 , "Office Camera (Medium Quality)");


// High quality
double[] coeffFuncoch = { 0.684, 0.316};
GenericFunction[] gftoch = { new MarkovianFunction(0.226), new SRDFunction(0.102)};
GenericFunction gfoch = new MixedFunction(gftoch ,coeffFuncoch );
double[] coeffDistoch = { 0.176, 0.824 };
GenericContinuousDistribution[] gdoch = { new GammaDistribution(25.83,1054.3), new LogNormalDistribution(10.046,0.047) };
GenericContinuousDistribution doch = new MixedDistribution(gdoch, coeffDistoch);
MGInfModel mginfoch = new MGInfModel(gfoch , doch,VideoExample.dSlot , VideoExample.dPacketSize);

final VideoDefinition OfficeCameraHigh = new VideoDefinition ( mginfoch , VideoCodecExample.MPEG4  , 60 , "Office Camera (Low Quality)");


}	
