/**
 * AudioCodecExample: Definition de plusieurs codecs audio standards
 * 
 * @author maria ladoue
 * @version 0.1, 06/11/02 
 */

package multimedia.codec;


public interface AudioCodecExample extends AudioCodecStandard {} 


/**
 * AudioCodecUDPIP: Codec for traffic over UDP/IP<BR>
 *<BR>
 * Packet header is: <BR>
 *  ____________________________________________________________________ _ _ _ _ _ _ _<BR>
 *  |                   |              |              |               |              |<BR>
 *  | Ehernet (14Bytes) | IP (20Bytes) | UDP (8Bytes) | RTP (12Bytes) |     data     |<BR>
 *  |___________________|______________|______________|_______________|_ _ _ _ _ _ _ |<BR>
 *<BR>
 * so for trafic over UDP/IP: header is 28 bytes length<BR>
 *                    RTP/IP: header is 40 bytes length<BR>
 *                    LAN   : header is 54 bytes length<BR>
 */

/*
 *
 * @author dsagnol
 * @date Aug 19, 2003
 *
 * This interface does not take into account the headers.
 * Headers will be had during the creation of the packet.
 */
interface AudioCodecStandard {
	// Defined an application throughput of 64kbps
	final AudioCodec G711 = new AudioCodec(96,0.012,"G711");
	// Defined an application throughput of 32kbps
	final AudioCodec G726 = new AudioCodec(64,0.016,"G726");
	// Defined an application throughput of 16kbps
	final AudioCodec G728 = new AudioCodec(60,0.024,"G728");
	// Defined an application throughput of 8kbps
	final AudioCodec G729 = new AudioCodec(30,0.03,"G729");
	// Defined an application throughput of 5.3-6.3kbps
	final AudioCodec G723_1 = new AudioCodec(24,0.03,"G723.1");  // packet size: 20 bytes for 5.3kbps, 24 bytes for 6.3kbps
}


interface AudioCodecUDPIP {

    // Defined an application throughput of 64kbps => real throughput(needed): 83kbps
    final AudioCodec G711 = new AudioCodec(124,0.012,"G711");
    // Defined an application throughput of 32kbps => real throughput(needed): 46kbps
    final AudioCodec G726 = new AudioCodec(92,0.016,"G726");
    // Defined an application throughput of 16kbps => real throughput(needed): 30kbps
    final AudioCodec G728 = new AudioCodec(88,0.024,"G728");
    // Defined an application throughput of 8kbps => real throughput(needed): 16kbps
    final AudioCodec G729 = new AudioCodec(58,0.03,"G729");
    // Defined an application throughput of 5.3-6.3kbps => real throughput(needed): 14kbps
    final AudioCodec G723_1 = new AudioCodec(52,0.03,"G723.1");  // packet size: 20 bytes for 5.3kbps, 24 bytes for 6.3kbps

}

interface AudioCodecRTPIP {

    // Defined an application throughput of 64kbps => real throughput(needed): 91 kbps
    final AudioCodec G711 = new AudioCodec(136,0.012,"G711");
    // Defined an application throughput of 32kbps => real throughput(needed): 52kbps
    final AudioCodec G726 = new AudioCodec(104,0.016,"G726");
    // Defined an application throughput of 16kbps => real throughput(needed): 34kps
    final AudioCodec G728 = new AudioCodec(100,0.024,"G728");
    // Defined an application throughput of 8kbps => real throughput(needed): 19kbps
    final AudioCodec G729 = new AudioCodec(70,0.03,"G729");
    // Defined an application throughput of 5.3-6.3kbps => real throughput(needed): 17kbps
    final AudioCodec G723_1 = new AudioCodec(64,0.03,"G723.1");  // packet size: 20 bytes for 5.3kbps, 24 bytes for 6.3kbps
}

interface AudioCodecLAN {

    // Defined an application throughput of 64kbps
    final AudioCodec G711 = new AudioCodec(150,0.012,"G711");
    // Defined an application throughput of 32kbps
    final AudioCodec G726 = new AudioCodec(118,0.016,"G726");
    // Defined an application throughput of 16kbps
    final AudioCodec G728 = new AudioCodec(114,0.024,"G728");
    // Defined an application throughput of 8kbps
    final AudioCodec G729 = new AudioCodec(84,0.03,"G729");
    // Defined an application throughput of 5.3-6.3kbps
    final AudioCodec G723_1 = new AudioCodec(78,0.03,"G723.1");  // packet size: 20 bytes for 5.3kbps, 24 bytes for 6.3kbps

}
