package multimedia.generation;

/**
 * GenericGeneration
 * @author mladoue
 * @version 0.1, 18/11/02 
 */

import java.io.*;
import java.util.Date;

import utils.*;
import utils.definitions.MyUnitDefinitions;

public abstract class GenericGeneration implements  MyUnitDefinitions {
		
	
	// Test duration in minutes
	protected int tpsSimul; 
	
	// Graphic mode
	protected boolean graphic;
	
	// to define which unit is used: BYTES, KILOBYTES...
	protected short unit;
	
	// Source number which have to be generated => permit to do a pseudo-aggregation
	// Default value is one!!!!
	protected int srcNumber = 1;
	
	/**
	 * Result File
	 */
	protected File resultFile;
	public String fileName = "";	// fileName is resultFile name
	protected FileWriter fichierReq;

	/**
 	*  To generate a scenario which could be played by the trafic generator
 	* 	Creates a file formated as follow:<BR>
 	* 		<i>Inter packet time(in micro-second)</i>	<i>packet size</i>  <BR>
	 */	  
	public void scenarioGeneration()  throws IOException {
		
		try {
			//resultFile =  new File( fileName );
			resultFile =  new File( "scenario.tmp" );
			this.fichierReq = FileTest.testExistent(resultFile.getName(),graphic);
		}
		catch (FileNotFoundException fnfe) {
			System.err.println("Name: " + fileName);
			fnfe.printStackTrace();
			
		}
		System.out.println("Start: " + new Date().toString());
		scenarioGenericGeneration( true);	// here serverName et portNumber aren't important
		System.out.println("End: " + new Date().toString());			
	}
	
	/**
	 *	Creates an "informative" scenario file of the aggregated flow<BR>
	 * Creates a file formated as follow:<BR>
	 * 	<i>date</i><BR>
	 *		Requested documents :  <i>reqDoc</i><BR>
	 *		Videos aggregation	Test Duration : <i>tpsSimul</i> mn <BR>
	 * 
	 *    date: t + <i>testTime</i> s,  IA : <i>interpacketIime</i>   <BR>
	 * 
	 * @param serverName gives the server name
	 * @param serverPort gives the number port
	 * @see multimedia.generation.GenericGeneration#scenarioGeneration()
	 */
	public void statisticsGeneration()  throws IOException {
		
		String fileName = getResultFileName();
		resultFile = changeScenarioFileExtension( new File( fileName ), false);
		this.fichierReq = FileTest.testExistent( resultFile.getName(), graphic);
		
		System.out.println("Start: " + new Date().toString());
		scenarioGenericGeneration( false );
		System.out.println("End: " + new Date().toString());			 	
	}

	/**
	 * Creates either a playable scenario file or an informative one.
	 * 
	 * @param playable	a boolean : true, to create a playable file
	 * @return average throughput in Bytes/s
	 * @throws IOException
	 */
	protected abstract double scenarioGenericGeneration( boolean playable) throws IOException;
	
	/**
   	* Returns the name of the generated scenario file
   	*/
    public String getResultFilePath() throws IOException  { return resultFile.getCanonicalPath();  }
    
    public void setResultFileName(String fileName) throws IOException {
    	this.fileName = fileName;
	}
	
	public String getResultFileName() { return this.fileName; }
    
    
	/**
	 * Method getResultFileSize.
	 * @return long
	 * @throws SecurityException
	 */
    public long getResultFileSize() throws SecurityException {	return resultFile.length();  }
    
    public void setDuration(int duration) { tpsSimul = duration; }
    public int getDuration() { return tpsSimul; }
    
    public void setGraphicMode(boolean graphic) { this.graphic = graphic; }
    public boolean getGraphicMode() { return graphic; }
    
	/**
	 * Method getGeneratedPacketNumber.
	 * @param idx if idx==0 => played scenario, idx==1 => informative scenario
	 * @return int
	 */
    public int getGeneratedPacketNumber(short idx) throws IOException {
    	if (idx == 0 ) {
    		return FileLineCounter.countLines(getResultFilePath() );
    	}
    	else	if (idx == 1) {
    		return FileLineCounter.countLines(getResultFilePath())-3;
    	}
    	else {
  			throw new IllegalArgumentException("GenericGeneration.getGeneratedPacketNumber : this parameter isn't allowed");
   		}
    }
    
	public void setUnit(short unit) { this.unit = unit; }

	public String getUnit() {
			return getUnit(unit);
	}
	 
	public static String getUnit(short unit) {
			   if (unit==BYTES) { return new String("");	}	
			   else if (unit==KILOBYTES) { return new String("k");	}
			   else if  (unit==MEGABYTES) { return new String("M");	}
			   else if  (unit==GIGABYTES) { return new String("G");	}
			   else {  	return new String("Error"); }
	}
    
    public void setSourceNumber( int srcNumber ) { this.srcNumber = srcNumber; }
    public int getSourceNumber( ) { return srcNumber; }
    	
    /**
	 *	To test if the file name given has its extension or not
	 *  And if there is an extension of this type, we rename the file without the extension
	 * @param rF
	 * @return String
	 */
	public static File changeScenarioFileExtension( File rF, boolean playable) {
		//Warning test for the extension of result file name
		File ans = rF;
		String name = rF.getName(), ext = "";
		String[] tmp = new String[2];
		if( playable ) { ext = ".scf"; }
		else { ext = ".stf"; }
		
		if ( ( name.endsWith(".scf") && !playable )  || ( name.endsWith(".stf") && playable ) ) {	 // si il y a une mauvaise extension 
			tmp = name.split(".", 2); 
			ans = new File( new File("./"), tmp[0]+ ext );
		}
		else if ( !name.endsWith(".scf") && !name.endsWith(".stf") ){	// si il n'y a pas d'extension
			ans = new File( new File("./"), name + ext );
		}
//		else {
//			// NOTHING TO DO
//		}

		return ans;
	}

	public void formatOutputFile(String tmp) throws IOException{
		// Format output file like the graphical interface...
		String rfnComplete = getResultFileName();
		System.out.println("SCENARIO NAME: " + rfnComplete ); //DEBUG
		String[] rfn = rfnComplete.split("/", 30); //on suppose qu'on n'a pas plus de 30 "/" pour definir le path du chemin
		int rfnLength = rfn.length;
		if (rfnLength != 1) {
			MyNumberFormat.timeSizeFormatage(tmp, rfn[rfnLength - 1]);
		} 
		else {
			MyNumberFormat.timeSizeFormatage(tmp, rfnComplete);
		}
	}

}
