/**
 * MMPPNGeneration: Pour generer les paquets suivant le modele MMPP-N
 *
 * @author maria ladoue
 * @version 0.1, 08/10/02 
 */

package multimedia.generation;

import java.io.*;
import java.util.*;

import statistic.*;
import utils.*;
import multimedia.codec.*;
import multimedia.model.*;


public class MMPPNGeneration extends GenericGeneration implements RandomGenerator,AudioCodecExample {

    protected MMPPNModel model;
 
 	 public MMPPNGeneration () {
 	 	graphic = false;
 	 }
 
    /**
   	 * Definition des parametres de generation de trafic
     *        ATTENTION ne marche pas dans le cas ou on defini un MMPP2 avec un taux d'emission non nul pour chacun des 2 etats     
     *
     *@param modelName
     *@param tpsSimul
     *@param fichierReq
     *@param graphic
     */
   public MMPPNGeneration( MMPPNModel model, int tpsSimul, int srcNumber, String fichierReq, boolean graphic)  throws IOException{	
		this.model = model;
		this.tpsSimul = tpsSimul;
		this.srcNumber = srcNumber;
		this.fileName = fichierReq;
		this.graphic = graphic;
    }


   /**
    * To generate a scenario file with information<BR>
    * Returns a file format as follow:<BR>
    *    <i>date</i> : <i>serverName</i><BR>
    *    <i>modelName</i>    IAmoy : <i>tpsIP_codec</i> numero de port : <i>number</i> packet size :<i>size</i> bytes<BR>
    *    date: t + <i>testTime</i> s,  IA : <i>interpacketIime</i> : 0.000000 s <BR>
    * 
    * @param playable
    * @param serverName gives the server name
    * @param serverPort gives the number port
    */  
	protected double scenarioGenericGeneration( boolean playable) throws IOException {
	
	// Definition des variables
	double time = 0, s;
	int newTpsSimul = tpsSimul*60;  // on le passe en secondes
	int etat = model.getEtatDepart();
	double T1 = 0, T2 = 0, p;
	int packetSize = model.codec.getSize();
	double debitIns = 0, sOld=0d;
	int nbPacket= 0, nboctet = 0;
    
	if(!playable) {    
		// Premiere ligne du scenario: date + nom_du_serveur
		fichierReq.write( new Date().toString() + "\n");
		fichierReq.write( model.getName() + "\t IAmoy : " + (model.getCodec()).getTpsIP() + "\t packet size : " + (model.getCodec()).getSize() + " bytes \n");
		fichierReq.write("Scenario format: date (in seconds) \t Inter-Packet Time(in milliseconds) \n\n" );// Format du fichier scenario
	}
	
    // Creation du scenario a proprement parler: le temps entre l'emission de chaque paquet
    while ( time < newTpsSimul ) {
	
	s = time;
	T1 = model.calculT( (model.getParam(etat,"alpha") + model.getParam(etat,"beta")) );  // Donne la duree de l'etat actif
	
//	if(playable) {
//		fichierReq.write( (s-sOld)*1000 + " \n");	// inter-packet time in s => in ms
//	}
//	else {
//		fichierReq.write("date: t + " + s + " s \t IA: " + (s-sOld)+ " s \n");
//	}
//	nbPacket++;

	if (etat != 0) {
	    while ( s < Math.min(time + T1 , newTpsSimul) ) {
		T2 = model.calculT( model.getParam(etat,"lambda") ); // Donne le temps d'inter-arrive dans l'etat actif
		
		if( (s + T2) < (time + T1) ) {   
		   
			if(playable) {
				fichierReq.write(Double.toString(T2*1000)+ "\t" + packetSize + "\n"); // inter-packet time in s => in ms
			}
			else {
		    	fichierReq.write("date: t + " + s + " s \t IA : " + (T2*1000) + "ms \t" + packetSize+ " Bytes \n");
			}
		    nbPacket++; // Pour calculer le debit moyen...
		    s +=  T2;
		}
		else { break; }
	    }
	}

	p = random.nextDouble();  // Determination de l'etat suivant
	if ( p <= model.getParam(etat,"beta") / ( model.getParam(etat,"alpha") + model.getParam(etat,"beta") ) ) {
	    etat++; 
	}
	else { etat--;	}
	
	sOld = s;
	time += T1;
	// Calcul du debit moyen: debitmoy = ((n-1)* debitmoy(n-1) + debitIns(n))/ n
	debitIns = nbPacket*(model.getCodec()).getSize()/T1; // Donne le debit instantane pour une periode: duree etat actif(T1)
	//debit = ( (nbPeriod - 1)*debit + debitIns )/nbPeriod; // Donne le debit moyen (depuis le debut de l'experimentation)
	//System.out.println("Debit instantane: " + debitIns);
	//System.out.println("Le debit moyen est de: " + debit  + " octets/s \n" + "soit :" + debit*8/1000  + " kbits/s \n" );
	
	nboctet += nbPacket*(model.getCodec()).getSize();
	nbPacket = 0;
    } //fin du while

	fichierReq.flush();
	fichierReq.close();

    double debit = nboctet/time;
    
	
	    String messageInfo ="";
	    messageInfo = new String("Le debit moyen est de: " + debit/1000  + " kB/s, soit : " + debit*8/1024  + " kb/s.\n"
	    + "Debit theorique: " + Math.round(model.meanRate()) + " kb/s");
	    //System.out.println("Le debit moyen est de: " + (nboctet/tpsSimul/1000)  + " kB/s, soit : " + nboctet/tpsSimul*8/1024  + " kb/s" );
	  
	    MessageManager.informationMessage( messageInfo, graphic);
	
    
 
	 
	MessageManager.informationMessage("File created as: " + resultFile.getCanonicalPath(), graphic);
	
	return debit;
	
    } // fin de scenarioGenericGeneration(boolean, String, int)

public void setMMPPNModel(MMPPNModel mmppnm) { model = mmppnm; }

/**
 * Methode: Main pour tester la classe
 */

    public static void main (String[] args) throws IOException {
	
	String s2 = "MMPPNGene", s3 = new String("IPPGene"), s4 = new String("MMPP2Gene");
	MMPPNModel mmpp20 = new MMPPNModel(20,1,3.12,0.78,new AudioCodec(1000,0.25));
	MMPPNGeneration g20 = new MMPPNGeneration(mmpp20,20,1,s2,false);
	
	//g20.statisticsGeneration();
	g20.scenarioGeneration();
	g20.formatOutputFile("scenario.tmp");
    }
}
