/**
 * OnOffGeneration: Pour generer les paquets suivant le modele On-Off
 *
 * @author maria ladoue
 * @version 0.1, 25/10/02 
 */

package multimedia.generation;

import java.io.*;
import java.util.*;


import statistic.distribution.*;
import statistic.*;
import utils.*;
import multimedia.codec.*;
import multimedia.model.OnOffModel;


public class OnOffGeneration extends GenericGeneration implements RandomGenerator,AudioCodecExample {

    protected OnOffModel model;

	 public OnOffGeneration () {
 		graphic = false;
 	}

    /**
     *@param model it must be an On-Off Model
     *@param tpsSimul it is the test duration in minutes
     *@param fichierReq it's a file where results will be enregistrated
     *@param graphic if you use graphic environment or not
     *@throws IOException
     */
    public OnOffGeneration ( OnOffModel model, int tpsSimul, int srcNumber, String fichierReq, boolean graphic) throws IOException {
		this.model = model;
		this.tpsSimul = tpsSimul;
		this.srcNumber = srcNumber;
		this.fileName = fichierReq;
		this.graphic = graphic;
    }

    
	protected double scenarioGenericGeneration( boolean playable) throws IOException {
	
		// Definition des variables
		double time = 0, s, sOld = 0;
		int newTpsSimul = tpsSimul * 60;  // on le passe en secondes
		int etat = model.getEtatDepart();
		double T1 = 0, T2 = 0;
	
		int nbPeriod = 0, nbBytes=0, cpt = 0;
		double tpsIP =(model.getCodec()).getTpsIP();
		int packetSize = model.getCodec().getSize();
		//double tpsIPIns = 0 ; // Temps inter-arrivee instantane
		//double debitIns = 0;
	
	
	    if( !playable ) {
			// Premiere ligne du scenario: date + nom_du_serveur
			fichierReq.write( new Date().toString() + "\n");	
			fichierReq.write( model.getName() + " Model, \t IAmoy : " + (model.getCodec()).getTpsIP() + "\t packet size : " + (model.getCodec()).getSize() + " bytes \n");	// Format du fichier scenario
			fichierReq.write("Scenario format: date (in seconds) \t Inter-Packet Time(in milliseconds) \t packet size(in Bytes) \t  rate(in kb/s) \n\n" );// Format du fichier scenario
	    }
	    
		while ( time < newTpsSimul ) {    
		    s=time;		
		    if (etat==0) {
			T1 = model.calculT(etat);
			while( s < Math.min(time + T1, newTpsSimul) ) {
				
				//nbPacket++; // Pour calculer le debit moyen...
				nbBytes += packetSize;
				if ( playable ) {
					fichierReq.write( (s-sOld)*1000 + "\t" + packetSize + "\n");  // inter-packet time in s => in ms
				}
				else { 
			 	   fichierReq.write("date : t + " + MyNumberFormat.myTimeFormatter.format(s) + " s \t IA: " + MyNumberFormat.myIAFormatter.format((s-sOld)*1000) + " ms \t" + packetSize  + " Bytes \t rate: " +MyNumberFormat.myDebitFormatter.format(nbBytes*8/1000/s) +" kb/s. \n");
			
				// Informations supplementaires
				// Calcul du debit moyen: debitmoy = ((n-1)* debitmoy(n-1) + debitIns(n))/n ou n est le nb de period
				//debitIns = nbPacket*size/(s-time); // Donner le debit instantane pour une periode: Ton+Toff en seconde
				//debit = ( nbBytes + nbPacket*size ) / s;
				}
				sOld=s;
				s += tpsIP;
			} // fin du deuxieme while
			time += T1;
			etat = 1;
		    } // fin du if
		    else {
				T2 = model.calculT(etat);
				time += T2;
				etat = 0;
		    } // fin du else
		    
		    
		    // determination du debit moyen...
		    if (cpt != 1 ) { cpt++; }
		    else {
				nbPeriod++;
				//nbBytes += nbPacket*size;
				//nbPacket = 0;	
				cpt = 0;
			}
		}  // fin du while
	
		fichierReq.flush();
		fichierReq.close();
		 
	
		double debit = nbBytes/time;
		
		if ( !playable ) {
			String messageInfo ="";
			messageInfo = new String("Amount of data transmitted during this test: " + nbBytes/1000 + " kBytes.\n"
			+	"Le debit moyen est de: " + MyNumberFormat.myDebitFormatter.format(debit/1000)  + " kB/s, soit : " + MyNumberFormat.myDebitFormatter.format(debit*8/1024)  + " kb/s.\n"
		 	+ "Debit theorique: " +  MyNumberFormat.myDebitFormatter.format(Math.round(model.meanRate())) + " kb/s.");
		 
		 	MessageManager.informationMessage(messageInfo,graphic);
		}
		
		
		MessageManager.informationMessage("File created as: " + resultFile.getCanonicalPath(), graphic);
		
		return debit;
	
    } //fin du scenarioGenericGeneration(boolean,serverName,serverPort)


public void setOnOffModel(OnOffModel oom) { 	model = oom; }


    
/**
 * Methode: Main pour tester la classe
 */
    public static void main (String[] args) throws IOException {

		String s1 ="OnOffGene";
		OnOffModel m1 = new OnOffModel(0,new ExponentialDistribution(0.2),new ExponentialDistribution(0.1),G726);
		OnOffGeneration oog1 = new OnOffGeneration(m1,30, 1,s1,false);
		oog1.scenarioGeneration();
		oog1.formatOutputFile("scenario.tmp");
    }
}
