/**
 * GenericAudioModel : definit les caracteristiques communes a tous les modeles utilises
 *
 * @author maria ladoue 
 * @version 0.1, 04/07/02 
 */

package multimedia.model;

import multimedia.codec.*;

public abstract class GenericAudioModel extends GenericModel {
    
    /**
     * States number of the modelName
     */
    protected int etatCount;   // Represente le nombre d'etats (i.e: N) 0--1--2 - - - N

    /**
     * It defines the state of the system at the start
     */
    protected int etatDepart;  // Represente l'etat de depart du processus
    
    /**
     * It's the audio codec assocaited to the choosen AudioModel 
     */
    public AudioCodec codec; // Permet d'obtenir le temps inter-paquet (et aussi la taille des paquets a generer)

    /**
     * Recupere la valeur des parametres ou d'un seul
     */
    public abstract double getParam(int unEtat, String unParam);
    
    /**
     * Positionne la valeur de tous les parametres ou d'un seul
     */   
    public abstract void setParam(String unNomParam,double uneValeur);
    
    /**
     * Recupere et fixe le nombre d'etat
     */
    public int getEtatCount() { return etatCount; };

    public void setEtatCount( int unEtatCount ) throws IllegalArgumentException { 
		if ( unEtatCount > 1 ) {
		    this.etatCount = unEtatCount; 
		}
		else { throw new IllegalArgumentException("N >= 2 : " + unEtatCount ); }	
    }
    
    /**
     * Recupere et fixe l'etat de depart
     */

    public int getEtatDepart() { return etatDepart; };
    
    public void setEtatDepart(int unEtatDepart) {
	
	if (( unEtatDepart >= 0) && ( unEtatDepart < this.getEtatCount() )) {
	    this.etatDepart = unEtatDepart;
	}
	else {  
	    throw new IllegalArgumentException("Les valeurs doivent etre comprises entre 0 et " + (getEtatCount()-1));
	}
    }

    /**
     * Return and fixe audio codec
     */
    public AudioCodec getCodec() {
	return codec;
    }
    
    public void setCodec(AudioCodec unCodec) {
	codec = unCodec;
    }


    //Permet de ...
    public abstract double calculT(double unParam);

}
