
package multimedia.model;

import utils.definitions.MyExceptionDefinitions;

/**
 * @author mladoue
 * @ version 0.1
 */
public class MGInfAggregatedFlowModel extends GenericAggregatedModel {

	protected MGInfModel[] models;
	protected int[] weights;  // it corresponds to the number of flow of each type (between 1 and N)
	protected short[] units;  // it correspond sto the unit (bytes, kiloBytes... of each flow type) : default is BYTES
	protected int nbDifferentFlow = 0;  // it's M
	protected int flowsNumber = 0;  // it's L => L= W1 + W2 + ... WM

	public MGInfAggregatedFlowModel() {
		name = new String("MGInfAggregatedFlow");
		paramsCount =2;  // TODO	
	}

	/**
	 * MGInfAggregatedFlowModel
	 * @param srcType
	 * @param weights
	 * @throws IllegalArgumentException
	 */
	   public MGInfAggregatedFlowModel( MGInfModel[] models, int[] weights) throws IllegalArgumentException {
			this();
			this.nbDifferentFlow = models.length;
		
			setModels(models);
			setWeights(weights);
			setSourceNumber();
	   }
	   
	/**
	 * MGInfAggregatedFlowModel
	 * @param srcType
	 * @param weights
	 * @throws IllegalArgumentException
	 */
	   public MGInfAggregatedFlowModel( MGInfModel[] models, int[] weights, String name) throws IllegalArgumentException {
			this( models,weights);
			this.name = name; 
	   }
	   

public MGInfModel[] getModels() { return models; }

public void setModels(MGInfModel[] models) throws IllegalArgumentException {
	if( models.length == nbDifferentFlow ) {
		this.models = models;
	}
	else { throw MyExceptionDefinitions.PARAMETERS_TABLE_SIZE; }	
}

public int[] getWeights() { return weights; }
public void setWeights( int[] weights ) throws IllegalArgumentException {
	if( weights.length == nbDifferentFlow ) {
		this.weights = weights;
	 }
	 else { throw MyExceptionDefinitions.PARAMETERS_TABLE_SIZE; }
}

public String getName() { return name; }
public void setName(String name ) { this.name = name; }

/**
 * Returns an M/G/Infinity model which corresponds to a the specified flow type
 * @param sourceType an int between 0 and nbDifferentFlow-1
 * @return an MGInfModel
 */
public MGInfModel getSpecificModel( int flowType ) {
	if (flowType<nbDifferentFlow && flowType>=0) {
		return models[flowType];
	}
	else {
		throw new IllegalArgumentException("There are..");  // TODO
	}
}

public void setSpecificModel( int flowType, MGInfModel model ) {
	this.models[flowType] = model;
}

public int getSpecificWeight(int flowType) { return weights[flowType]; }

	/**
	 * To compute the number of sources => nb = w1 + w2 + ... +wn
	 * @return the number of all sources: independent or not!!!
	 */
	public int getSourceNumber() { return flowsNumber;	}
	
	public void setSourceNumber() {
		for (int i=0;i< weights.length;i++ ) {
			flowsNumber += weights[i];
		}
	}
	
	
	public int getFlowTypeNumber() { return nbDifferentFlow; }
	public void setFlowTypeNumber( int nbDifferentFlow ) { this.nbDifferentFlow = nbDifferentFlow; }

	public static void main(String[] args) {
	}
}
