/**
 * MGInfModel : Modele M/G/Infini stationnaire 
 *
 * @version 0.1
 * @author maria ladoue
 */

package multimedia.model;


import statistic.distribution.*;
import statistic.function.*;
import multimedia.*;
import multimedia.codec.*;


public class MGInfModel extends GenericModel implements  AudioCodecExample, VideoCodecExample, VideoExample {

    // Definition des parametres

    public  GenericDiscreteDistribution sigma; // Distribution du temps de service
    public  GenericDiscreteDistribution chapeauSigma; // Distribution du temps de service de l'etat initial
    public double lambda; //Parametre de la loi de Poisson associe aux arrivees
    public GenericContinuousDistribution distributionTest;
    public GenericDistribution distributionSlot, distributionPacketSize;
    protected GenericFunction gf;
   
    protected PoissonDistribution loiPoisson;

    protected static int To=0;
    protected static int SLOTI=1;

    public static double SIGMA = 0;	 // caracterise la loi definissant le temps de service
    public static double CHAPEAUSIGMA = 1;	 // caracterise la loi definissant le temps de service de l'etat initial
    public static double LAMBDA = 2;	// caracterise la loi de poisson definissant le processus d'occupation de la file M/G/infini
 

 //  Definition des constructeurs
	/**
	 * MGInfModel : constructeur par defaut
	 * @see java.lang.Object#Object()
	 */
public MGInfModel() {
		name = new String("MGInf");
		paramsCount = 2;
		
		this.gf = new SRDFunction(0.04);
		sigma = new SigmaDistribution(gf);
		chapeauSigma = new ChapeauSigmaDistribution((SigmaDistribution)sigma);
		this.lambda = 1d;
		
	
		setDistributions( new ExponentialDistribution(10d), new ConstantDistribution(1d), new ConstantDistribution(1000d));
}

    
	/**
	 * Method MGInfModel.
	 * @param autoCorr
	 */
    public MGInfModel (GenericFunction autoCorr) {
		this();
		this.gf = autoCorr;
	
		((SigmaDistribution)sigma).setAutocorr(autoCorr);
		chapeauSigma = new ChapeauSigmaDistribution((SigmaDistribution)sigma);
    }

    /**
     * MGInfModel 
     *
     * @param autoCorr it could be a markovian, SRD or LRD Function
     * @param distributionTest defines the distribution of clients during the entire test
     *        it's could be logNormal, Gamma or Pareto distribution
     * @param distributionsSlot defines the distribution of clients in a slot => inter-packet time
     * @param distributionsPacketSize defines the distribution of packet size => each packet size
     */

    public MGInfModel (GenericFunction autoCorr, GenericContinuousDistribution distributionTest, GenericDistribution distributionSlot, GenericDistribution distributionPacketSize) {
		this(autoCorr);
		paramsCount = 4;
		
		setDistributions(distributionTest, distributionSlot, distributionPacketSize);
    }


    /**
     * MGInfModel : Constructeur permettant de tout definir....Modele M/G/inf de reference 
     *
     * @param autoCorr it could be a markovian, SRD or LRD Function
     * @param lambda the parameter of Poisson Distribution : it's a double.
     *        WARNING: poisson parameter is lambda*E(sigma)!!
     * @param distributionTest defines the distribution of data(in KBytes) during the entire test
     *        it's could be logNormal, Gamma or Pareto distribution
     * @param distributionsSlot defines the distribution of clients in a slot.
     * @param distributionsPacketSize defines the distribution of packet size.
     */
    public MGInfModel (GenericFunction autoCorr, double lambda, GenericContinuousDistribution distributionTest, GenericDistribution distributionSlot, GenericDistribution distributionPacketSize) {
		this(autoCorr,distributionTest,distributionSlot,distributionPacketSize);
		paramsCount = 5;

		this.lambda = lambda;
    }

	/**
	 * Method MGInfModel.  Used to define MGInf model in VideoDefinition => VideoExample
	 * @param autoCorr
	 * @param weight
	 * @param lambda
	 * @param distributionTest
	 * @param distributionSlot
	 * @param distributionPacketSize
	 */
	//TODO: DS 27/08 A supprimer ?
 	public MGInfModel(GenericFunction[] autoCorr, double[] weight, double lambda, GenericContinuousDistribution distributionTest, GenericDistribution distributionSlot, GenericDistribution distributionPacketSize) {
	
		name = new String("MGInfComplex");
		paramsCount = 6;
		gf = new MixedFunction(autoCorr,weight);
	
		sigma = new SigmaDistribution(gf);
		chapeauSigma = new ChapeauSigmaDistribution((SigmaDistribution)sigma);
	
		this.lambda = lambda;
		setDistributions(distributionTest, distributionSlot, distributionPacketSize);
    }


 	public void setDistributions( GenericContinuousDistribution distributionTest, GenericDistribution distributionSlot, GenericDistribution distributionPacketSize) {
		this.distributionTest = distributionTest;
		this.distributionSlot = distributionSlot;
		this.distributionPacketSize = distributionPacketSize;
 	}
 	 
	/**
	 * Method calculNbClients.
	 * @param unMoment
	 * @return int
	 * @throws IllegalArgumentException
	 */
    public int calculNbClients(int unMoment) throws IllegalArgumentException {
		double paramPoisson = 0d;  // invalid value...just to initialize
		if ( unMoment == To ) {	// tirage Poisson( lambda * E(sigma) )
	    	paramPoisson = lambda * sigma.mean();
		}
		else if ( unMoment == SLOTI ) {	// tirage Poisson( lambda )
	    	paramPoisson = lambda;
		}
		else { 
			throw new IllegalArgumentException("MGInfModel.calculNbClients  On ne peut determiner un tirage que pour t=0 ou t=i");
	 	}

		loiPoisson = new PoissonDistribution( paramPoisson );	
		return (int)(loiPoisson.rand()); 
    }

	/**
	 * Method calculT to be compatible with GenericModel
	 * @param uneDistribution : it could be 0 = Sigma, 1=ChapeauSigma or 2=Poisson
	 * @return double
	 * @see multimedia.GenericModel#calculT(double)
	 */
    public double calculT(double uneDistribution) throws IllegalArgumentException {
		if (uneDistribution == SIGMA) {
	    	return sigma.rand();
		}
		else if (uneDistribution == CHAPEAUSIGMA) {
	    	return chapeauSigma.rand();
		}
		else if (uneDistribution == LAMBDA) {
	    	return loiPoisson.randInt();
		}
		else {
	    	throw new IllegalArgumentException("MGInfModel.calculT : Invalid value");
		}
    }// fin de la methode calculT
      
  
	/**
	 * Method getParameter. Return some parameters of this model
	 * @param idx
	 * @return Object
	 */
    public Object getParameter(int idx) {
	
	switch (idx) {
	case 0: return ((SigmaDistribution)sigma).getAutocorr();
	case 1: return new Double(lambda);
	case 2: return distributionTest;
	case 3: return distributionSlot;
	case 4: return distributionPacketSize;
	case 5: return sigma;
	case 6: return chapeauSigma;
	default: throw new IllegalArgumentException("MGInfModel.getParam : it's not a valid parameter");
	}
    }

  public String getCompleteName() { 	 return new String("Model M/G/Infinity"); }
    

	/**
	 * Method setSigmasDistribution. to position Sigma and ChapeauSigma distribution
	 * @param autoCorr
	 */
	public void setSigmasDistribution(GenericFunction autoCorr) {
		//if (! autoCorr.getName().equals("Mixed")) {
		((SigmaDistribution)sigma).setAutocorr(autoCorr);
		chapeauSigma = new ChapeauSigmaDistribution((SigmaDistribution)sigma);
//		}
//		else { //Definit uniquement pour les modeles M/G/Inf utilisant MixedFunction!!!!!
//			sigma = new SigmaAggregatedDistribution(((MixedFunction)autoCorr).getSpecificFunctions(),((MixedFunction)autoCorr).getWeight()); 
//			chapeauSigma = new ChapeauSigmaAggregatedDistribution((SigmaAggregatedDistribution)sigma);
//		}
	}
	
	public void setLambda(double lambda) { this.lambda = lambda; }
	public double getLambda() { return lambda; }
	
	public void setDistributionTest( GenericContinuousDistribution distributionTest ) { this.distributionTest = distributionTest; }
	public GenericContinuousDistribution getDistributionTest( ) { return distributionTest; }
	
	public void setDistributionSlot( GenericDistribution distributionSlot ) { this.distributionSlot = distributionSlot; }
	public GenericDistribution getDistributionSlot( ) { return distributionSlot; }
	
	public void setDistributionPacketSize( GenericDistribution distributionPacketSize ) { this.distributionPacketSize = distributionPacketSize; }
	public GenericDistribution getDistributionPacketSize( ) { return distributionPacketSize; }
	
	// Utiliser uniquement pour le MGInf Complex
	public GenericFunction getAutoCorr() { return gf; }
	public void setAutoCorr(GenericFunction gf) { this.gf = gf; }
	
	public double[] getFunctionsParameter() {
		double[] ans;
		if (! gf.getName().equals("Mixed")) {
			ans = new double[1];
			ans[0] = gf.getParam();
		}
		else {
			//ans = new double[ ((MixedFunction)gf).getFunctionsNumber() ];
			ans = ((MixedFunction)gf).getFunctionsParameter();
		}
		return ans;
	}

    /**
     * Methode: Main pour tester la classe
     */    
    public static void main (String[] args) {
	MGInfModel mginf = new MGInfModel(new SRDFunction(0.04),2d,new GammaDistribution(5d,1d/10d),new ExponentialDistribution(0.2d),new ExponentialDistribution(0.3d));
	System.out.println(mginf.getParameter(1));

	//MGInfModel modelName  =new MGInfModel(new SRDFunction(0.08),new GammaDistribution(19.78,0.71),new ExponentialDistribution(0.2d),new ExponentialDistribution(0.001d));
	//for (int i=0;i<50;i++) {
	//System.out.println((new PoissonDistribution(((Double)(modelName.getParameter(1))).doubleValue())).rand());
	//}
	/**  VideoDefinition[] vd =  { JurassicParkLow,BeanLow};
        double[] weight ={2,3} ;
	MGInfAggregationModel mginfa = new MGInfAggregationModel(vd , weight);
	MGInfModel model = mginfa.getMGInfProcessAggregated();
	System.out.println(model.getParameter(1));
	*/
 
 System.out.println(BeanLow.getModel().getDistributionTest().getName());

 }

}
