package statistic.distribution;

/**
 * CompoundDistribution : It's a "distribution" where you define several distributions with a weight
 * 								And each distribution is choosen according to its weight
 * 
 * @author mladoue
 * @version 0.1, 23/09/02 
 */


public class CompoundDistribution extends GenericContinuousDistribution {

    protected GenericContinuousDistribution[] distribution;
    protected double[] weightDistribution;


	/**
     * CompoundDistribution
     *
     *@param distrib it defines which distributions are involved ( => weigth of each distribution is 1/distribution number )
     */

    public CompoundDistribution( GenericContinuousDistribution[] distrib ) {
		paramsCount = 1;
		name = new String("Compound");
	
		distribution = new GenericContinuousDistribution[distrib.length];
		weightDistribution = new double[distrib.length];

		for (int j=0;j<distrib.length;j++) {
	   	 	distribution[j] = distrib[j];
	   	 	weightDistribution[j] = 1d;
		}
    }
    
    
    /**
     * MixedDistribution
     *
     *@param distrib it defines which distributions are involved
     *@param weigth it defines the weigth of each distribution (>=1)
     */
    public CompoundDistribution( GenericContinuousDistribution[] distrib, double[] weigth) {
		this(distrib);
		paramsCount = 2;
		weightDistribution = new double[weigth.length];

		for (int j=0;j<weigth.length;j++) {
	    	weightDistribution[j] = weigth[j];
		}
		
    }

	public boolean hasValidParam(double[] desParams) {
		return true; // TODO: should we check all the distribution in the Compount ?
	}


	 /**
     * pdf :
     *
     *@param x it's a double
     *@param the pdf(x) value of this 'mixed' distribution
     */
    public double pdf(double x) {
		double result = 0d;
    	for (int i=0;i<weightDistribution.length;i++) {
    		result += weightDistribution[i] * distribution[i].pdf(x);	
    	}
		return result;
    }

    
    /**
     * Returns the value of the cumulative distribution function at x
     */
    public double cdf(double x) {
	double result = 0d;
    	for (int i=0;i<weightDistribution.length;i++) {
    		result += weightDistribution[i] * distribution[i].cdf(x);	
    	}
		return result;
    }


    /**
     * Return a random value in [0 1]
     */
    public double rand() {
		double result = 0d;
    		for (int i=0;i<weightDistribution.length;i++) {
    			result += weightDistribution[i] * distribution[i].rand();	
    		}
		return result;
    }

    /**
     * Returns the mean of the distribution
     */
    public double mean() {
    	double mean = 0d;
    	for (int i=0;i<weightDistribution.length;i++) {
    		mean += weightDistribution[i]*distribution[i].mean();	
    	}
		return mean;
    }

    /**
     * Return the value of Inverse Function at x 
     * @param x with x>=0 and x<=1
     */
    public double inverseCdf(double x) {
		double result = 0d;
    	for (int i=0;i<weightDistribution.length;i++) {
    		result += weightDistribution[i] * distribution[i].inverseCdf(x);	
    	}
		return result;
    }
    

    /**
     * Returns the variance of the distribution
     */
    public double var() {
    		double var = 0d;
    	for (int i=0;i<weightDistribution.length;i++) {
    		var += weightDistribution[i]*distribution[i].var();	
    	}
    	
	return var;
    }
    
	public String getEquation() {
		return new String("no equation yet..."); //TODO: Put an equation
	}

 
     /**
     * Methode: Main pour tester la classe
     */
	public static void main(String[] args) {
		GenericContinuousDistribution[] distributionsTest = { new GammaDistribution(19.78,0.71),new GammaDistribution(21.4,1.98) ,new LogNormalDistribution(2.93,0.48)};
        double[] weight ={5d,2d,3d} ;
		GenericContinuousDistribution distributionTest = new CompoundDistribution(distributionsTest,weight);
		//System.out.println(distributionTest);
		System.out.println(distributionTest.mean()/0.48 + "kB");
		System.out.println(distributionTest.mean()*0.48*7500/1000000);
		System.out.println((5*distributionsTest[0].mean()+2* distributionsTest[1].mean()+3*distributionsTest[2].mean())/0.48/1000 + "MB");
	
	}
}
