/**
 * ConstantDistribution : Definition de la loi comstante
 *
 * @author maria ladoue
 * @version 0.1, 04/07/02 
 */

package statistic.distribution;

import statistic.*;

public class ConstantDistribution  extends GenericContinuousDistribution implements RandomGenerator  {

    /**
     * Equation de la densite: 
     *    f(x)= a;
     */

	private static int A = 0;

    public ConstantDistribution() {
		paramsCount = 1;
		name = new String("Constant");
	
		params = new double[paramsCount]; // initialisation du tableau des valeurs des parametres
		paramNames = new String[paramsCount]; // initialisation du tableau des noms des parametres
		params[A] = 1d;
		paramNames[A] = new String("a");
    }

    public ConstantDistribution(double a) {
		this();
		setParam(A,a);
    }

	public boolean hasValidParam(double[] desParams) {
		return true; // TODO: no constraint on A value ?
	}


// Definitions de quelques methodes

    /**
     * Returns the value of the probability distribution function at x
     */
    public double pdf(double x) { return params[A];  }

    /**
     * Returns the value of the cumulative distribution function at x (=Fonction de repartition)
     */
    public double cdf(double x) {
        if (x > 0) {
            return 1d;
        } 
		else { return 0d; }
    }

    /**
     * Return the value of Inverse Function at x 
     * @param x with x>=0 and x<=1
     */
    public double inverseCdf(double x) throws IllegalArgumentException {
	if ( x>=0 && x<=1 ) {
	    return 1d;
	}
	else {
	    throw new IllegalArgumentException("ConstantDistribution.inverseCdf : x must be >=0 and <=1");
	}
    }
    
    public double mean() {
        return params[A];
    }
    
    public double var() {
        return 0d;
    }

    public double rand() {
	return params[A];
    }
 
	public String getEquation() {
		return new String("f(x)= a");
	}



}
