/**
 * GammaDistribution : Definition de la loi Gamma
 *
 * @author maria ladoue
 * @version 0.1, 30/08/02 
 */

package statistic.distribution;

import java.lang.Math;

import JSci.maths.SpecialMath;


import statistic.*;

public class GammaDistribution extends GenericContinuousDistribution implements RandomGenerator,ComputerArithmetic  { 

    /**
     * Equation de la densite:<BR>
     *  f(x)= lambda^a * x^(a-1) * exp(-lambda*x) / Gamma(a)<BR>
     *     where alpha = shape parameter, must be > 0 <BR>
     *           lambda = scale parameter, must be > 0 <BR>
     *           x = the random variable,  must be >= 0 <BR>
     */


    public static int A=0;
    public static int LAMBDA=1;

    /**
     * Definition des constructeurs
     */

    public GammaDistribution() {
	paramsCount = 2;
	name = new String("Gamma");
	params = new double[paramsCount]; // initialisation du tableau des valeurs des parametres
	paramNames = new String[paramsCount]; // initialisation du tableau des noms des parametres
	params[A] = 1.0d;
	params[LAMBDA] = 0.5d;
	paramNames[A] = new String("a");
	paramNames[LAMBDA] = new String("lambda");
    }

    public GammaDistribution(double a,double lambda) throws IllegalArgumentException {
	this();
	if ( a > 0 && lambda > 0 ) {
	    params[A] = a;
	    params[LAMBDA] = lambda;
	}
	else {
	    throw new IllegalArgumentException("GammaDistribution.constructor : a and lambda must be greater than 0");
	}
    }

    public GammaDistribution(double[] params) throws IllegalArgumentException {
		this();
		if (hasValidParam(params))
			setParams(params);
		else
			throw new IllegalArgumentException("GammaDistribution.constructor : a and lambda must be greater than 0");
    }
    
	public boolean hasValidParam(double[] desParams) {
		if (desParams[A] >0 && desParams[LAMBDA]>0)
			return true;
		else
			return false;
	}
	
    /**
    * Constructeur permettant a l'utilisateur de specifier uniquement le mean et le var qu'il veut
    * Dans le cas du modele MGInf
    *      mean: represente le nombre moyen d'octets par images (i.e: taille moyenne des images)
    *      variance: represente (a peu pres) l'ecart entre la valeur consideree et la moyenne
    */
    /**public GammaDistribution(double mean,double var) throws IllegalArgumentException {
	this();
	if (mean > 0 && var>0) {
	    params[A] = Math.pow(mean,2) / var;
	    params[LAMBDA] = mean / var;
	}
	else {
	    throw new IllegalArgumentException("GammaDistribution.constructor : mean and var must be greater than 0");
	}
	}*/

    public void setParams(double [] params) throws IllegalArgumentException {
	if (params.length >= paramsCount && params[A] > 0 && params[LAMBDA] > 0 ) {
	    this.params[A] =  params[A];
	    this.params[LAMBDA] =  params[LAMBDA];
	}
	else {
	    throw new IllegalArgumentException("GammaDistribution.setParams : a and lambda must be greater than 0");
	}
    }

    public void setParam(int idx, double p) throws IllegalArgumentException {
        if (idx < paramsCount && (idx != LAMBDA || p > 0)) {
            params[idx] = p;
        } else {
            throw new IllegalArgumentException("GammaDistribution.setParam : a and lambda must be greater than 0");
        }
    }

    /**
     * Returns the value of the probability distribution function at x
     */
    public double pdf(double x) {
	return ( Math.pow(params[LAMBDA],params[A])/ 
		 SpecialMath.gamma(params[A]) *
		 Math.pow( x , params[A]-1 ) / 
		 Math.exp( params[LAMBDA] * x));
    }

    /**
     * Returns the value of the cumulative distribution function at x (=Fonction de repartition)
     */
    public double cdf(double x) {
	//return Mathematic.incompleteGamma(params[A],params[LAMBDA] * x );
	return (1d - Mathematic.incompleteGammaComplemented(params[A],params[LAMBDA] * x ));
		//cern.jet.stat.Gamma.incompleteGamma(params[A],params[LAMBDA] * x ) / 
		
		// / SpecialMath.gamma(params[A]));
	
    }

    /**
     * Return the value of Inverse Function at x
     * @param x with x>=0 and x<=1
     */
    public double inverseCdf(double x) {
	// return ( 1/params[LAMBDA] * Mathematic.incompleteGammaComplementedInverse(this.getParam(0),SpecialMath.gamma(params[A])*(1-x)) );
	return 1 / params[LAMBDA] *  Mathematic.incompleteGammaComplementedInverse(this.getParam(0),1-x);
    }
    
    public double mean() {
        return params[A]/params[LAMBDA];
    }
    
    public double var() {
        return params[A] / Math.pow(params[LAMBDA],2d);
    }
    
    public double rand() {

	if (params[A]==1 && params[LAMBDA]>0) {
	    return ((-1.0d/params[LAMBDA]) * Math.log(1-random.nextDouble()));
	}
	/**else if (params[A]>0 &&  params[A]<1 && params[LAMBDA]==1) {
	    double c = (Math.exp(1) + params[A])/Math.exp(1);
	    double p = c*random.nextDouble();
	    double v = random.nextDouble();
	    if (p>1) {
		double Xtmp = -Math.log((c-p)/params[A]);;
		if(Xtmp<0) {
		    p = c*random.nextDouble();
		}
		else { 
		    if (v <= Math.pow(Xtmp,params[A]-1) ) {
			return Xtmp;
		    }
		    else {
			p = c*random.nextDouble();
		    }
		}
	    }
	    else {
		 double Xtmp = Math.pow(p,1/params[A]);
		 if (v <= Math.exp(-Xtmp)) {

		 }
	    }
	}
	*/ //TO DO
	else if (params[A]>1 && params[LAMBDA]==1) {
	    double y = ((-1.0d/params[LAMBDA]) * Math.log(1-random.nextDouble()));
	    if ( Math.log(random.nextDouble()) > ((params[A]-1)*(1-y+Math.log(y))) ) {
		y = ((-1.0d/params[LAMBDA]) * Math.log(1-random.nextDouble()));
	    }
	    return params[A]*y;
	}

	else {
	   
	    return inverseCdf(random.nextDouble());
	}
    } 

	public String getEquation() {
		return new String(" f(x)= lambda^a * x^(a-1) * exp(-lambda*x) / Gamma(a) where alpha = shape parameter, must be > 0, lambda = scale parameter, must be > 0  x = the random variable,  must be >= 0");
	}

    /**
     * Methode: Main pour tester la classe
     */

    public static void main (String[] args) {
    
	GammaDistribution gd1 = new GammaDistribution(5d,1d/10d);
	GammaDistribution gd2 = new GammaDistribution(12d,12d);

	for (int i=0;i<1000;i++) {
	    System.out.println(gd1.inverseCdf(random.nextDouble()));
	    //System.out.println(gd1.nextDouble(2,3));
	}

	
	//System.out.println(gd1.rand());



    }
    
}
