/**
 * GenericDistribution : Definit les caracteristiques communes a toutes les lois (= distributions) utilisees
 * 
 * @author maria ladoue
 * @version 0.1, 06/08/02 
 */

package statistic.distribution;

import JSci.maths.statistics.NormalDistribution;

public abstract class GenericDistribution {

   // Listes de variables
    /**
     * Le nom de la distribution (e.g., Pareto, Weibull, ...)
     */
    public String name;

    /**
     * le nombre de parametres caracterisant la distribution
     */
    protected int paramsCount;

    /**
     * Tableaux des valeurs des differents parametres
     */
    public double params[];

    /**
     * Le nom des parametres
     */
    public String paramNames[];
    
    

    // Definition des principales valeurs statistics: cdf,pdf..
    
       /**
     * Retourne la valeur de la pdf(x)
     */
    public abstract double pdf(double x);
    
    /**
     * Returns the value of the cumulative distribution function at x
     */
    public abstract double cdf(double x);
    
    /**
     * Returns the mean of the distribution
     */
    public abstract double mean();
  
    /**
     * Returns the variance of the distribution
     */
    public abstract double var();
    
    /**
     *  Return a random value 
     */
    //Methode a definir pour chaque distribution permettant d'obtenir un tirage
    public double rand() {
	return (double)(this.rand());
    }
       
    protected NormalDistribution normalStandard = new NormalDistribution();
    
    /**
     * Returns the value of the normal standar cumulative distribution function at x
     */
    public double normalStandardCdf(double x) {
	return normalStandard.cumulative(x);
    }


    /**
     * Retourne la valeur du idx-ieme parametre
     */
    public double getParam(int idx) { return params[idx]; }

    /**
     * Positionne la valeur d'un ou de plusieurs parametres
     */
    public void setParam(int unParam,double uneValeur) throws IllegalArgumentException {
	if ( unParam < paramsCount ) {
	    if ( Double.compare(0,uneValeur) < 0 ) {
		params[unParam] = uneValeur;
	    } 
	    else { 
		throw new IllegalArgumentException( getParamName(unParam) + " doit etre superieur a zero." + uneValeur );}
	}
	else { throw new IllegalArgumentException( "Une loi "+ getName() +" est fonction que de " + paramsCount + " parametre." ) ;}
    } 

    public void setParams(double[] desParams) throws IllegalArgumentException {
	
	if ( desParams.length == getParamsCount() &&hasValidParam(desParams) ) {
	    for (int i=0;i<paramsCount;i++) {
		if ( desParams[i]>0 ) {
		    params[i]=desParams[i];
		}
		else { 
			throw new IllegalArgumentException("le parametre " + getParamName(i) + " doit etre positif." ); }
	    }
	}
	else { 
		throw new IllegalArgumentException("Le nombre de parametre de la loi de " + getName() + " est " + paramsCount ); }
    } 

     /**
     * Retourne et positionne le nom de la loi
     */
    public String getName() { return name; }

    protected void setName(String name) { this.name = name;}

	// Make it abstract so subclasses must implement it !
 	public abstract String getEquation();
    
    /*
     * Check that the parameters are valid for the distribution.
     */
	public abstract boolean hasValidParam(double[] desParams);
	
    /**
     * Retourne et positionne le nombre de parametre de la distribution
     */
    public int getParamsCount() { return paramsCount; }
    
    protected void setParamsCount(int unNbParams) { this.paramsCount = unNbParams; }

    
    // Methodes a probleme
    public double[] getParams()  { return params;}

    protected void setParamsName(String names[]) { paramNames = names;}

    protected void setParamName(int idx, String pName) { paramNames[idx] = pName;}

    public String[] getParamsNames()  { return paramNames;}

    public String getParamName(int idx)  { return paramNames[idx];}
}
