/**
 * GeometricDistribution : Definition de la loi Normale
 *
 * @author maria ladoue
 * @version 0.1, 30/08/02 
 */

package statistic.distribution;
  
import java.lang.Math;

import statistic.*; 

public class GeometricDistribution extends GenericDiscreteDistribution implements RandomGenerator {

    /**    
     * Equation de la densite: (probability density)<BR> 
     *  f(x) =  p*(1-p)^(x-1)
     *     where p = probability of success in one trial
     */

	private static int P = 0;
    final public boolean cdf = false;

    public GeometricDistribution() {
        paramsCount = 1;
        name = new String("Geometric");
        paramNames = new String[paramsCount];
        paramNames[P] = new String("p");
        params = new double[paramsCount];
    }

    public GeometricDistribution(double p) throws IllegalArgumentException {
        this();
        if (p > 0 && p<=1) {
            params[P] = p;
        } 
	else {
            throw new IllegalArgumentException("p must be greater than 0 and smaller or equal to 1.");
        }
    }

    public GeometricDistribution(double[] params) throws IllegalArgumentException {
        this();
        if (hasValidParam(params))
        	setParams(params);
		else
			throw new IllegalArgumentException("p must be greater than 0 and smaller or equal to 1.");
    }

	public boolean hasValidParam(double[] desParams) {
		if (desParams[P] > 0 && desParams[P]<=1)
			return true;
		else
			return false;
	}

    public void setParams(double[] par) throws IllegalArgumentException {
        if (par.length >= paramsCount && par[P] > 0 &&  par[P] <= 1) {
            params[P] = par[P];
        } 
	else {
            throw new IllegalArgumentException("p must be greater than 0 and smaller or equal to 1.");
        }
    }

    public void setParam(int idx, double p) throws IllegalArgumentException {
        if (idx < paramsCount && p > 0 && p<=1) {
            params[idx] = p;
        } 
	else {
            throw new IllegalArgumentException("p must be greater than 0 and smaller or equal to 1.");
        }
    }

  

    /**
     * Returns the value of the probability distribution function of x
     *   Prob(X=x)  for x=1,2,...
     */
    public double pdf(int x) {
        if (x >= 1) {
            return Math.pow(1d - params[P], x - 1) * params[P];
        } 
	else { return 0d; }
    }

 
    /**
     * Returns the value of the cumulative distribution function of x
     *    Prob(X<=x) for x=1,2,...
     */

    public double cdf(int x) {
	return 1d - Math.pow( (1-params[P]), x );
    }

 
    /**
     * Returns the mean of the distribution
     */
    public double mean() {
        return (1.0d - params[P]) / params[P];
    }
 
    /**
     * Returns the variance of the distribution
     */
    public double var(){
        return ((1.0 - params[P]) / Math.pow(params[P], 2.0d));
    }
 
    public int randInt() {
	int ans = 1; 
	if (params[P]==1) {
	    return 1;
	}
	else {
	    while (true) {
		double u = random.nextDouble();
		if ( u > params[P] ){
		    ans = ans+1;
		}
		else {
		    return ans;
		}
	    }
	}
    }

 public double getSumCdf(int k)   throws IllegalArgumentException {
 return 0;	
 }

	public String getEquation() {
		return new String("f(x) =  p*(1-p)^(x-1) where p = probability of success in one trial");
	}
	
    /**
     * Methode: Main pour tester la classe
     */
    
    public static void main (String[] args) {
	GeometricDistribution gd1 = new GeometricDistribution(0.5);
	System.out.println(gd1.randInt());
	System.out.println(gd1.cdf(4));
    }



}
