/**
 * PoissonDistribution : Definition de la loi de Poisson
 *
 * @author
 * @version 0
 */

package statistic.distribution;

import java.lang.Math;
import JSci.maths.SpecialMath;

import statistic.*;

public class PoissonDistribution extends GenericDiscreteDistribution implements RandomGenerator {


final public boolean cdf = false;

    /**
     *  Equation de la densite : <BR>
     *  f(x) = lambda^x * exp(-lambda) / x!<BR>
     *     where lambda = expected number of events per time period <BR>
     */

    private static int LAMBDA = 0;
    
    public PoissonDistribution () {
		paramsCount = 1;
		name = new String("Poisson");
		paramNames = new String[paramsCount];
		paramNames[LAMBDA] = new String("lambda");
		params = new double[paramsCount];
		params[LAMBDA] = 1.0d;
    }

	/**
	 * Method PoissonDistribution.
	 * @param lambda
	 * @throws IllegalArgumentException lambda must be > 0
	 */
    public PoissonDistribution ( double lambda ) throws IllegalArgumentException {
		this();
		if (lambda > 0 && lambda != Double.POSITIVE_INFINITY) {
	    	params[LAMBDA] = lambda;
		} 
		else {
		    throw new IllegalArgumentException("PoissonDistribution.constructor : lambda <= 0: " + lambda);
		}
    }

    public PoissonDistribution (double[] params) throws IllegalArgumentException {
		this();
		if (hasValidParam(params))
				setParams(params);
		else
			throw new IllegalArgumentException("PoissonDistribution.constructor : lambda <=0 || = infinity: " + params[LAMBDA]);
    }

	public boolean hasValidParam(double[] desParams) {
		if (desParams[LAMBDA]  > 0 && desParams[LAMBDA]  != Double.POSITIVE_INFINITY)
			return true;
		else
			return false;
	}
    // Definition de quelques methodes

    public void setParams(double params[]) throws IllegalArgumentException {
		if (params.length >= paramsCount && params[LAMBDA] > 0) {
	    	this.params[LAMBDA] = params[LAMBDA];
		} 
		else {
	    	throw new IllegalArgumentException("PoissonDistribution.setParams : lambda <= 0: "+ params[LAMBDA] );
		}
    }

    public void setParam(int idx, double p) throws IllegalArgumentException {
		if (idx < paramsCount && p > 0) {
	    	params[idx] = p;
		}
		 else {
	    	throw new IllegalArgumentException("PoissonDistribution.setParam : lambda <= 0: "+ params[LAMBDA] );
		}
    }   
    
    
    /**
     * Method pdf : Returns the value of the probability distribution function at x
     */
    public double pdf(double x) {
	if (x >= 0.0) { 
	    return Math.pow(params[LAMBDA],x) * Math.exp(-params[LAMBDA]) / SpecialMath.gamma(x+1);
	}
	else { return 0.0; }
    }
    

    /**
     * Method cdf : Returns the value of the cumulative distribution function at x
     */
    public double cdf(int x) {
	double sum=0.0;
	if (x > 0.0) {
	    for(double i=0.0; i <= Math.floor(x); i++) {
		sum = sum + Math.pow( params[LAMBDA],i) / 
		    SpecialMath.gamma(i+1);   // Correspond a factorielle car gamma(n)=(n-1)!
	    }
	}
	return sum*Math.exp(-1d*params[LAMBDA]);
    }



    /**
     * Method inverseCdf : Return the value of Inverse Function at x 
     * @param x with x>=0 and x<=1
     */
    public double inverseCdf(double x)  { return 0d; } //TO Do

    public double mean() { return params[LAMBDA];  }

    public double var() { return params[LAMBDA];  }
   

    public int randInt() {

	double a = Math.exp(-params[LAMBDA]);
	int n = 0;
	double u = random.nextDouble();
	while(u>a) {
	    u = u * random.nextDouble();
	    n++;
	}
	return n;
    }


public double getSumCdf(int k)   throws IllegalArgumentException {  return 0;	 }  // To do

	public String getEquation() {
		return new String(" f(x) = lambda^x * exp(-lambda) / x!  where lambda = expected number of events per time period");
	}
	
    /**
     * Methode: Main pour tester la classe
     */
    
    public static void main (String[] args) {
	//PoissonDistribution lp1 = new PoissonDistribution(10*(new SigmaDistribution(new SRDFunction(0.04))).mean());
	PoissonDistribution lp1 = new PoissonDistribution(1);
	//ParetoDistribution pd1 = new ParetoDistribution(1,2);
	for (int i=0;i<500;i++) {
	    //System.out.println(pd1.inverse(lp1.cdf((int)lp1.rand())));
	    //System.out.println(lp1.cdf((int)lp1.rand()));
	    System.out.println(lp1.rand());
	}

    }
}
