/**
 * UniformDistribution : Definition de la loi Uniforme
 *
 * @author maria ladoue
 * @version 0.1, 30/08/02 
 */
package statistic.distribution;

import java.lang.Math;

import statistic.*;

public class UniformDistribution extends GenericContinuousDistribution implements RandomGenerator {

    /**
     * Equation de la densite: (probability density)<BR>
     *  f(x) = 1/(b-a) if (x>a and x<b)
     */
	private static int A = 0;
	private static int B = 1;

    public UniformDistribution() {
        paramsCount = 2;
        name = new String("Uniform");
        paramNames = new String[paramsCount];
        paramNames[A] = new String("a");
        paramNames[B] = new String("b");
        params = new double[paramsCount];
        params[A] = 0.0d;
        params[B] = 1.0d;
    }

    public UniformDistribution(double a, double b)
                        throws IllegalArgumentException {
        this();

        if (b > a) {
            params[A] = a;
            params[B] = b;
        } else {
            throw new IllegalArgumentException("UniformDistribution.constructor : b < a");
        }
    }

    public UniformDistribution(double[] params)
                        throws IllegalArgumentException {
        this();
        if (hasValidParam(params))
        	setParams(params);
        else
			throw new IllegalArgumentException("UniformDistribution.constructor : b < a");
    }

	public boolean hasValidParam(double[] desParams) {
		if (desParams[A] >= desParams[B])
			return false;
		else
			return true;
	}

    public void setParams(double[] params)
                   throws IllegalArgumentException {

        if (params.length >= paramsCount && params[B] > params[A]) {
            this.params[A] = params[A];
            this.params[B] = params[B];
        } else {
            throw new IllegalArgumentException("UniformDistribution.setParams : b < a");
        }
    }

    public void setParam(int idx, double p)
                  throws IllegalArgumentException {

        if (idx < paramsCount && 
            ((p < params[B] && idx == A) || (p > params[A] && idx == B))) {
            params[idx] = p;
        } else {
            throw new IllegalArgumentException("UniformDistribution.setParam : b < a");
        }
    }

    /**
     * Returns the value of the probability distribution function at x
     */
    public double pdf(double x) {

        if (x < params[A] || x > params[B]) {

            return 0.0d;
        } else {

            return 1.0d / (params[B] - params[A]);
        }
    }

    /**
     * Returns the value of the cumulative distribution function at x
     */
    public double cdf(double x) {

        if (x < params[A]) {

            return 0.0d;
        } else if (x > params[B]) {

            return 1.0d;
        } else {

            return (x - params[A]) / (params[B] - params[A]);
        }
    }

    /**
     * Return the value of Inverse Function at x
     * @param x with x>=0 and x<=1
     */
    public double inverseCdf(double x)
                      throws IllegalArgumentException {

        if (x >= 0 && x <= 1) {
            return (params[A] + (params[B] - params[A]) * x);
        } 
        else {
            throw new IllegalArgumentException("UniformDistribution.inverseCdf : it is defined on [0,1].");
        }
    }

    public double mean() {

        return (params[B] + params[A]) / 2;
    }

    public double var() {

        return Math.pow((params[A] - params[B]), 2) / 12.0;
    }

    public double rand() {

        return random.nextDouble() * (params[B] - params[A]) + params[A];
    }
    
    public String getEquation() {
    	return new String(" f(x) = 1/(b-a) if (x>a and x<b)");
    }
}