package temp;

import java.net.*;
import javax.swing.*;
import java.util.*;


import traffic.*;
import utils.*;

/**
 * GestionUDPClient
 * 
 * Cration et gestion d'un client pour l'application GestionUDP.
 * <br><u><b>Remarque importante</b></u> : l'envoi de messages correspond  un stockage dans la file d'envoi.
 * La barre de progression indique donc la progression du stockage des messages dans la file.
 * Ainsi, la partie du programme envoyant les messages est excute quasiment instantanment.
 */


public class GestionUDPClient extends Thread implements UDPSocketReader {

    //Identification du serveur sur lequel se connecter
    protected String idServeur;
    //Port sur lequel se connecter
    protected int portServeur;
    //Port du client
    protected int portClient;
    //La classe parente
    protected GestionUDP gestionUDP;
    //Le JTextArea pour les messages concernant l'tat de la connexion
    protected JTextArea textAreaEtatConnexion;
    //Le JTextArea pour les messages concernant l'excution de l'application
    protected JTextArea textAreaEtatApplication;
    //La barre de progression de l'application
       protected JProgressBar progressBar;
    //La taille des buffers de la "connexion" UDP
    protected int tailleBufferReception, tailleBufferEmission;
    //Taille du buffer pour la rception de paquets; voir DatagramPacket(byte[]buf, intlength)
    protected int tailleBufferRceptionPaquet;
    //Le client
    protected UDPClient client;
    //Le Thread de lecture sur le socket
    protected SocketLecture socketLecture;
    //Le Thread d'criture sur le socket
    protected SocketEcriture socketEcriture;
    
    /**
     * Cration d'un client.
     * @param gestionUDP La classe parente
     * @param textAreaEtatConnexion La JTextArea pour afficher les messages concernant l'tat de la connexion
     * @param textAreaEtatApplication La JTextArea pour afficher les messages concernant l'excution de l'application
     * @param progressBar La barre de progression de l'application
     */

    public GestionUDPClient(GestionUDP gestionUDP, JTextArea textAreaEtatConnexion, JTextArea textAreaEtatApplication, JProgressBar progressBar){
	this.gestionUDP = gestionUDP;
	this.textAreaEtatConnexion = textAreaEtatConnexion;
	this.textAreaEtatApplication = textAreaEtatApplication;
	this.progressBar = progressBar;
	boolean incorrect = true;
	boolean continuer = true;
	while (incorrect && continuer){
	    continuer=demanderInfoClient();
	    if (continuer){
		try {
		    client=new UDPClient(idServeur,portServeur,portClient,tailleBufferReception,tailleBufferEmission,this,tailleBufferRceptionPaquet,2);
		}
		catch(UnknownHostException e) {
			 System.out.println(e.toString()); 
			 }
		catch(SocketException e) {
			System.out.println(e.toString());
			}
		socketEcriture=client.getThreadEcriture();
		socketLecture=client.getThreadLecture();
		if (socketLecture==null)
		    textAreaEtatConnexion.append("Serveur inconnu : "+idServeur+"\n");
		else 
		    incorrect=false;
	    }
	}
	if (continuer){
	    gestionUDP.accsMenuClient();
	    socketLecture.setUDPSocketReader(this);
	    this.start();
	}
    }


    /**
     * Implantation.
     */
    public synchronized void wakeUp(){
	notify();
    }

    /**
     * Appel  la mthode wait() de la classe Object.
     */
    public synchronized void attendre(){
	try{
	    wait();
	}
	catch (InterruptedException e) {
	    System.out.println(e.toString());
	}
    }
    
    /**
     * Demande les informations concernant les paramtres du client  l'utilisateur.
     * @return un boolen concernant la faon dont a t ferme la boite de dialogue par l'utilisateur.
     */
    protected boolean demanderInfoClient(){
	Vector v1=new Vector();
	v1.add("Nom/Adresse IP du serveur");
	v1.add("Port du serveur");
	v1.add("Port du client");
	v1.add("Taille du buffer pour la rception des datagrammes");
	v1.add("Taille du buffer pour l'mission des datagrammes");
	v1.add("Taille du buffer pour la rception de paquets (34 min)");
	Vector v2=new Vector(); // Valeur par defaut des differents parametre
	v2.add("kawa");
	v2.add("5000");
	v2.add("4900");
	v2.add("1000");
	v2.add("1000");
	v2.add("34");
	BoiteSaisies f=new BoiteSaisies(gestionUDP,"Paramtres du client",v1,v2);
	Vector v3=f.getValeursChamps();
	idServeur=(String)(v3.get(0));
	String s;
	try {
	    s=(String)(v3.get(1));
	    portServeur = Integer.parseInt(s);
	    s=(String)(v3.get(2));
	    portClient = Integer.parseInt(s);
	    s=(String)(v3.get(3));
	    tailleBufferReception = Integer.parseInt(s);
	    s=(String)(v3.get(4));
	    tailleBufferEmission = Integer.parseInt(s);
	    s=(String)(v3.get(5));
	    tailleBufferRceptionPaquet = Integer.parseInt(s);
	}
	catch (Exception e) {
	    System.out.println(e.toString());
                }
	return f.getEtat();
    }

	public void setClientInformations(String serverName, int serverPort, int clientPort) {
		idServeur = serverName;
		portServeur = serverPort;
		portClient = clientPort;
		tailleBufferReception= 1000;
		tailleBufferEmission = 1000;
		tailleBufferRceptionPaquet = 34; 
	}

    /**
     * Afficher les ventuels messages concernant la connexion.
     */
    public void rafraichir(){
	while (!client.fileMessageConnexionVide()){
	    textAreaEtatConnexion.append(client.getPremierMessageConnexion()+"\n");
	}
    }
    
    /**
     * Implantation de la mthode run de la classe Thread.
     * Attend les demandes de gestions du serveur.
     */
    public void run(){
    	
    	System.out.println("UDP client ready....");
    	 
	while (!socketLecture.estTermin()){
	    if (socketLecture.pasDeMessagesReu()){
		attendre();
	    }
        else{
			    String fonction=socketLecture.premierMessageReu();
			    //System.out.println("GestionUDPClient.run :" + fonction);
			    if (fonction.equals("pingpong"))
				gestionPingPong();
			    if (fonction.equals("envoidemessages"))
				gestionEnvoiMessages();
				 if (fonction.equals("scenarioplayer"))
				gestionScenarioPlayer();
                        }
	}
                attendre(2000);
                rafraichir();
                gestionUDP.accsMenuQuitter();
    }

    /**
     * Arrte le processus pendant une dure dtermine ou jusqu' un notify.
     * @param milli dure en millisecondes
     */
    private synchronized void attendre(int milli){
                try{
		    wait(milli);
                }
                catch (InterruptedException e) {
		    System.out.println("GestionUDPClient:attendre"+e.toString());
                }
    }
    
        /**
         * Ferme la connexion avec le serveur.
         */
    public void fermerConnexion(){
		socketEcriture.demandeFermetureSocket();
		attendre(100);
		rafraichir();
		gestionUDP.accsMenuQuitter();
    }
    
    //Gestion ping pong.
    private void gestionPingPong(){
	while (socketLecture.pasDeMessagesReu()){
	    attendre();
	}
	//on recoit le nombre de ping-pong  effectuer
	String param=socketLecture.premierMessageReu();
	int nbPingPong=0;
	try {
	    nbPingPong = Integer.parseInt(param);
	}
	catch (Exception e) {
	    System.out.println(e.toString());
	}
	textAreaEtatApplication.append("*\t*\t*\nGestion ping-pong: "+nbPingPong+" ping-pong  effectuer.\n");
	progressBar.setStringPainted(true);
	float f=nbPingPong;
	f=100/f;
	//On effectue les ping-pong,  savoir: rception d'un message, envoi d'un accus de rception, ...
	for (float i=0;i<nbPingPong;i++){
	    progressBar.setValue((int)(f*i));
	    while (socketLecture.pasDeMessagesReu()){
		attendre();
	    }
	    socketEcriture.messageAEnvoyer(socketLecture.premierMessageReu());
	}
	progressBar.setValue(0);
	progressBar.setStringPainted(false);
    }
 
    //Gestion envoi de messages.
    private void gestionEnvoiMessages(){
	while (socketLecture.pasDeMessagesReu()){
	    attendre();
	}
	//On rcupre le nombre de messages que l'on va recevoir
	String param = socketLecture.premierMessageReu();
	int nbMessages=0;
	try {
	    nbMessages = Integer.parseInt(param);
	}
	catch (Exception e) {
	    System.out.println(e.toString());
	}
	
	textAreaEtatApplication.append("*\t*\t*\nGestion envoi de messages: "+nbMessages+" messages  recevoir.\n");
	progressBar.setStringPainted(true);
	float f=nbMessages;
	f=100/f;
	String message;
	int[] tpsArriveMessages=new int[nbMessages];
	Date date=new Date();
	//temps initial
	//on va dcrmenter les temps d'arrive des messages par le temps initial
	//le temps d'arrive des messages commencera donc  zro
	//de plus il sera toujours suffisamment petit pour pouvoir etre stocke dans un "int" au lieu d'un "long"
	//les "long" sont en effet trs problmatiques  manipuler avec Java
	long init=date.getTime();
	//rception des messages
	for (int i=0;i<nbMessages;i++){
	    progressBar.setValue((int)(f*i));
	    while (socketLecture.pasDeMessagesReu()){
		attendre();
	    }
	    date=new Date();
	    tpsArriveMessages[i]=(int)(date.getTime()-init);
	    message=socketLecture.premierMessageReu();
	}
	progressBar.setValue(0);
	progressBar.setStringPainted(false);
	
	textAreaEtatApplication.append(">Renvoi des rsultats du gestion envoi de messages\n");
	progressBar.setStringPainted(true);
	//envoi des temps d'arrive des messages au serveur
	for (int i=0;i<nbMessages;i++){
	    progressBar.setValue((int)(f*i));
	    message=tpsArriveMessages[i]+"";
	    socketEcriture.messageAEnvoyer(message);
	}
	progressBar.setValue(0);
	progressBar.setStringPainted(false);
    }
    
    
     //Gestion scenarioPlayer
    private void gestionScenarioPlayer(){
    	
	while (socketLecture.pasDeMessagesReu()){
	    attendre();
	}
	//On rcupre le nombre de messages que l'on va recevoir
	String param = socketLecture.premierMessageReu();
	int nbMessages=0;
	try {
	    nbMessages = Integer.parseInt(param);
	}
	catch (Exception e) {
	    System.out.println(e.toString());
	}
	
	textAreaEtatApplication.append("\nBench Scenario Player: " + nbMessages+" messages  recevoir.\n");
	progressBar.setStringPainted(true);
	float f = nbMessages;
	f=100/f;
	String message;
	int[] tpsArriveMessages = new int[nbMessages];
	Date date=new Date();
	//temps initial
	//on va dcrmenter les temps d'arrive des messages par le temps initial
	//le temps d'arrive des messages commencera donc  zro
	//de plus il sera toujours suffisamment petit pour pouvoir etre stocke dans un "int" au lieu d'un "long"
	//les "long" sont en effet trs problmatiques  manipuler avec Java
	long init=date.getTime();
	//rception des messages
	for (int i=0;i<nbMessages;i++){
	    progressBar.setValue((int)(f*i));
	    while (socketLecture.pasDeMessagesReu()){
		attendre();
	    }
	    date=new Date();
	    tpsArriveMessages[i]=(int)(date.getTime()-init);
	    message = socketLecture.premierMessageReu();
	}
	progressBar.setValue(0);
	progressBar.setStringPainted(false);
	
	textAreaEtatApplication.append(">Renvoi des rsultats du bench Scenario Player\n");
	progressBar.setStringPainted(true);
	//envoi des temps d'arrive des messages au serveur
	for (int i=0;i<nbMessages;i++){
	    progressBar.setValue((int)(f*i));
	    message = tpsArriveMessages[i]+"";
	    socketEcriture.messageAEnvoyer(message);
	}
	progressBar.setValue(0);
	progressBar.setStringPainted(false);
    }
    
    
}
