/**
 * GestionUDPS2 :
 * Avec la premire machine connecte.
 * Nombre de ping-pong paramtrable.
 * Taille des messages paramtrable.
 */

package temp;

import javax.swing.*;
import java.util.*;
import java.io.FileNotFoundException;
import java.net.*;

import traffic.*;
import utils.*;

public class GestionUDPS2 extends Thread implements UDPSocketReader {

    protected GestionUDP gestionUDP;
    protected UDPServer server;
    protected JTextArea textAreaEtatApplication;
    protected JProgressBar progressBar;
    protected int longueurMessages;
    protected int nbPingPong;
    //Tableau contenant les rsultats du gestion.
    //Contient pour chaque ping-pong sa dure.
    protected int[] rsultatsGestion;

    /**
     * Initialise le Thread.
     * @param gestionUDP La classe parente
     * @param serveur L'instance du serveur
     * @param textAreaEtatApplication La JTextArea pour afficher les messages concernant l'excution de l'application
     * @param progressBar La barre de progression de l'application
     */

    public GestionUDPS2(GestionUDP gestionUDP, UDPServer server, JTextArea textAreaEtatApplication, JProgressBar progressBar) {
	this.gestionUDP=gestionUDP;
	this.server=server;
	this.textAreaEtatApplication=textAreaEtatApplication;
	this.progressBar=progressBar;
	if (demanderParamtres()) { 
	    this.start(); }
        }
    
    /**
     * Implantation.
     */
    public synchronized void wakeUp(){
	notify();
    }
    
    /**
     * Appel  la mthode wait() de la classe Object.
     */
    protected synchronized void attendre(){
	try{
	    wait();
	}
	catch (InterruptedException e) {
	    System.out.println(e.toString());
	}
    }
    
    /**
     * Gestion ping pong.
     */
    public void run(){

	if ( server.nbConnection()==0 ){
	    textAreaEtatApplication.append("*\t*\t*\n Il n'y aucun client de connect.\n");
	}
	else{
	    textAreaEtatApplication.append("*\t*\t*\nGestion ping-pong.\n");
	    SocketLecture sl=server.getThreadLecture(0);
	    sl.setUDPSocketReader(this);
	    SocketEcriture se=server.getThreadEcriture(0);
	    se.messageAEnvoyer("pingpong");
	    se.messageAEnvoyer(nbPingPong+"");
	    Date date;
	    String accus;
	    //dure ping-pong le plus rapide
	    long min=1000000;
	    //dure ping-pong le plus lent
	    long max=0;
	    long somme=0;
	    ChaineAleatoire ca=new ChaineAleatoire(longueurMessages);
	    progressBar.setStringPainted(true);
	    float f=nbPingPong;
	    f=100/f;
	    rsultatsGestion=new int[nbPingPong];
	    //Ralisation des ping-pong,  savoir: envoi d'un message, rception de l'accus de rception, ...
	    for (float i=0;i<nbPingPong;i++){
		progressBar.setValue((int)(f*i));
		date=new Date();
		long time=date.getTime();
		se.messageAEnvoyer(ca.gnrerChane());
		while (sl.pasDeMessagesReu()){
                                        attendre();
		}
		accus=sl.premierMessageReu();
		date=new Date();
                                //calcul de la dure du ping-pong
		time=date.getTime()-time;
                                //sauvegarde de la dure du ping-pong
		rsultatsGestion[(int)i]=(int)time;
                                //statistiques
		if (time<min)
		    min=time;
		if (time>max)
		    max=time;
		somme=somme+time;
	    }
	    progressBar.setValue(0);
	    progressBar.setStringPainted(false);
	    
	    statistiques(min,max,somme,nbPingPong);
	    sauvegardeRsultats((int)min,(int)max);
	}
    }
    
    //Statistiques
    protected void statistiques(long min, long max, long somme, int nbPingPong) {
	long moyenne=somme/nbPingPong;
	textAreaEtatApplication.append(">Ping pong minimum: "+min+" millisecondes.\n");
	textAreaEtatApplication.append(">Ping pong maximum: "+max+" millisecondes.\n");
	textAreaEtatApplication.append(">Ping pong moyen: "+moyenne+" millisecondes.\n");
    }
    
    //Sauvegarde des rsultats dans un fichier sur le disque pour un traitement ultrieur.
    protected void sauvegardeRsultats(int min, int max){
	SocketLecture sl=server.getThreadLecture(0);
	DatagramSocket socket=sl.getSocket();
	int tailleBufferReception=0;
	int tailleBufferEmission=0;
	try {
	    tailleBufferReception=socket.getReceiveBufferSize();
	    tailleBufferEmission=socket.getSendBufferSize();
	}
	catch (SocketException e) {
	    System.out.println(e.toString());
	}
	GestionUDPS2S b=new GestionUDPS2S(rsultatsGestion,min,max,longueurMessages,nbPingPong,tailleBufferReception,tailleBufferEmission,sl.getTailleBufferRceptionPaquet());
	textAreaEtatApplication.append(">>Les rsultats du gestion ont t sauvegards dans le fichier : "+b.getNomFichier()+"\n");
        }
    
    /**
     * Demande  l'utilisateur les paramtres pour effectuer le gestion
     * @return un boolen concernant la faon dont a t ferme la boite de dialogue par l'utilisateur.
     */

    protected boolean demanderParamtres(){
	Vector v1=new Vector();
	v1.add("Nombre de ping-pong");
	v1.add("Longueur des messages");
	Vector v2=new Vector();
	v2.add("1000");
	v2.add("20");
	BoiteSaisies f=new BoiteSaisies(gestionUDP,"Paramtres gestion ping-pong",v1,v2);
	Vector v3=f.getValeursChamps();
	try {
	    nbPingPong = Integer.parseInt((String)(v3.get(0)));
	    longueurMessages = Integer.parseInt((String)(v3.get(1)));
	}
	catch (Exception e) {
	    System.out.println(e.toString());
	}
	return f.getEtat();
    }

    /**
     * Methode main: destinee a tester al classe courante
     */

   public static void main(String args[]) throws FileNotFoundException {

       JTextArea textAreaEtatApplication = new JTextArea();
       JProgressBar progressBar = new JProgressBar();
       JTextArea textAreaEtatConnexion = new JTextArea();
       GestionUDP gestionUDP = new GestionUDP();
       UDPServer server = new UDPServer(1500, 256, 256, new GestionUDPServeur(gestionUDP,textAreaEtatConnexion,textAreaEtatApplication,progressBar), 34);
     
       GestionUDPS2 b = new GestionUDPS2(gestionUDP , server, textAreaEtatApplication, progressBar );
    }


}
