
/**
 * Bench envoi de messages.
 * Avec la premire machine connecte.
 * Nombre de messages paramtrable.
 * Taille des messages paramtrable.
 * <br><u><b>Remarque importante</b></u> : l'envoi de messages correspond  un stockage dans la file d'envoi.
 * La barre de progression indique donc la progression du stockage des messages dans la file.
 * Ainsi, la partie du programme envoyant les messages est excute quasiment instantanment.
 */
package temp;

import javax.swing.*;
import java.util.*;
import java.net.*;

import traffic.*;
import utils.*;

public class GestionUDPS3 extends Thread implements UDPSocketReader {

        protected GestionUDP gestionUDP;
        protected UDPServer server;
        protected JTextArea textAreaEtatApplication;
        protected JProgressBar progressBar;
        protected int longueurMessages;
        protected int nbMessages;

        /**
         * Initialise le Thread.
         * @param benchUDP La classe parente
         * @param serveur L'instance du serveur
         * @param textAreaEtatApplication La JTextArea pour afficher les messages concernant l'excution de l'application
         * @param progressBar La barre de progression de l'application
         */
        public GestionUDPS3(GestionUDP gestionUDP, UDPServer serveur, JTextArea textAreaEtatApplication, JProgressBar progressBar){
                this.gestionUDP = gestionUDP;
                this.server = serveur;
                this.textAreaEtatApplication=textAreaEtatApplication;
                this.progressBar=progressBar;
                if (demanderParamtres()) { this.start(); }
        }

        /**
         * Implantation.
         */
        public synchronized void wakeUp(){
                notify();
        }

        /**
         * Appel  la mthode wait() de la classe Object.
         */
        protected synchronized void attendre(){
	    try{
		wait();
	    }
	    catch (InterruptedException e) {
		System.out.println(e.toString());
	    }
        }

        /**
         * Bench envoi de messages.
         */
        public void run(){
	    if (server.nbConnection()==0){
		textAreaEtatApplication.append("*\t*\t*\nIl n'y aucun client de connect.\n");
	    }
	    else{
		textAreaEtatApplication.append("*\t*\t*\nBench envoi de messages.\n");
		SocketEcriture se=server.getThreadEcriture(0);
		SocketLecture sl=server.getThreadLecture(0);
		sl.setUDPSocketReader(this);
		se.messageAEnvoyer("envoidemessages");
		se.messageAEnvoyer(nbMessages+"");
		ChaineAleatoire ca=new ChaineAleatoire(longueurMessages);
		progressBar.setStringPainted(true);
		float f=nbMessages;
		f=100/f;
		//envoi des messages au client
		for (float i=0;i<nbMessages;i++){
		    progressBar.setValue((int)(f*i));
		    se.messageAEnvoyer(ca.gnrerChane());
		}
		progressBar.setValue(0);
		progressBar.setStringPainted(false);
		
		textAreaEtatApplication.append(">Rception des rsultats du bench.\n");
		progressBar.setStringPainted(true);
		int[] tpsArriveMessages=new int[nbMessages];
		String message;
		//rception des temps d'arrive des messages chez le client
		for (int i=0;i<nbMessages;i++){
		    progressBar.setValue((int)(f*i));
		    while (sl.pasDeMessagesReu()){
			attendre();
		    }
		    message=sl.premierMessageReu();
		    try {
			tpsArriveMessages[i] = Integer.parseInt(message);
		    }
		    catch (Exception e) {
			System.out.println(e.toString());
		    }
		}
		progressBar.setValue(0);
		progressBar.setStringPainted(false);
		
		sauvegardeRsultats(tpsArriveMessages);
	    }
        }
 
    //Sauvegarde des rsultats dans un fichier sur le disque pour un traitement ultrieur.
    protected void sauvegardeRsultats(int[] rsultatsBench){
	SocketLecture sl=server.getThreadLecture(0);
	DatagramSocket socket=sl.getSocket();
	int tailleBufferReception=0;
	int tailleBufferEmission=0;
	try {
	    tailleBufferReception=socket.getReceiveBufferSize();
	    tailleBufferEmission=socket.getSendBufferSize();
	}
	catch (SocketException e) {
	    System.out.println(e.toString());
	}
	GestionUDPS3S b=new GestionUDPS3S(rsultatsBench,longueurMessages,tailleBufferReception,tailleBufferEmission,sl.getTailleBufferRceptionPaquet());
	textAreaEtatApplication.append(">>Les rsultats du bench ont t sauvegards dans le fichier : "+b.getNomFichier()+"\n");
        }
    
    /**
     * Demande  l'utilisateur les paramtres pour effectuer le bench
     * @return un boolen concernant la faon dont a t ferme la boite de dialogue par l'utilisateur.
     */
    protected boolean demanderParamtres(){
	Vector v1=new Vector();
	v1.add("Nombre de messages");
	v1.add("Longueur des messages");
	Vector v2=new Vector();  // Valeur par defaut
	v2.add("1000");
	v2.add("20");
	BoiteSaisies f = new BoiteSaisies(gestionUDP,"Paramtres bench envoi de messages",v1,v2);
	Vector v3=f.getValeursChamps();
                try {
		    nbMessages = Integer.parseInt((String)(v3.get(0)));
		    longueurMessages = Integer.parseInt((String)(v3.get(1)));
                }
                catch (Exception e) {
		    System.out.println(e.toString());
                }
                return f.getEtat();
    }
}
