/**
 * GestionUDPServeur :  Cration et gestion d'un serveur pour l'application GestionUDP.
 *
 * @author maria ladoue
 * @version 0.2, 06/08/02
 */

package temp;


import java.io.*;
import javax.swing.*;
import java.util.*;


import traffic.*;
import utils.*;


public class GestionUDPServeur implements UDPSocketReader {

    //Port du serveur
    protected int port;
    //La classe parente
    protected GestionUDP gestionUDP;
    //Le JTextArea pour les messages concernant l'tat de la connexion
    protected JTextArea textAreaEtatConnexion;
    //Le JTextArea pour les messages concernant l'excution de l'application
    protected JTextArea textAreaEtatApplication;
    //La barre de progression de l'application
    protected JProgressBar progressBar;
    //La taille des buffers de la "connexion" UDP
    protected int tailleBufferReception, tailleBufferEmission;
    //Taille du buffer pour la rception de paquets (c.f.DatagramPacket(byte[]buf, intlength))
    protected int tailleBufferRceptionPaquet;
    //Le serveur
    protected UDPServer server;
   
    
    /**
     * Cration du serveur.
     * @param gestionUDP La classe parente
     * @param textAreaEtatConnexion La JTextArea pour afficher les messages concernant l'tat de la connexion
     * @param textAreaEtatApplication La JTextArea pour afficher les messages concernant l'excution de l'application
     * @param progressBar La barre de progression de l'application
     */ 
    public GestionUDPServeur(GestionUDP gestionUDP, JTextArea textAreaEtatConnexion, JTextArea textAreaEtatApplication, JProgressBar progressBar) throws FileNotFoundException  {
	this.gestionUDP = gestionUDP;
	this.textAreaEtatConnexion = textAreaEtatConnexion;
	this.textAreaEtatApplication = textAreaEtatApplication;
	this.progressBar = progressBar;
	if (demanderInfoServeur()){
	    gestionUDP.accsMenuServeur();
	    server = new UDPServer(port,tailleBufferReception,tailleBufferEmission,this,tailleBufferRceptionPaquet);
	}
    }

    /**
     * Implantation.
     */
    public synchronized void wakeUp(){
    }

    /**
     * Demande les informations concernant les paramtres du serveur  l'utilisateur.
     * @return un boolen concernant la faon dont a t ferme la boite de dialogue par l'utilisateur.
     */
    protected boolean demanderInfoServeur(){
	
	Vector v1=new Vector();
	v1.add("Port d'ecoute");
	v1.add("Taille du buffer pour la rception des datagrammes");
	v1.add("Taille du buffer pour l'mission des datagrammes");
	v1.add("Taille du buffer pour la rception de paquets (34 min)");
	Vector v2=new Vector();
	v2.add("5000");
	v2.add("1000");
	v2.add("1000");
	v2.add("34");
	BoiteSaisies f = new BoiteSaisies(gestionUDP,"Server parameters",v1,v2);
	Vector v3 = f.getValeursChamps();
	String s;
	try {
	    s=(String)(v3.get(0));
	    port = Integer.parseInt(s);
	    s=(String)(v3.get(1));
	    tailleBufferReception = Integer.parseInt(s);
	    s=(String)(v3.get(2));
	    tailleBufferEmission = Integer.parseInt(s);
	    s=(String)(v3.get(3));
	    tailleBufferRceptionPaquet = Integer.parseInt(s);
	}
	catch (Exception e) {
	    System.out.println(e.toString());
	}
	return f.getEtat();
    }
    
    /**
     * Afficher les ventuels messages concernant la connexion.
     */
    public void rafraichir(){
	while (!server.fileMessageConnexionVide()){
	    textAreaEtatConnexion.append(server.getPremierMessageConnexion()+"\n");
	}
    }
    
    /**
     * Ferme les connexions avec les clients et le ServerSocket.
     */
    public void fermerConnexions() {
    	
		int nb = server.nbConnection();
	    SocketEcriture se;
	    for (int i=0;i<nb;i++){
		se = server.getThreadEcriture(i);
		se.demandeFermetureSocket();
	    }
	    server.fermetureServeur();
	    attendre(100);
	    rafraichir();
	    gestionUDP.accsMenuQuitter();
    }
 
    /**
     * Arrte le processus pendant une dure dtermine ou jusqu' un notify.
     * @param milli dure en millisecondes
     */
    private synchronized void attendre(int milli){
	try{
	    wait(milli);
	}
	catch (InterruptedException e) {
	    System.out.println(e.toString());
	}
    }
    
    /**
     * Bench ping pong.
     */
    public void gestionPingPong(){
	GestionUDPS2 b = new GestionUDPS2(gestionUDP,server,textAreaEtatApplication,progressBar);
    }
    
    /**
     * Gestion envoi de messages.
     */
    public void gestionEnvoiMessages(){
	GestionUDPS3 b = new GestionUDPS3(gestionUDP,server,textAreaEtatApplication,progressBar);
    }
    
     /**
     * Gestion Scenario Player
     */
    public void gestionScenarioPlayer() throws FileNotFoundException {
	BenchUDPScenarioPlayer b = new BenchUDPScenarioPlayer(gestionUDP,server,textAreaEtatApplication,progressBar);
    }
}
