package test.model;

import java.io.*;
import java.util.*;

import multimedia.VideoExample;
import multimedia.codec.*;
import multimedia.model.MGInfModel;
import statistic.function.*;
import statistic.distribution.*;
import utils.*;
import utils.myPlotter.*;

/**
 * MGInfTest it's defined to test MGInfGeneration class
 * 
 * @author mladoue
 * @version 0.1, 25/10/02 
 */
public class MGInfTest extends GenericModelTest implements  VideoCodecExample, VideoExample {

  	protected MGInfModel model;
    protected CodecAV codec;
    protected String name;
    protected short unite;
    //protected String resultFileName;
    protected int N;
    
    protected static short BYTES = 0;
    protected static short KILOBYTES  = 1;
	protected  static short MEGABYTES  = 2;


public MGInfTest( MGInfModel model, int tpsSimul, String fichierReq, VideoCodec codec, String name, short unite, boolean graphic) throws IOException {
	  
  	   	this.model = model;
        this.tpsSimul = tpsSimul;	// en minutes
		resultFileName = fichierReq;
        this.codec = codec;
        this.name = name;
        this.unite = unite;
        this.graphic = graphic;
        
		N = getSlotNumber();
}

/**
 * Method getDateDistributionInSlot. Donne la quantite de donnee generee par slot dans le cas ou 
 * 												ou on considere une distribution poissonienne des donnees
 * @return int[]
 * @throws IOException
 */
 public int[] getDataDistributionInSlot() throws IOException {
        
        //Processus d'occupation de la file M/G/infini
        int[] slots = new int[N + 1];
        SortedMap ts1 = new TreeMap();
        int deadTime, sigmaIJ, sigmaXo;
        int A, poids, alive = 0, dead = 0;

        // Nombre de clients presents dans le systeme a To
        slots[0] = model.calculNbClients(0);
        alive = slots[0]; // Nombre de clients vivants a t=0

        if (slots[0] != 0) {

            for (int j = 1; j <= slots[0]; j++) {

                // Temps de service en slots de chacun des Xo (=j) clients presents a To
                sigmaXo = (int)(model.calculT(1));

                if (sigmaXo > 1) {

                    // Temps de fin de vie de chacun des j clients cree a To
                    deadTime = sigmaXo - 1;

                    if (ts1.containsKey(new Integer(deadTime))) {
                        poids = ((Integer)(ts1.get(new Integer(deadTime)))).intValue();
                        ts1.put(new Integer(deadTime), new Integer(poids + 1));
                    } 
                    else {
                        ts1.put(new Integer(deadTime), new Integer(1));
                    }
                } 
                else if (sigmaXo == 1) {
                    dead++;
                } // Nombre de clients qui meurent a t=0
            }
        } //end if (slots[0] != 0) {

        for (int i = 1; i <= N; i++) {
            A = model.calculNbClients(1);

            // Generation de nouveaux clients au slot i
            alive += A;
            slots[i] = alive - dead;

            if (A != 0) {
                for (int j = 1; j <= A; j++) {

                    // Generation du temps de service (en slots) du client j present sur le slot i
                    sigmaIJ = (int)(model.calculT(0));
                    deadTime = sigmaIJ + i - 1;

                    if (ts1.containsKey(new Integer(deadTime))) {
                        poids = ((Integer)(ts1.get(new Integer(deadTime)))).intValue();
                        ts1.put(new Integer(deadTime), new Integer(poids + 1));
                    } 
                    else {
                        ts1.put(new Integer(deadTime), new Integer(1));
                    }
                }
            } // end  if (A != 0 ) {

            // Pour supprimer les clients correspondant au slot courant (dont on n'a plus besoin)
            if (ts1.containsKey(new Integer(i))) {
                int newDead = ((Integer)(ts1.get(new Integer(i)))).intValue();
                ts1.remove(ts1.firstKey());
                dead += newDead;
            }
        } //  end for (int i=1;i<=N;i++) {
        
	return slots;
 }
 
 
/**
 * Donne la quantite de donnee generee par slot dans le cas ou 
 * 														ou on considere une distribution gamma ou lognormale... des donnees
 * @param slots
 * @throws IOException
 */
public int[] getDataSpecificDistributionInSlot(int[] slots) throws IOException {

        int[] slotNew = new int[N + 1];
        double dureeSlot = (double)tpsSimul*60 / (double)N; // en secondes
        double time = 0; //en secondes
        int errorNumber = 0;
         // Definit le nb de bytes deja envoye
        int nbBytesTotal = 0; // en bytes

        for (int l = 0; l <= N; l++) {

         // Transforme une distribution poissonienne (qui repond au parametre d'autocorrelation) en une autre distribution.
         try {
          double cdfPoisson = ( new PoissonDistribution(((SigmaDistribution)(model.getParameter(5))).mean())).cdf( slots[l] );
          GenericContinuousDistribution distributionTest = (GenericContinuousDistribution)(model.getParameter(2));

            // fournit le nb de KBytes qu'il faut generer pdt le slot
			slotNew[l] = (int)(distributionTest.inverseCdf(cdfPoisson));
			
			if (slotNew[l] == 0) { 
				errorNumber++;
				System.out.println("Error on data computation in slot: " + l + " in this case cdfPoisson is: " +  cdfPoisson);
				slotNew[l] = (int)(distributionTest.inverseCdf(cdfPoisson));
				System.out.println(" New value: " +  slotNew[l]) ;
				//slotNew[l] = Math.round((float)distributionTest.mean()); }
         }
         }
         catch (ArrayIndexOutOfBoundsException aioobe) { 
         	aioobe.printStackTrace();
         	System.out.println("index value:" + l + ", slots number: " + N);  //DEBUG
         }
			
			
          } // end of for
          if (errorNumber != 0) {
				System.out.println("There are: " + errorNumber + " errors, on a total of: " + N + " slots."); 
          }
          return slotNew;
}
 
public double[] rateEvolutionComputation(int[] newSlots) throws IOException {
	
	Vector rates = new Vector();
	double slotDuration = (double)tpsSimul*60 / (double)N; // on passe en secondes 
	for (int i=0;i<newSlots.length;i++) {
		rates.add(i, new Double((double)newSlots[i]/slotDuration * 8 / 1000));
	}
	
	return MyTableOperation.transformV2DT(rates);
}



 
/**
 * Method getSlotNumber. To calulate N: the number of slots corresponding to duration test
 * @return int
 */
public int getSlotNumber() {
	int tpsSimulS =  tpsSimul*60; // on le passe en secondes
	return  (int)(tpsSimulS*(((VideoCodec)codec).getFramesNumber()) / (((VideoCodec)codec).getCompression().getGOP().getLength()));
}


public void createResultFile(int[] slotValues) throws IOException {
	
	this.fichierReq = FileTest.testExistent( resultFileName, graphic);
	
	for (int i=0;i<slotValues.length;i++) {
		fichierReq.write(slotValues[i] + "\n");	
	}
	fichierReq.flush();
	fichierReq.close();
	System.out.println("File " + resultFile.getName() + " created!");
}


public void createResultFile(int[] slotValues, String fileName) throws IOException {
	
	this.fichierReq = FileTest.testExistent(fileName,graphic);
	
	for (int i=0;i<slotValues.length;i++) {
		fichierReq.write(slotValues[i] + "\n");	
	}
	fichierReq.flush();
	fichierReq.close();
	System.out.println("File " + fileName + " created!");
}


/**
 * Method traceRateEvolution
 * @param newSlots
 * @throws IOException
 */
public void traceRateEvolution(int[] newSlots) throws IOException {
	
	double[] rates = rateEvolutionComputation(newSlots);
	int[] ratesTransformed = MyTableOperation.transformDT2IT(rates);
	int N = getSlotNumber();
	OneGraphPlotter rp = new OneGraphPlotter();
	
	int YMaxRange = TestParameters.getMaximumIntValue(ratesTransformed);
	System.out.println("Max: " + YMaxRange);
	
	
	PlotterProperties.setYTick(rp.getSpecificPlot(0), YMaxRange);
	PlotterProperties.setXTick( rp.getSpecificPlot(0), N);
	rp.plotIntTableData( ratesTransformed, tpsSimul, "Througput (in kb/s) for "+ name, "Time (in mn)", "Throughput (in kb/s)", false, false);
} 
 


/**
 * Method traceDataPerSlotDistribution : this permit to plot an array of data
 * 											Here it is defined to plot data per slots
 * 
 * @param slotValues
 */
public void traceDataPerSlotDistribution(int[] slotValues) {
	
	OneGraphPlotter rp = new OneGraphPlotter();
	//PlotterProperties.setPlotProperties( rp.getSpecificPlot(0),700, 600, plotTitle, 0, TestParameters.getMaximumIntValue(data), 0, testDuration, XLabel, YLabel );
	//PlotterProperties.setBasicPlotProperties(rp.getSpecificPlot(0),700, 600,"Data per slot",  "slots", "Amount of data");
	PlotterProperties.setYTick(rp.getSpecificPlot(0), TestParameters.getMaximumIntValue( slotValues));
	PlotterProperties.setXTick(rp.getSpecificPlot(0), slotValues.length);
	
	//PlotterProperties.setSpecificPlotProperties( rp.getSpecificPlot(0), impulses, false, "none");
	rp.getSpecificPlot(0).addLegend( 0, "toto");
	rp.plotIntTableData(slotValues, (int)(tpsSimul/60), "Data per slot", "slots", "Amount of data", true, false);
}

/**
 * Method traceComparaisonOfDataPerSlotDistribution : this permit to compare throw a plot two array of data
 * 														Here it is defined to compare Xi and Yi
 * 
 * @throws IOException
 */
public void traceComparaisonOfDataPerSlotDistribution() throws IOException {
	int[] slotValues = this.getDataDistributionInSlot();
	int[] newSlotValues = this.getDataSpecificDistributionInSlot( slotValues );
	
	TwoGraphPlotter trs = new TwoGraphPlotter();
	trs.plotTwoIntTableData(slotValues, newSlotValues, tpsSimul);
	trs.setTitle("Comparaison between Xi ans Yi");
}

/**
 * Methode: Main
 */

    public static void main (String[] args) throws IOException {

	 String f1 = new String("MGInfGene");
     String f2 = new String("MGInfJPLow");
      
      
      /**  MGInfModel mgi1 = new MGInfModel(new SRDFunction(0.04),1d,new GammaDistribution(5d,1d/10d),new ExponentialDistribution(0.2d),new ExponentialDistribution(0.3d));
        
        MGInfTest mgig1 = new MGInfTest( mgi1,30,f1,MPEG4,new String("indefinite"),(short)0,false);
        
        System.out.println("Debut: " + new Date().toString());
        mgig1.createResultFile(mgig1.getDateDistributionInSlot());
		System.out.println("Fin: " + new Date().toString());
      */
      
		// Exemple Jurassik park Medium:   
	double[] coeffFuncjpm = { 0.3343, 0.6657 };
	 GenericFunction[] gftjpm = { new MarkovianFunction(0.0207), new SRDFunction(0.269) };
	 double[] coeffDistjpm = { 0.0413, 0.9587 };
	 GenericContinuousDistribution[] gdjpm = { new GammaDistribution(19.1288,1499.9), new LogNormalDistribution(9.5081,0.5425)};
	 GenericContinuousDistribution djpm = new MixedDistribution(gdjpm,coeffDistjpm);
	 MGInfModel mginfjpm = new MGInfModel(gftjpm,coeffFuncjpm, 1.0 , djpm, VideoExample.dSlot , VideoExample.dPacketSize);

	 MGInfTest mgigjpm= new MGInfTest( mginfjpm, 60,"JPMedium",MPEG4,new String("Jurassik park"),(short)0,false);
        
		   System.out.println("Debut: " + new Date().toString());
		   int[] slots = mgigjpm.getDataDistributionInSlot();
			int[] newSlots = mgigjpm.getDataSpecificDistributionInSlot(slots);
			mgigjpm.traceComparaisonOfDataPerSlotDistribution();
			//mgigjpm.traceRateEvolution(newSlots);
			//mgigjpm.traceDataPerSlotDistribution(newSlots);
			//mgigjpm.createResultFile(slots,"JPMediumXi");
			//mgigjpm.createResultFile(mgigjpm.getDataSpecificDistributionInSlot(slots),"JPMediumYi");
			System.out.println("Fin: " + new Date().toString());
	
			
	//	Exemple Bean	High quality
	/**
	 double[] coeffFuncbh = { 0.451, 0.549 };
	 GenericFunction[] gftbh = { new MarkovianFunction(0.129), new SRDFunction(0.108) };
	 GenericFunction gfbh = new MixedFunction(gftbh ,coeffFuncbh);
	 double[] coeffDistbh = { 0.309, 0.691 };
	 GenericContinuousDistribution[] gdbh = { new GammaDistribution(18.212,1998), new LogNormalDistribution(10.347,0.441) };
	 GenericContinuousDistribution dbh = new MixedDistribution(gdbh, coeffDistbh);
	 MGInfModel mginfbh = new MGInfModel(gftbh, coeffFuncbh, 1.0,dbh ,VideoExample.dSlot , VideoExample.dPacketSize);	
      
	 MGInfTest mgigbh= new MGInfTest( mginfbh, 60,"BeanHigh",MPEG4,new String("Bean"),(short)0,false);
        
			   System.out.println("Debut: " + new Date().toString());
			   int[] slots1 = mgigbh.getDataDistributionInSlot();
				mgigbh.createResultFile(slots,"BeanHighXi");
			   System.out.println("Fin: " + new Date().toString());
				System.out.println("Debut: " + new Date().toString());
				mgigbh.createResultFile(mgigbh.getDataSpecificDistributionInSlot(slots1),"BeanHighYi");
				System.out.println("Fin: " + new Date().toString());
				*/	
    }
    
    
    
}
