package traffic.distributed;

import java.rmi.*;
import graphics2.*;
import graphics2.panel.monitor.*;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JMenuBar;
import javax.swing.JFrame;

import java.awt.event.*;
import java.awt.*;

public class GraphicsMasterNode extends JFrame {

	protected MonitorPanel desktop;
	protected GestionTest gestionTest;
	protected int nbInternalFrames;
	MasterNode mnode;

	public GraphicsMasterNode(GestionTest gestionTest, String name) {
		super(name);
		this.gestionTest = gestionTest;

			// Make the big window be indented 200 pixels from each edge of the screen.
		int inset = 200;
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		setBounds(inset, inset, screenSize.width - inset*2, screenSize.height-inset*2);
		
			//Quit this app without the big window closes. !!!
		this.addWindowListener(new WindowAdapter( ) {
			public void windowClosing(WindowEvent we) {
				dispose();
			}
		});
      
		      
		//Set up the GUI.
		desktop = new MonitorPanel(gestionTest); //a specialized layered pane
		setContentPane(desktop);
		setJMenuBar(createMenuBar());

		//Make dragging faster:
		desktop.putClientProperty("JDesktopPane.dragMode", "outline");
		
		this.setVisible(true); 
	}

	protected JMenuBar createMenuBar() {
		JMenuBar menuBar = new JMenuBar();

		JMenu menuCreate = new JMenu("Options");
		menuCreate.setMnemonic(KeyEvent.VK_C);

		JMenuItem menuItemMosaic = new JMenuItem("Mosaic");
		menuItemMosaic.setAccelerator(javax.swing.KeyStroke.getKeyStroke('M', 2));
		menuItemMosaic.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				makeMosaic();
			}
		});
		menuCreate.add(menuItemMosaic);
		
		JMenuItem menuItemQuit = new JMenuItem("Quit");
		menuItemQuit.setAccelerator(javax.swing.KeyStroke.getKeyStroke('Q', 2));
		menuItemQuit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				GraphicsMasterNode.this.dispatchEvent(
						 new WindowEvent(GraphicsMasterNode.this, 
							 WindowEvent.WINDOW_CLOSING));
			}
		});
		menuCreate.add(menuItemQuit);
       
		menuBar.add(menuCreate);

		return menuBar;
	}
    
 
	
	protected void makeMosaic() {
		desktop.makeMosaic();
	}
	

	public void startMasterNode(int rmiPort, String serviceName, String xmlfilename) {
		mnode = new MasterNode(rmiPort, serviceName, xmlfilename);
	}   

	void deployApplication() throws RemoteException {
		GraphicsOutputManager gom= new GraphicsOutputManager(this.desktop);
		mnode.deployApplication(gom);
	}
	
		/*
		 *  Main program
		 */
	public static void main(String[] args) {
		String serviceName;
		String xmlfile;
		int rmiPort;

		// Usage: java traffic.distributed.MasterNode <rmi-port> <xmlhostfile> [serviceName] 
	   	if(args.length < 1 ) {
			System.err.println("Usage: java traffic.distributed.GraphicsMasterNode <rmi-port> <xmlhostfile> [service-name]");
			return;
	   	}	

		System.out.println("Main de la class TestIHM"); 
		GraphicsMasterNode ihm = new GraphicsMasterNode(null, "Allegro Monitor");

		rmiPort = Integer.parseInt(args[0]);
		if (args.length > 2)
			serviceName = args[2];
		else
			serviceName=HostInformations.name;
	
		if (args.length > 1)
			xmlfile = args[1];
		else
			xmlfile="";

		try {
			ihm.startMasterNode(rmiPort,serviceName, xmlfile);
			ihm.deployApplication();
		} catch(RemoteException re)	{
			System.err.println(re);
		}
	} // main
}