package traffic.distributed;

/**
 * HostInformationsFile: definit un fichier qui contient quasiment les memes infos que
 * 							le fichier de configuration mais qui portera toujours le meme nom
 * 							et sera accessible tout le temps!!!
 * 
 * @author mladoue
 * @version 0.1, 12/02/03
 */

import java.io.*;
import java.util.SortedSet;
import java.util.TreeSet;



import utils.*;

public class HostInformationsFile {

        // Servers Parameters (Several servers now)
    String[] ipAddress;
    String[] serverNames;
    int[] serverPort;
        // Store in alreadyServedServer[i] if serverNames[i] info has already been sent
        // to another process.
    boolean[] alreadyServedServer;

    
        // reference to the position in serverNames of the server used by client a pos
        // For example, client clientsName[i] has to contact serverNames[serverToContact[i]] server.
    protected int[] serverToContact;

    int machinesNb=0, scenarioNb;
    protected String[] clientsName, scFileName, codecParameters = new String[2], testType;
    protected int[] clientsPort, clientsCos;
    protected boolean[] clientsIpv4;
    protected boolean[] alreadyServedClient;

    protected String hostinfofile;
    
    public HostInformationsFile(String hostinfofile) throws IOException {
        this.hostinfofile=hostinfofile;
        if(FileTest.isExistent(hostinfofile)) {
            readFile();	 // permet d'initialiser les valeurs de clientsName,  clientsPort, clientsCos...
        }
        else {
            MessageManager.informationMessage("You have to create a "+hostinfofile+" file!",true);	
        }
    }
	
	
      public void readFile() throws IOException {
      	
          BufferedReader in = new BufferedReader( new FileReader("host.info") );
          
          try {
              serverNames = new String[1];
              ipAddress = new String[1];
              serverPort = new int[1];
              alreadyServedServer = new boolean[1];
              serverNames[0] = in.readLine();
              ipAddress[0] = in.readLine();
              serverPort[0] = Integer.valueOf(in.readLine()).intValue();
              alreadyServedServer[0]=false;
     	
     	
         // Lecture des caracteristiques des experimentations	
   		machinesNb = Integer.parseInt( (in.readLine()).split(" ",2)[0] );
       	clientsName = new String[machinesNb];
       	scFileName = new String[machinesNb];
       	clientsPort = new int[machinesNb];
       	clientsCos = new int[machinesNb];
       	clientsIpv4 = new boolean[machinesNb];
       	alreadyServedClient = new boolean[machinesNb];
       	// Pb s'il ya plusieurs profiles pour la meme machine...+> A ameliorer
       	for (int i=0;i<machinesNb;i++) {
            String[] tmp1 = (in.readLine()).split(" ",5);
            alreadyServedClient[i]=false;
       		clientsName[i] = tmp1[0];
       		String[] tmp2 = (in.readLine()).split(" ",7);
       		clientsPort[i] = Integer.parseInt(tmp2[1]);
       		if ( tmp2[2] == "ipv4" ) { clientsIpv4[i] = true; }
       		else { clientsIpv4[i] = false; }
       		scFileName[i] = tmp2[4];
       		clientsCos[i] =  Integer.parseInt(tmp2[6]);
       	} 	
		}
		catch (IOException ioe) {
			MessageManager.informationMessage("host.info is not a valid file!",true);	
		}    	
        in.close();
    }
    
   

  // Serialisation
    protected void writeFile(FileWriter file) throws IOException {

	// Ecriture des caracteristiques server
	file.write( serverNames[0] + '\n');
    file.write( ipAddress[0] + '\n');
    file.write( serverPort[0] + '\n');
	
	
    
     file.flush();
    }
   
    
    // Definition de quelques methodes visant a recuperer les parametres...
        // The next three functions ensure compatibility with previous files
        // DS: MUST BE removed soon !
    public String getServerName() { return serverNames[0]; }
    public int getUDPServerPort() { return serverPort[0]; }
    public String getServerIPAddress() { return ipAddress[0]; }
  
    public String[] getServerList() {
        return serverNames;
    }

    public int[] getServerPortList() {
        return serverPort;
    }

    public String getServerNameForClient(String clientName, int port) throws IOException { 
        int k= -1;
        for ( int i=0; i<clientsName.length;i++ ) {
            if ( clientsName[i].equals(clientName) && clientsPort[i]==port) { k=i; }
        }
        
        if (k == -1) { throw new IllegalArgumentException("HostInformationsFile.getServerPortForClient : Unable to find the server of client: "+ clientName );}
        
        return serverNames[serverToContact[k]];
    }
    
    public int getServerPortForClient(String clientName, int port) throws IOException { 
        int k= -1;
        for ( int i=0; i<clientsName.length;i++ ) {
            if ( clientsName[i].equals(clientName) && clientsPort[i]==port) { k=i; }
        }
        
        if (k == -1) { throw new IllegalArgumentException("HostInformationsFile.getServerPortForClient : Unable to find the server of client: "+ clientName );}
        
        return serverPort[serverToContact[k]];
    }

    public int getUDPServerPort(String serverName) throws IOException { 
        int k = -1;
        for ( int i=0; i<serverNames.length;i++ ) {
            System.out.println (serverNames[i]+" "+alreadyServedServer[i]+" "+i);
            if ( ( k==-1 &&serverNames[i].equals(serverName) && alreadyServedServer[i]!=true)) { 
                alreadyServedServer[i]=true;
                k=i; 
            }
        }

        if (k == -1) { throw new IllegalArgumentException("The server: "+ serverName + " doesn't participate to the test." );}
        
        return serverPort[k];
    }

    public String getServerIPAddress(String serverName, int port) {
        int k = -1;
        for ( int i=0; i<serverNames.length;i++ ) {
            if ( ( serverNames[i].equals(serverName) && serverPort[i]==port)) { k=i; }
        }

        if (k == -1) { throw new IllegalArgumentException("The server: "+ serverName + " doesn't participate to the test." );}
        
        return ipAddress[k];
    }

    public int getUDPClientPort(String clientN) throws IOException, NumberFormatException { 
        int k = -1;
        for ( int i=0; i<clientsName.length;i++ ) {
            if ( k==-1 && clientsName[i].equals(clientN) && alreadyServedClient[i]!=true ) { 
                alreadyServedClient[i]=true;
                k=i;
            }
        }
        if (k == -1) { throw new IllegalArgumentException("The client: "+ clientN + " doesn't participate to the test." );}
        
        return clientsPort[k];
    }
  	
    public int getClientCOS(String clientName, int port) throws IOException, NumberFormatException {
        int k = -1;
        for ( int i=0; i<clientsName.length;i++ ) {
            if ( clientsName[i].equals(clientName) && clientsPort[i]==port) { k=i; }
        }
        if (k == -1) { throw new IllegalArgumentException("The client: "+ clientName + " doesn't participate to the test." );}
        
        return clientsCos[k];
    }
  	
    public boolean isClientIPv4(String clientName, int port) throws IOException {
        int k = -1;
        for ( int i=0; i<clientsName.length;i++ ) {
            if ( clientsName[i].equals(clientName) && clientsPort[i]==port) { k=i; }
        }
        if (k == -1) { throw new IllegalArgumentException("The client: "+ clientName + " doesn't participate to the test." );}
        
        return clientsIpv4[k];
    }
  	
        // Not so simple since we only returns scenario for the clients of this serverName.
    public String[] getScenarioFilesList(String serverName, int port) {
            // Find the position of this server...
        int k = -1;
        for ( int i=0; i<serverNames.length;i++ ) {
            if ( serverNames[i].equals(serverName) && serverPort[i]==port) { k=i; }
        }
            // Count the number of clients having serverName for server
        int count = 0;
        for(int j=0; j < serverToContact.length; j++) {
            if (serverToContact[j] == k) {
                count++;
            }
        }

        String[] scFile = new String[count];
            // Populate the file...
        int tmp=0;
        for(int j=0; j < serverToContact.length; j++) {
            if (serverToContact[j] == k) {
                scFile[tmp]=scFileName[j];
                tmp++;
            }
        }
        return scFile;
    }
    
    public int getScenarioFileNumber(String serverName, int port) { return getScenarioFiles(serverName, port).size();}

	/**
	 * Method getScenarioFiles. Permer de donner la liste des differents scenaro a jouer
	 * 									Evite de transferer 2 fois le meme
	 * @return SortedSet
	 */
    public SortedSet getScenarioFiles(String serverName, int port) {
        SortedSet name = new TreeSet();
        String[] scFile = getScenarioFilesList(serverName, port);
        for (int i=0;i<scFile.length;i++) {
            name.add(new String(scFile[i]));
        }
        return name;
    }

    public String[] getClientsList(String serverName, int port) { 
            // Find the position of this server...
        int k = -1;
        for ( int i=0; i<serverNames.length;i++ ) {
            if ( serverNames[i].equals(serverName) && serverPort[i]==port) { k=i; }
        }
            // Count the number of clients having serverName for server
        int count = 0;
        for(int j=0; j < serverToContact.length; j++) {
            if (serverToContact[j] == k) {
                count++;
            }
        }

        String[] clientsN = new String[count];
        int tmp=0;
            // Populate the file...
        for(int j=0; j < serverToContact.length; j++) {
            if (serverToContact[j] == k) {
                clientsN[tmp]=clientsName[j];
                tmp++;
            }
        }
        return clientsN;
    }

    public int[] getClientsPortList(String serverName, int port) { 
            // Find the position of this server...
        int k = -1;
        for ( int i=0; i<serverNames.length;i++ ) {
            if ( serverNames[i].equals(serverName) && serverPort[i]==port) { k=i; }
        }
            // Count the number of clients having serverName for server
        int count = 0;
        for(int j=0; j < serverToContact.length; j++) {
            if (serverToContact[j] == k) {
                count++;
            }
        }

        int [] clientsP = new int[count];
        int tmp=0;
            // Populate the file...
        for(int j=0; j < serverToContact.length; j++) {
            if (serverToContact[j] == k) {
                clientsP[tmp]=clientsPort[j];
                tmp++;
            }
        }
        return clientsP;
    }

    public String[] getClientsList() {  return clientsName; }

    public int[] getClientsPortList() { return clientsPort;  }
    
    public String getScenarioFileName(String clientN, int port) {
        int k = -1;
        for ( int i=0; i<clientsName.length;i++ ) {
            if ( clientsName[i].equals(clientN) && clientsPort[i]==port) { k=i; }
        }
        if (k == -1) { throw new IllegalArgumentException("The client: "+ clientN + " doesn't participate to the test." );}
        
        return scFileName[k];
    }
    
   // public MachinesList getMachinesList() { return machinesList; }
    

public static void main(String[] args) {
	
	try {
		HostInformationsFile hf = new HostInformationsFile("host.info");
		
	System.out.println("ServerName "+ hf.getServerName() );	
	System.out.println("ServerPort " +  hf.getUDPServerPort() );
//	System.out.println("Client nb:" + hf.getClientNumber());
//	System.out.println("Scenario nb:" + hf.getScenarioFileNumber());
	System.out.println("UDPClientPort 1: " + hf.getUDPClientPort("toto") );
	System.out.println("UDPClientPort 2: " + hf.getUDPClientPort("wsopi12b") );
	System.out.println("UDPClientPort 3: " + hf.getUDPClientPort("mama") );
	System.out.println("UDPClientPort 4: " + hf.getUDPClientPort("chouaia") );
	}
	catch (IOException ioe) {
	ioe.printStackTrace();	
	}		
	}

}
