package traffic.distributed;

import java.io.*;
import java.net.*;

import utils.*;

/**
 * RemoteMultipleClientUDPServer : Doir etre lance sur la machine qui va servir de serveur multimedia
 * 
 * @author mladoue
 * @version  0.1, 12/02/03 
 */


public class RemoteMultipleClientUDPServer extends Thread {

	 // Port sur lequel le serveur attend les connexions
    protected int srcPort;
    // socketUDP est le socket d'ecoute du serveur
    protected DatagramSocket socketUDP;
    // Booleen indiquant si il faut ou non attendre des connexions -> arreter/fermer le DatagramSocket
    protected boolean waitConnection;

	protected UDPServerManager rmcus;

    //Nombre de connexions = nb de sockets ouvertes
    protected int nbConnection, nbClients;
    protected String[] clientsName, scenarioFilesName;
    protected int[] clientsPort;

    public RemoteMultipleClientUDPServer(int srcPort, UDPServerManager rmcus) {
		
		this.srcPort = srcPort;
		this.rmcus = rmcus;

		nbConnection = 0;
		nbClients = rmcus.getScenarioFilesList().length;
        scenarioFilesName = rmcus.getScenarioFilesList();
        clientsName = rmcus.getClientsList();
		clientsPort = rmcus.getClientsPortList();
		try {
			if ( LocalUDPPortsScanner.isSpecificPortFree(srcPort) ) {
	    		socketUDP = new DatagramSocket(srcPort);
			}
	    	else {
      			System.out.println("The port: " + srcPort + " is already in use. We try to find another one....");
      			boolean free= false;
      			while(!free) {
      				srcPort++;
      				if(	LocalUDPPortsScanner.isSpecificPortFree(srcPort)) {
      					free = true;	
      				}
      			}
      			socketUDP = new DatagramSocket(srcPort);
      			System.out.println("We choose port number: "  + srcPort + ".");
      		}
      		
	    InetAddress server = InetAddress.getLocalHost();
	    System.out.println("Server created on: " +  server.getHostName() +" "+server.getHostAddress()+  ": listening port: " + srcPort + ".");
 
	}
	catch (UnknownHostException uhe) { uhe.printStackTrace(); }
	catch( NullPointerException npe) { npe.printStackTrace(); }	
	catch(IOException e) { System.out.println("RemoteMultipleClientUDPServer: UDPServer =>" + e.toString()); }
	catch(ArrayIndexOutOfBoundsException aioobe) {
			System.out.println("RemoteMultipleClientUDPServer: UDPServer =>" + aioobe.toString());
		    aioobe.printStackTrace();
	}

	waitConnection = true;
	this.start();    
		
	}


   /**
     * Methode run() de la classe Thread.<br>
     * Attend la connexion des clients. A chaque arrive d'un nouveau client,
     * elle lui dedie un DatagramSocket (i.e un port de la machine) auquel il pourra se "connecter".
     */

    public void run() {

	// Definition des variables
	boolean start = false;

	int destPort;  // Port de destination = port du client
	int newPort = srcPort;  // Port source(= port du client) utiliser pour la connection UDP avec le client de destPort
	DatagramPacket packet = null;
	byte[] tampon = new byte[256];  // taille par defaut a l'etablissement de la connexion
        DatagramSocket newDS; // A chaque nouveau client un DatagramSocket est cree
	InetAddress clientAddress;
	int messageSize = rmcus.getMessageSize();
   
        try  {
		
	    System.out.println("Client connection wait...\n");
	    
	    while ( waitConnection || !start ) {  
		start = true;
		// Attendre un message emis par le client
		packet  = new DatagramPacket(tampon, tampon.length);
		socketUDP.receive(packet);
		
		if (waitConnection) {
			
		    // On recupere l'addresse IP et le port
		    clientAddress = packet.getAddress();
		    destPort = packet.getPort();
		
		    System.out.println("New connection with client " + "<" + clientAddress.getHostAddress() +" , "+clientAddress.getHostName()+" , " + destPort  + ">");
		    newPort++;

			if ( LocalUDPPortsScanner.isSpecificPortFree(newPort) ) {
		    	newDS = new DatagramSocket(newPort);
		    }
	    	else {
      			System.out.println("The port: " + newPort + " is already in use. We try to find another one....");
      			boolean free= false;
      			while(!free) {
      				srcPort++;
      				if(	LocalUDPPortsScanner.isSpecificPortFree(newPort)) {
      					free = true;	
      				}
      			}
      			newDS = new DatagramSocket(newPort);
      			System.out.println("We choose port number: "  + newPort + ".");
      		}
		    
		
		    // Envoyer la reponse vers le client
		    packet = new DatagramPacket( tampon, tampon.length, clientAddress, destPort);
		    newDS.send(packet);
			
		    nbConnection++;  // on peut considere qu'une connexion au serveur a eu lieu!!!
		    
                        // On doit threader la suite...
                    SingleUDPServer userver = new traffic.distributed.SingleUDPServer(newDS, clientAddress, destPort, messageSize, this);
		    
		    if(nbConnection == nbClients) { 
		    	System.out.println( "All expected clients are connected now..." );
				serverClosing();
		     }
			}
	    }
		}
		catch (InterruptedException ie) { ie.printStackTrace(); }
		catch (UnknownHostException uhe) {
	    System.out.println("RemoteMultipleClientUDPServer:run : Don't know about host:" + packet.getAddress());
	    System.exit(1);
	} 
	catch (IOException ioe) { System.out.println("RemoteMultipleClientUDPServer:run : " + ioe.toString() + " port : " +  packet.getPort()); }
    }  // end of run
	
		
		/**
		 * 
		 * Get the scenario file for a specific client(Host, port number).
		 * @param clientN is the name of the host
		 * @param port is the port number on the host.
		 * @return the scenario file name
		 * @throws IOException
		 * @throws NumberFormatException
		 */
    public synchronized String getScenarioFileName(String clientN, int port) throws IOException, NumberFormatException { 
        
        int k = -1;
        for ( int i=0; i<clientsName.length;i++ ) {
        	// Must also check that clientN is not the IP string 
        	// This can occur when the IP adresse cannot be resolved
        	String ipaddr = InetAddress.getByName(clientN).getHostAddress();
            if ( (clientsName[i].equals(clientN) || ipaddr.equals(clientN)) && clientsPort[i]==port) { k=i; }
        }
        if (k == -1) { 
        	// Does not mean that the client is not in the list...	
        	throw new IllegalArgumentException("The client: "+ clientN + " doesn't participate to the test." );
        }
        return scenarioFilesName[k];
    }


    /**
     * Methode pour arreter le serveur UDP: serverClosing()
     */    
    public void serverClosing() throws InterruptedException { 
    	
    waitConnection = false; // il faut se connecter au serveur pour qu'il n'attende plus de connexion

	try {
	    String message="Server is going to be shut down in 60 seconds!";
	    byte[] sendBuf = message.getBytes();
	    DatagramPacket packet = new DatagramPacket(sendBuf,sendBuf.length,InetAddress.getByName("localhost"),srcPort);
	    socketUDP.send(packet);
	    sleep(60);
	    socketUDP.close();
	}	
	catch (UnknownHostException e) { System.out.println("The IP address of the local host could not be determined."); }
	catch (IOException e) { System.out.println("RemoteMultipleClientUDPServer:serverClosing => fermetureServeur() " + e.toString()); }

    }
  
    // Renvoie le nombre de connections actives
    public int getConnectionNb() {	return nbConnection;   }
    
 
	
}

