package traffic.distributed;

import java.io.*;
import java.net.*;
import java.util.*;


/**
 * SingleUDPServer: Lance par RemoteMultipleClientUDPServer a chaque connexion d'un client
 * 
 * @author dsagnol
 * @version  0.1, 11/06/03 
 */


public class SingleUDPServer extends Thread {
        
    String fileName;
    String clientName;
    DatagramSocket newDS;
    int destPort;
    int messageSize;
    InetAddress clientAddress;

    public SingleUDPServer( DatagramSocket newDS, InetAddress clientAddress, int destPort, int messageSize, RemoteMultipleClientUDPServer rmcus) {
            //Lancer  la generation de trafic...
        String clientName = clientAddress.getHostName();
        String[] tmp = (clientName.replace('.',' ')).split(" ",3); // Suppose que le nom des machines n'est pas sous la forme 
        clientName = tmp[0];                                       // toto.inria.fr dans le fichier hostinfo
        this.newDS=newDS;
        this.clientName=clientName;
        this.messageSize=messageSize;
        this.destPort=destPort;
        this.clientAddress=clientAddress;
        try {
            fileName = rmcus.getScenarioFileName(clientName, destPort);
        }
        catch(IOException ioe) {
            System.out.println(ioe);
        }
        this.start();
    }
    
    public void run () {
        try {
            FileReader fr = new FileReader(fileName);
            BufferedReader br = new BufferedReader(fr);
            
            System.out.println("Start emission threaded: " + new Date().toString() + " on " + clientName + " of: " + fileName);
            
            StringTokenizer st;
            String c = "";
            
            // Accumulate inter packet time for with used for the sleep call
            long tempoMs=0;
            int tempoNs=0;
            while ( (c = br.readLine()) != null ) {
                
                st = new StringTokenizer(c);
                    //definition of scenario file type
                int nbTokens = st.countTokens();
                    
                String token1 = st.nextToken(), token2 = st.nextToken();
                
                    // Emission du paquet
                if(nbTokens == 2) {
                    byte[] tampon2 = new byte[messageSize];
                    DatagramPacket message2 = new DatagramPacket (tampon2 , tampon2.length, clientAddress, destPort);
                    newDS.send(message2);
                }
                
                if(nbTokens == 3) {
                    try {
                        String token3 = st.nextToken();
                        byte[] tampon3 = new byte[Integer.valueOf(token3).intValue()];
                        DatagramPacket message3 = new DatagramPacket (tampon3, tampon3.length , clientAddress, destPort);   // enleve qualificatif final 21/02/03
                        newDS.send(message3);
                    }
                    catch(NoSuchElementException e) {  System.out.println(e); }
                }	
                // Update tempoMs and tempoNs...
                tempoMs +=Long.valueOf(token1).longValue();
				tempoNs += Integer.valueOf(token2).intValue();
				if (tempoNs > 999999) {
					tempoMs++;
					tempoNs -= 999999;
				}
					// Only make a temporisation when we accumulate more than 20ms.                
                if (tempoMs > 20) {
                		// Take current time.
                	long beforeSleepTime= System.currentTimeMillis();
					try {
						Thread.sleep(tempoMs, tempoNs);
					}
					 catch(InterruptedException e) {  System.out.println(e); }
					long afterSleepTime= System.currentTimeMillis();
					
					// correct tempoMs value with the difference between expected 
					// temporisation (tempoMs) and the real one (afterSleepTime-beforeSleepTime).
					tempoMs = tempoMs - (afterSleepTime-beforeSleepTime);
					tempoNs = 0; // Could be more precise... 
                }
            } // end while
            
            br.close();
            
            System.out.println("End of emission: " + new Date().toString() + " on " + clientName + " of: " + fileName );
            
            System.out.println("Connexion with: " + clientName + " on port: " + newDS.getLocalPort() +  " is going to be closed!");
            newDS.close();     	
        } // fin du try
        
        catch (UnknownHostException uhe) {
            System.out.println("RemoteMultipleClientUDPServer:run : Don't know about host:" + clientAddress.getAddress());
            System.exit(1);
            } 
        catch (IOException ioe) { System.out.println("RemoteMultipleClientUDPServer:run : " + ioe.toString() + " port : " +  destPort); }
        
    } // fin run
}

