package traffic.distributed;

/**
 * UDPServerManager 
 * @author mladoue
 * @version 0.1, 20/02/03
 */

import utils.RegistryListor;


public class UDPServerManager {

 //Le serveur
 protected RemoteMultipleClientUDPServer server;
 protected String serverName;
 protected int rmiPort, srcPort, messageSize;
 protected String[] filesName, clientsName;
 protected int[] clientsPort;
 protected boolean serverAvailable = false;
 protected boolean isIpv4;

/**
 * Method UDPServerManager.
 * @param serverName name of the "rmi" server: HostInformationsServer
 * @param rmiPort port of this rmi server
 * @param messageSize size of message in the case of an audio generation
 * @throws InterruptedException
 */
public UDPServerManager(String serverName, int rmiPort, int messageSize) throws InterruptedException {
		
		while(!serverAvailable) {
			if( (new RegistryListor(serverName, rmiPort)).isServiceAvailable("//"+serverName+":"+rmiPort+"/"+HostInformations.name) ||
                            (new RegistryListor(serverName, rmiPort)).isServiceAvailable("rmi://"+serverName+":"+rmiPort+"/"+HostInformations.name) ) { serverAvailable = true; }
			//else { wait(60); } // sinon on attend 60 secondes avant de redemander
		}
		
		HostInformationsClient hic = new HostInformationsClient( serverName, rmiPort, HostInformations.name,1);
		srcPort = hic.getUDPServerPort();
		filesName = hic.getScenarioFilesList();
		clientsName = hic.getClientsList();
        clientsPort = hic.getClientsPortList();
		
            // Get Ipv4/Ipv6 info based on one client
        isIpv4 = hic.isClientIPv4();
        server = new RemoteMultipleClientUDPServer(srcPort,this);
}


protected String[] getScenarioFilesList() { return filesName; }
protected String[] getClientsList() { return clientsName; }
protected int[] getClientsPortList() { return clientsPort; }
protected int getMessageSize() { return messageSize; }
	
public static void main(String[] args) {
	
	String serverName;
	int rmiPort, messageSize;
	
	
	// Usage: traffic.generation.RemoteUDPServer  <rmi-server-name> <rmi-port> <file-name> [packet-size]
	if(args.length < 2 ) {
            System.err.println("Usage: java traffic.generation.UDPServerManager <rmi-server-name> <rmi-port> [packet-size]");
	return;
	}
	serverName = args[0];  // nom du server RMI HostInformations
	rmiPort = Integer.parseInt( args[1] );
	
	if (args.length == 3 ) { messageSize =  Integer.parseInt(args[2]); }
	else { messageSize = 1000; }
	
	try {
		UDPServerManager usm = new UDPServerManager(serverName,rmiPort, messageSize);
	} 
	catch (InterruptedException ie) { ie.printStackTrace();}
}
}
