package traffic.generation;

/**
 * ScenarioPlayer: Permet de jouer un scenario dans les cas avec et sans controle en mode texte
 * 
 * @author mladoue
 * @version 0.1, 19/12/02 
 */

import java.util.*;
import java.net.*;
import java.io.*;
import java.text.*;

import traffic.SimpleUDPServer;
import traffic.SocketEcriture;
import traffic.SocketLecture;
import traffic.UDPServer;
import traffic.UDPSocketReader;
import utils.*;



public class ScenarioPlayer extends Thread implements UDPSocketReader {

   	protected UDPServer server;  // dans le cas ou on utilise le systeme de controle de traffic avec les files d'attentes...
   	protected SimpleUDPServer simpleServer;
 
   	 // Fichier scenario
   	protected File file;
    protected FileReader fr;
   	
   	protected int messageSize;  // fixe dans certains cas comme l'audio
    protected int nbMessages;
    protected int cos;

  	//Numro du datagramme envoye
    // le premier datagramme de donnees correspondra au Numero 2 car le 1 est reserve au mangement...
    protected int noDatagrammeEnvoye=2;
	protected boolean control;  // pour specifier si on veut marquer les paquets pour faire du controle a posteriori
	
  		/**
         * Method ScenarioPlayer : Initialise le Thread dans le cas d'un lancement mode graphique
         * @param server type serveur graphique
         * @param scenarioFileName
         * @param messageSize
         * @param control
         * @throws FileNotFoundException
         */
        public ScenarioPlayer(UDPServer server,String scenarioFileName,int messageSize, boolean control) throws FileNotFoundException{ 
                this.server = server; 
                setScenarioFile(scenarioFileName);
                this.fr = new FileReader(file.getName());
                this.messageSize = messageSize;
                this.control = control;
                
                this.start();
        }
        
		/**
		 * Method ScenarioPlayer : Initialise le Thread dans le cas d'un lancement mode text
		 * @param server type serveur en mode texte
		 * @param scenarioFileName
		 * @param messageSize
		 * @throws FileNotFoundException
		 */
          public ScenarioPlayer(SimpleUDPServer simpleServer, String scenarioFileName, int messageSize) throws FileNotFoundException{ 
                this.simpleServer = simpleServer; 
                setScenarioFile(scenarioFileName);
                this.fr = new FileReader(file.getName());
                this.messageSize = messageSize;
                this.control = false; // le controle n'est possible que dans le cas d'un UDPServer pour l'instant!!!!
                
                this.start();
        }
        
        
        /**
         * Implantation.
         */
        public synchronized void wakeUp() { notify();  }

        /**
         * Appel  la mthode wait() de la classe Object.
         */
        protected synchronized void attendre(){
	    	try{ wait();	}
	    	catch (InterruptedException e) { System.out.println(e.toString()); }
        }

        /**
         * Method run
         */
        public void run() {
        
        if ( clientConnectionExistence() ) { 
	    
	    	System.out.println("Bench: Scenario Player: " + file.getName() + ".\n"); 
	    	
			nbMessages = FileLineCounter.countLines(file.getName());
			BufferedReader br = new BufferedReader(fr);
			String c ="";  // ligne courante du document a regarder
		
			if(control) { controlScenarioPlay(c,br);	}
			else { scenarioPlay(c,br); }

			System.out.println("End emission: " + new Date().toString() + ".\n");
	  
		}
        }  // end run()
 
 
 		
		/**
		 * Method controlScenarioPlay : dans le cadre d'une generation de trafic avec un control
		 *  Bench scenario player : Les paquets envoyes vont etre formes comme suit:
         * 			<i>numeroPaquet</i> + " " + "random data" 
         * 			ou la taille de l'en-tete "numeroPaquet" est fixe et est fonction de la taille du fichier
		 * 
		 * @param c to read the scenario file
		 * @param br
		 */
 		public void controlScenarioPlay(String c,BufferedReader br) {
 			
 			SocketEcriture se = server.getThreadEcriture(0);
			SocketLecture sl = server.getThreadLecture(0);
			
			sl.setUDPSocketReader(this);
			
			// packetNumberSize corresponds to the size of PacketNumber + one space: " "
			short packetNumberSize = (short)(MyNumberFormat.getPacketNumberHeaderSize(nbMessages) + 1);
			int dataSize = messageSize - packetNumberSize;
			String decF = "0";
			for (int k=1;k<packetNumberSize;k++) {
				decF = decF.concat("0");
			}
			
			DecimalFormat myPacketNumberSizeFormatter = new DecimalFormat(decF);
		
			if (control) {
			se.messageAEnvoyer("1 scenarioplayer");   // premier message envoye
			se.messageAEnvoyer("1 "+ nbMessages); //+ "Number of messages to be send");
			}
		
 			// Initialized only for control version
			ChaineAleatoire ca; // chaine de caracteres aleatoires qui constituera les messages envoyes	
 			StringTokenizer st;
		
 			System.out.println("Start emission: " + new Date().toString() + ".\n");
 				
			//envoi des messages au client
			try {
		  	while ( (c = br.readLine()) != null ) {
	  	
		  	st = new StringTokenizer(c);
			
			//definition of scenario file type
			int nbTokens = st.countTokens();
			
			String token1 = st.nextToken();
			String token2 = st.nextToken();
			
			// Emission du message
			if(nbTokens == 2) {
				
				if(control) {
				ca = new ChaineAleatoire(dataSize);
				se.messageAEnvoyer(myPacketNumberSizeFormatter.format(noDatagrammeEnvoye) + " " + ca.gnrerChane());
				noDatagrammeEnvoye++;
				}
				else {
				ca = new ChaineAleatoire(messageSize);
				se.messageAEnvoyer(ca.gnrerChane());
				}
			}
			
			if(nbTokens == 3) {
		 		try {
		 		String token3 = st.nextToken();
		 		 
		 		 if (control) {
		 		//Determine le debit applicatif => ne tient pas compte de l'overhead IP...
		 		ca = new ChaineAleatoire(Integer.valueOf(token3).intValue()-packetNumberSize);
				se.messageAEnvoyer(myPacketNumberSizeFormatter.format(noDatagrammeEnvoye) + " " + ca.gnrerChane());
				noDatagrammeEnvoye++;
		 		 }
		 		 else {
		 		 ca = new ChaineAleatoire(Integer.valueOf(token3).intValue());
				se.messageAEnvoyer(ca.gnrerChane());
		 		 }		
  				}
  				
		 	 	catch(NoSuchElementException e) {
		  		System.out.println(e);	
		  		}
			}				
			// Attente entre 2 paquets
			try {
			    sleep(Long.valueOf(token1).longValue(),Integer.valueOf(token2).intValue());
			}
			catch(InterruptedException e) {
			    System.out.println(e);
			}
					  	 
		} // end while

			}  // end try
			catch(IOException e) {
				System.out.println("UDPServer:run : " + e.toString());
			}	
		}
   			
 
/**
 * Method scenarioPlay : dans le cadre d'une generation de trafic sans control avec un SimpleUDPServer
 * @param c
 * @param br
 * @throws IllegalArgumentException
 */
 public void scenarioPlay(String c,BufferedReader br) throws IllegalArgumentException {
 	
 		StringTokenizer st;
 		
 		if( !simpleServer.getConnectionState() ) {
 		InetAddress clientAddress = simpleServer.getClientAddress();
 		int destPort = simpleServer.getDestinationPort();
 		DatagramSocket newDS = simpleServer.getDatagramSocket();
		
		
 			System.out.println("Start emission: " + new Date().toString() + ".\n");		
			//envoi des messages au client
			try {
		  	while ( (c = br.readLine()) != null ) {
	  	
		  	st = new StringTokenizer(c);
			
			//definition of scenario file type
			int nbTokens = st.countTokens();
			
			String token1 = st.nextToken();
			String token2 = st.nextToken();
			
			// Emission du message
			if(nbTokens == 2) {
					byte[] tampon2 = new byte[messageSize];
					DatagramPacket message2 = new DatagramPacket (tampon2 , tampon2.length, clientAddress, destPort);
					newDS.send(message2);	
			}
			
			// Dans le cas ou on specifie la taille des paquets (video scenario)
			if(nbTokens == 3) {
		 		try {
		 		String token3 = st.nextToken();
		 		//Determine le debit applicatif => ne tient pas compte de l'overhead IP...
		 		byte[] tampon3 = new byte[Integer.valueOf(token3).intValue()];
		  		final DatagramPacket message3 = new DatagramPacket (tampon3, tampon3.length , clientAddress, destPort);
		  		
		 		newDS.send(message3);
		 		
  				}
		 	 	catch(NoSuchElementException e) {
		  		System.out.println(e);	
		  		}
			}
									
			// Attente entre 2 paquets
			try {
			    sleep(Long.valueOf(token1).longValue(),Integer.valueOf(token2).intValue());
			}
			catch (IllegalArgumentException iae ) { System.out.println( "seconds: " + Long.valueOf(token1).longValue() +", ms: " + Integer.valueOf(token2).intValue()); }
			catch(InterruptedException e) {
			    System.out.println(e);
			}
			
	
		} // end while
			   br.close();
			}  // end try
			catch(IOException e) {
				System.out.println("UDPServer:run : " + e.toString());
			}
 		}
 		
 		else {
 			System.out.println("No connected client.\n");
 		}
 
 }	//end scenarioPlay
 	
 	public boolean clientConnectionExistence() {
 		  if(control) {	
	  		  if (server.nbConnection() == 0 ){
	  		  	server.putApplicationMessage("Operation canceled because no client is connected!");
	    		System.out.println("No connected client.\n");
	    		return false;
	    	}
        }
        else {
        if( simpleServer.getConnectionState() ) {
        	System.out.println("No connected client.\n");
	    	return false;
        }
        }
        return true; 
 	}

   
/**
 * Method setScenarioFile.
 * @param fileName
 * @return boolean
 * @throws FileNotFoundException
 */
   public boolean setScenarioFile(String fileName) throws FileNotFoundException   {
        	if(FileTest.isExistent(fileName)) {
        		file = new File(fileName);
        		return true;
        	}
        	else {
        		System.out.println("File: " + fileName +" doesn't exist!" );
        	 	return false;
        	}
        }

public void setMessageSize(int messageSize) {	this.messageSize = messageSize; }


	public static void main(String[] args) throws SocketException, FileNotFoundException {		
		
	}
}
