package utils;

/**
 * File Test ; this class is used to test if a file exists...
 * @author mladoue
 * @version 0.1, 88/11/02 
 */

import java.io.*;
import java.util.*;

import javax.swing.JOptionPane;

public class FileTest {

// file extension for scenario files
public final static String scf = "scf";
// file extension for statistics files
public final static String stf = "stf";
// file extension for configuration files
public final static String conf = "xml";
// file extension for log files
public static String log = "log";
// fiel extension for each client scenario files
public static String rep = "rep";

	/**
	 * Method getExtension.   Get the extension of a file.
	 * @param f
	 * @return String
	 */
    public static String getExtension(File f) {
        String ext = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');

        if (i > 0 &&  i < s.length() - 1) {
            ext = s.substring(i+1).toLowerCase();
        }
        return ext;
    }


/**
 * Method isReadable. Test if a file is readable or not
 * @param fileName
 * @return boolean
 */
public static boolean isReadable(String fileName) {
		if ((new File(fileName)).canRead()) {
		return true;
	}
	else {
		return false;
		}
}
	

/**
 * Method isExistent. Test l'existence du fichier passe en parametre
 * @param fileName
 * @return boolean
 */
public static boolean isExistent(String fileName) {
	if ((new File(fileName)).exists()) {
		return true;
	}
	else {
		return false;
		}
}

/**
 * Method testExistent. Test l'existence du fichier passe en parametre 
 * 								Si il existe deja soit on le remplace soit on en cree un nouveau
 * 								Si il n'existe pas on le cree
 * @param fwName
 * @param graphic  mode text/graphic
 * @return a FileWriter
 * @throws IOException
 */
public static FileWriter testExistent(String fwName,boolean graphic)  throws IOException {
	String answer = null;
	BufferedReader brInput = new BufferedReader(new InputStreamReader(System.in)); // pour recuperer la reponse de l'utilisateur
	FileWriter fw = null;


if ((new File(fwName)).exists()) {
	
	if( graphic ) {
	
  	int n = JOptionPane.showConfirmDialog(null, "File: " + fwName + " exists.\n Would you replace it?", "Warning: File problem", 
            JOptionPane.YES_NO_OPTION );
 
	if( n == JOptionPane.YES_OPTION )	 {
 		fw = new FileWriter(fwName);      
	}

	else if (n == JOptionPane.NO_OPTION) {
		String inputValue = JOptionPane.showInputDialog("Please enter the new file name: "); 
 		fw = new FileWriter(inputValue);             	   
     } 
 	 else {
              JOptionPane.showMessageDialog(null, "Operation aborted!", "Alert", JOptionPane.ERROR_MESSAGE);     
               System.exit(0);
             }
}

else {
	    System.out.println("File: " +fwName + " exists.\n Would you replace it?(y/n)");
	 
	    try {
		answer = brInput.readLine();
	    }
	    catch (IOException e) {
		System.out.println("Erreur de lecture de la console.");
		System.exit(0);
	    }
	    if (!answer.equals("Y") && !answer.equals("y")) {
		System.out.println("Please enter the new file name: ");
		try {
		    answer = brInput.readLine();
		}
		catch (IOException e) {
		    System.out.println("Erreur de lecture de la console.");
		    System.exit(0);
		}
		fw = new FileWriter(answer);
	    }
	    else {
		fw = new FileWriter(fwName);
	    }   
	}
}
	else {
	    fw = new FileWriter(fwName);
	}
	return fw;	
}

/**
 * Method isValidScenarioFile. It permits to test that there all the necessary data
 * @param fwName name of the scenario file
 * @param columnNumber Theoritical columns number
 * @return boolean if it's valid or not
 * @throws FileNotFoundException
 * @throws IOException
 */
public static boolean isValidScenarioFile(String fwName, int columnNumber) throws FileNotFoundException, IOException  {
	isReadable(fwName);
	FileReader fr = new FileReader(fwName);
	BufferedReader br = new BufferedReader(fr);
	String current="";
	int lineNumber = FileLineCounter.countLines(fwName);
	int k=0;
	
	try {     
			while( (current = br.readLine()) != null ) {		
     	 
     	 StringTokenizer st = new StringTokenizer(current);
     	 int nbTokens = st.countTokens() ;
     	 //System.out.println("Nb de token: " +  nbTokens +" lines number: " +lineNumber  );
     	  k++;
     	 if( nbTokens != columnNumber) {
     	 System.out.println("File:" +  fwName + ", Error on line: " + k + ", tokens number is : " +  nbTokens + " in spite of: " + columnNumber );
     	 return false;
     	 }
		}
		fr.close();
	}
	catch(IOException ioe) {
		System.out.println(ioe);
	}
	
	//System.out.println("File:" +  fwName + " line number " + FileLineCounter.countLines(fwName) + " lines" );
	return true;
}


/**
 * Method isHardwareConformScenario.
 * @param fileName name of the scenario to test
 * @param minValue corresponds to the value below which we cannot generate packets with a good frequency
 * 							it's in ms with a precision of nano seconds
 * @param percent  with an acceptance percentage of 
 * @return boolean
 * @throws FileNotFoundException
 */
public static boolean isHardwareConformScenario(String fileName, double minValue, double percent) throws FileNotFoundException {
	
	FileReader fr = new FileReader(fileName);
	BufferedReader br = new BufferedReader(fr);
	String current = "";
	int compteur = 0;	
	int ms = new Double(minValue).intValue();

	int nanos = (int)((minValue - ms)*1000000);
	System.out.println(ms + " " + nanos);
	int linesNb = FileLineCounter.countLines(fileName);
	
	try {     
		while( (current = br.readLine()) != null ) {		
     	 
     	 StringTokenizer st = new StringTokenizer(current);
     	 
     
     	int token1 = Integer.parseInt(st.nextToken()); 
     	int token2 = Integer.parseInt(st.nextToken());
     	 if (token1 < ms) { 
     	 	if (token2 < nanos) { compteur++; }
     	 }	
     	 }
     	 	
		br.close();
		fr.close();
     		
	}
	catch(IOException ioe) {
		System.out.println(ioe);
	}
	 if ( (double)compteur /(double)linesNb > percent ) { 
    	 	//MessageManager.informationMessage("Warning: The scenario file: " + fileName + " , contains more than " + (percent*100) +"%  ( "+  ((double)compteur /(double)linesNb *100) + "% ) of non valid value.", false);
    	 	MessageManager.informationMessage("Warning: The scenario file: " + fileName + " , contains more than " + (percent*100) +"%  of non valid value.", false);
     	 	return false;
    	 } 
	 return true;
}


	/**
     * Methode: Main pour tester la classe
     */
	
	public static void main(String[] args)  throws IOException{
		File f3 = new File("MGInfJPMed");
	//System.out.println(f3.getName());
	//System.out.println(FileLineCounter.countLines("./JPLow"));
	//System.out.println(isValidScenarioFile("./JPLow",2));
	//System.out.println(isValidScenarioFile("./JPLowFormat",3));
	
	  System.out.println("Debut: " + new Date().toString());

	//System.out.println(isValidScenarioFile("./JPLowFormatLight1",3));
	System.out.println(isHardwareConformScenario("JPLowFormat", 25.05,0.05));
	
   System.out.println("Fin: " + new Date().toString());
	}
	
}
