package utils;

/**
 * MyDynamicTree
 * 
 * @author mladoue
 */

import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;



public class MyDynamicTree extends JPanel  {

	protected DefaultMutableTreeNode rootNode;
    protected DefaultTreeModel treeModel;
    protected JTree tree;
    protected Toolkit toolkit = Toolkit.getDefaultToolkit();
    
    
    public MyDynamicTree(String rootTitle) {
    	
		this.setLayout(new BoxLayout(this,BoxLayout.Y_AXIS ));
    	
        rootNode = new DefaultMutableTreeNode(rootTitle);
        treeModel = new DefaultTreeModel( rootNode );
        treeModel.addTreeModelListener( new MyTreeModelListener() );

        tree = new JTree(treeModel);
        tree.setEditable(true);
        tree.getSelectionModel().setSelectionMode (TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.setShowsRootHandles(true);

		//		Enable tool tips.
	 	 ToolTipManager.sharedInstance().registerComponent(tree);

        JScrollPane scrollPane = new JScrollPane(tree);
		
		add(scrollPane);
    }

    /** 
     * Remove all nodes except the root node. 
     */
    public void clear() {
        rootNode.removeAllChildren();
        treeModel.reload();
    }

	
    /**
     * Remove the currently selected node.
     */
	public int removeCurrentNode() {
		DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();
		
		if ( selectedNode != null && selectedNode.getParent() != null) {
			treeModel.removeNodeFromParent(selectedNode);
			return 0;
		}
		else {
			toolkit.beep();
			return 1;
		 }
	}
	
	/**
	 * Returns true if the selectedNode is a parent(as defined below), false if it's a child 
	 * WARNING: this method has to be used with hasSelectedNode method!!!!
	 * 	 rootNode
	 * 	*--> first parent
	 * 	*	  *--> first child
	 * 	*	  *--> second child
	 * 	*--> second parent
	 * 
	 * @return a boolean
	 */
	public boolean isAFirstLevelParent() {
		DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();

		if ( selectedNode.getParent() == rootNode) {
			return true; // it's a parent
		}
		else { return false; }
	}
	
	public boolean isAFirstLevelChild() {
		DefaultMutableTreeNode selectedNodeParent = (DefaultMutableTreeNode)((DefaultMutableTreeNode) tree.getLastSelectedPathComponent()).getParent();

		if ( isAFirstLevelParent( selectedNodeParent) ) {
			return true; // it's a child
		}
		return false;
	}
	
	public boolean  isAFirstLevelParent( DefaultMutableTreeNode node ) {
		if ( node.getParent() == rootNode) { 
			return true; // it's a parent
		}
		return false;
	}
	
	public boolean hasSelectedNode() {
		DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode)tree.getLastSelectedPathComponent();
		if ( selectedNode != null ) {
			return true;
		}
		return false;	
	}
	
	public int removeCurrentNode( boolean parent ) {
			DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();
			if ( selectedNode != null ) { 
				if ( parent && isAFirstLevelParent(selectedNode)) {
					treeModel.removeNodeFromParent(selectedNode);
					return 0;
				}
				else if ( !parent && selectedNode.getParent() != null) {
					treeModel.removeNodeFromParent(selectedNode);
					return 0;
				}
				else { 
					toolkit.beep();
					return 1; //NOK!
				}
			}
			else {
				toolkit.beep();
				return 1;//NOK!
			 }
		}
	
	
    /** 
     * Add child to the currently selected node.
     */    
    public DefaultMutableTreeNode addObject(Object child) {
    	
        DefaultMutableTreeNode parentNode = null;
        TreePath parentPath = tree.getSelectionPath();

        if (parentPath == null) { parentNode = rootNode; }
         
        else { parentNode = (DefaultMutableTreeNode) (parentPath.getLastPathComponent());  }

        return addObject(parentNode, child, true);
    }

    public DefaultMutableTreeNode addObject( DefaultMutableTreeNode parent, Object child) {
        return addObject(parent, child, false);
    }

    public DefaultMutableTreeNode addObject(DefaultMutableTreeNode parent, Object child,  boolean shouldBeVisible) {
        DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(child);

        if (parent == null) {  parent = rootNode;  }

        treeModel.insertNodeInto(childNode, parent, parent.getChildCount());

        // Make sure the user can see the lovely new node.
        if (shouldBeVisible) {
            tree.scrollPathToVisible(new TreePath(childNode.getPath()));
        }
        return childNode;
    }

	public JTree getTree() { return tree; }

    protected class MyTreeModelListener implements TreeModelListener {
    	
        public void treeNodesChanged(TreeModelEvent e) {
            DefaultMutableTreeNode node;
            node = (DefaultMutableTreeNode) (e.getTreePath().getLastPathComponent());

            /*
             * If the event lists children, then the changed
             * node is the child of the node we've already
             * gotten.  Otherwise, the changed node and the
             * specified node are the same.
             */
            try {
                int index = e.getChildIndices()[0];
                node = (DefaultMutableTreeNode) (node.getChildAt(index));
            } 
            catch (NullPointerException exc) { exc.printStackTrace(); }

            System.out.println("The user has finished editing the node.");
            System.out.println("New value: " + node.getUserObject());
        }
        
        
        public void treeNodesInserted(TreeModelEvent e) {        }
        
        public void treeNodesRemoved(TreeModelEvent e) {        }
        
        public void treeStructureChanged(TreeModelEvent e) {        }
    }
}
