package utils;

import java.util.*;


/**
 * TestParameters
 * 
 * @author mladoue
 * @version 0.1
 */

public class TestParameters {


	public static boolean has2DIntTableSameColumnsNumber( int[][] values ) {
		 for (int i=0;i<values.length;i++) {
			if( values[0].length != values[i].length) {
				return false;
			}
		 }
		 return true;
	}
	

	/**
	 * Returns true if all values of this table(double) are the same, else false
	 * @param table of double
	 * @return a boolean
	 */
	public static boolean hasDoubleTableSameValues(double[] table) {
		for (int i=1;i<table.length;i++) {
			if( table[0] != table[i]) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Returns true if all values of this table(int) are the same, else false
	 * @param table of int
	 * @return a boolean
	 */
	public static boolean hasIntTableSameValues(int[] table) {
		for (int i=1;i<table.length;i++) {
			if( table[0] != table[i]) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Returns the number of value which are different in this table
	 * @param table it's an array of  int
	 * @return 0 if all the value are equals, 1 there is 2 different values etc
	 */
	public static int getIntNumbersInATable(int[] table) {
		int differentNumber = 1; // it means all numbers of this table are different : there is only 1 number
		SortedSet newT = new TreeSet();
		for(int i=0;i<table.length;i++) {
			newT.add( new Integer(table[i]) );
		}
		differentNumber = newT.size();
		return (differentNumber-1);
	}
	
	/**
	 * Returns the number of value which are different in this table
	 * @param table it's an array of  double
	 * @return 0 if all the value are equals, 1 there is 2 different values etc
	 */
	public static int getDoubleNumbersInATable(double[] table) {
			int differentNumber = 1; // it means all numbers of this table are different : there is only 1 number
			SortedSet newT = new TreeSet();
			for(int i=0;i<table.length;i++) {
				newT.add( new Double(table[i]) );
			}
			differentNumber = newT.size();
			return (differentNumber-1);
		}
		
		
	/**
	 * Method getMaximumIntValue : To determine in an int[] which is the greatest value
	 * @param data
	 * @return the greatest value in a double array
	 */
	public static int getMaximumIntValue(int[] data) {
		int ans = data[0];
		for (int i=1;i<data.length;i++) {
			ans = Math.max(ans,data[i]);
		}
		return ans;	
	}
	
	/**
	 * Method getMaximumDoubleValue : To determine in an double[] which is the greatest value
	 * @param data an array of double
	 * @return the greatest double in a table
	 */
	public static double getMaximumDoubleValue(double[] data) {
		double ans = data[0];
		for (int i=1;i<data.length;i++) {
			ans = Math.max(ans,data[i]);
		}
		return ans;	
	}
	
	/**
	* Method getMinimumIntValue : To determine in an int[] which is the smallest value
	* @param data
	* @return the smallest int in a table
	*/
		public static int getMinimumIntValue(int[] data) {
			int ans = data[0];
			for (int i=1;i<data.length;i++) {
				ans = Math.min(ans,data[i]);
			}
			return ans;	
		}
	
	/**
	 * Method getMinimumDoubleValue : To determine in an double[] which is the smallest value
	 * @param data an array of double
	 * @return the smallest value in a double array
	 */
	public static double getMinimumDoubleValue(double[] data) {
		double ans = data[0];
		for (int i=1;i<data.length;i++) {
			ans = Math.min(ans,data[i]);
		}
		return ans;	
	}
		
	/**
	 * Method testNormalizedParameter. Test if the parameter value is normalized or not
	 * @param parameter
	 * @return boolean
	 */
	public static boolean testNormalizedParameter(double[] parameter) {
		
		 for (int i=0;i<parameter.length;i++) {
		if ( parameter[i] < 0d ||  parameter[i] > 1d) {	
			return false;
		}
		 }
		return true;
	}

	public static boolean isAudioTest(String modelName) {
		
	if ( modelName.equals("OnOff")) { return true; }
 	else if ( modelName.equals("MMPP2")) {	return true; }
	 else if ( modelName.equals("MMPPN")) { return true;  }
 	else if ( modelName.equals("MGInf")) { 	return false; }
 	else if ( modelName.equals("MGInfComplex")) { return false; }
 	else if( modelName.equals("OnOffHeterogeneousAggregatedFlow")) { return true; }
	else if( modelName.equals("MGInfAggregatedFlow")) { return false; }
 	else { throw new IllegalArgumentException("The model: " + modelName + " doesn't exist!"); }
	}

 /**
     * Methode: Main pour tester la classe
     */
    
	public static void main(String[] args) {
		double[] test = {0.1,0.3,1} ;
		System.out.println( testNormalizedParameter( test) );
		
		int[] t = {1 , 2 ,2 , 3, 5 ,1};
		System.out.println(getIntNumbersInATable(t));
	}
}
