package utils.myPlotter;


import utils.*;
import ptolemy.plot.Plot;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

/**
 * OneGraphPlotter : This permit to plot data (from files, table...)
 * 					This class is based on ptplot5_2
 * @author mladoue
 * @version 0.1, 15/04/03
 */
public class OneGraphPlotter extends GenericPlotter {
	
	protected FileReader fichierReq;
	protected int fileLineNumber;
	protected Plot simplePlot;
	protected static int graphNb = 1;

	public OneGraphPlotter() {
		super(graphNb);
		// Instantiate the plot.
		simplePlot = new Plot();
	}
	
	/**
	 * OneGraphPlotter 
	 * @param XSize : it's the X size of window
	 * @param YSize : it's the Y size of window
	 * @param windowName : it's the name of the window
	 */
	public OneGraphPlotter(int XSize, int YSize, String windowName) {
			super(XSize, YSize, windowName, graphNb);
			// Instantiate the plot.
			simplePlot = new Plot();
	}
	
	
	/**
	 * Method plotAudioFileData
	 * @param fileName
	 * @param packetSize
	 * @throws FileNotFoundException
	 */
	public void plotAudioFileData( String fileName, int packetSize ) throws FileNotFoundException {
		
		int fileLineNumber = FileLineCounter.countLines(fileName);
		
		PlotterProperties.setPlotProperties(simplePlot,getWindowSize(true), getWindowSize(false), "Audio flow: Packet Distribution in Time", 0, 1100,0, fileLineNumber, "time (in ms)", "octets" );
		PlotterProperties.setYTick(simplePlot, packetSize);
		
		PlotterMethods.createPointsFromAudioFile(simplePlot, fileName, true, packetSize);
					
		// Layout the plot
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		this.getContentPane().setLayout(gridbag);

		// Handle the result Plot
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 1;
		gridbag.setConstraints(simplePlot, c);
		this.getContentPane().add(simplePlot);

		show();
	}
	
	
	public void plotVideoFileData( String fileName ) throws FileNotFoundException{
		
		int fileLineNumber = FileLineCounter.countLines(fileName);
		PlotterProperties.setPlotProperties(simplePlot,700, 600, "Video flow: Packet Distribution in Time", 0, 1100,0, fileLineNumber, "time (in ms)", "octets" );

		PlotterMethods.createPointsFromVideoFile(simplePlot, fileName, true);

		// Layout the plot
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		this.getContentPane().setLayout(gridbag);

		// Handle the result Plot
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 1;
		gridbag.setConstraints(simplePlot, c);
		getContentPane().add(simplePlot);

		show();
	}
	

	public void plotIntTableData(int[] data, int testDuration, String plotTitle,String XLabel, String YLabel, boolean connected, boolean impulses) {
		
//		PlotterProperties.setPlotProperties(simplePlot,700, 600, plotTitle, 0, TestParameters.getMaximumIntValue(data), 0, testDuration, XLabel, YLabel );
//			PlotterProperties.setYTick(simplePlot, TestParameters.getMaximumIntValue(data));
//			PlotterProperties.setXTick(simplePlot, testDuration);
//			simplePlot.setBars(true);
//			PlotterProperties.setSpecificPlotProperties( simplePlot, impulses, false, "none");
		
		PlotterProperties.setBasicPlotProperties(simplePlot, (int)Math.round(getWindowSize(true)*90/100), (int)Math.round(getWindowSize(false)*90/100),plotTitle, XLabel, YLabel);
		
		PlotterMethods.createPointsFromIntTable( simplePlot, data, testDuration, connected);
		
		// Layout the plot
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		this.getContentPane().setLayout(gridbag);

		// Handle the result Plot
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 1;
		gridbag.setConstraints(simplePlot, c);
		this.getContentPane().add(simplePlot);

		show();
	}
	
	
	public void plotDoubleTableData(double[] data, int testDuration, boolean connected, boolean impulses) {
		
		PlotterProperties.setYTick( simplePlot, TestParameters.getMaximumDoubleValue(data));
		PlotterProperties.setXTick( simplePlot, testDuration);
		PlotterProperties.setSpecificPlotProperties( simplePlot, impulses, false, "cross");
		
		PlotterMethods.createPointsFromDoubleTable(simplePlot, data, testDuration, connected);
		
		// Layout the plot
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		getContentPane().setLayout(gridbag);

		// Handle the result Plot
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 1;
		gridbag.setConstraints(simplePlot, c);
		this.getContentPane().add(simplePlot);

		show();
	}
	
		
	public Plot getSpecificPlot(int Number) { return simplePlot; }


	public static void main(String args[]) throws IOException {
		//OneGraphPlotter rp0 = new OneGraphPlotter("OnOff_30Format.scf",1000);
		//rp0.plotIntTableData();
	}
}

